



							KEY-X FUNCTION
							
							
;*****************************************************************************************
;
;      NAME:    KEY_x_FUNC
;
;   PURPOSE:    Run a subroutine based on which keypad button is pressed.
;
;     INPUT:    None.
;
;    OUTPUT:    None.
;
;*****************************************************************************************

;-----------------------------------------------------------------------------------------
;               Key 1 function.	        ( Stop/start relative power measurements )
;-----------------------------------------------------------------------------------------

KEY_1_FUNC:     BTFSC   FAST_WOBBLE      ;       Is the fast sweep function enabled ?
               _GOTO    KEYPAD_EXIT     ;       Y. Clear flags & return.
                                        ;
                MOVLW   6               ;       Power meter command
               _CALL    SET_TX_COMMAND  ; (4)   6 = Stop/start relative power measurements
                                        ;
                MOVLW   REL_MASK        ;       Change the state of the relative flag.
                XORWF   FLAG3,F         ;
                CLRW                    ;
                BTFSC   RELATIVE_FLAG   ;
                MOVLW   1               ;
               _CALL    TX_BYTE_END     ; (4)   Send the byte & finish the command.
                                        ;
               _CALL    RX_COMMAND_14   ; (4)   Power meter command 14 = Return the power.
                                        ;
               _GOTO    KEYPAD_EXIT     ;       Clear flags & return.

;-----------------------------------------------------------------------------------------
;               Key 2 function.	        ( Change the power display mode )
;-----------------------------------------------------------------------------------------

KEY_2_FUNC:     BTFSC   FAST_WOBBLE      ;       Is the fast sweep function enabled ?
               _GOTO    KEYPAD_EXIT     ;       Y. Clear flags & return.
                                        ;
                MOVLW   1               ;       Power meter command
               _CALL    SET_TX_COMMAND  ; (4)   1 = Set the power display mode.
                                        ;
                INCF    MODE,F          ;       Inc the display mode.
                MOVLW   1               ;
                BTFSC   MODE,2          ;
                MOVWF   MODE            ;
                MOVF    MODE,W          ;
               _CALL    TX_BYTE_END     ; (4)   Send the byte & finish the command.
                                        ;
                                        ;       Power meter command
               _CALL    TX_COMMAND_7    ; (4)   7 = Saves changes made to the memory.
                MOVLW   0               ;       0 = Save changes to the setup
               _GOTO    FINISH_7        ;           made by commands 1, 2 or 3.

;-----------------------------------------------------------------------------------------
;               Key 3 function.	        ( Select power meter memory )
;-----------------------------------------------------------------------------------------

KEY_3_FUNC:     BTFSS   SWEEP_ACTIVE    ;       Is the sweep function enabled or
                BTFSC   NO_PWR_METER    ;       is the power meter not fitted ?
               _GOTO    KEYPAD_EXIT     ;       Y. Clear flags & return.
                                        ;
                MOVLW   MEM_MASK        ;       Change the state of the mem select flag.
                XORWF   FLAG3,F         ;
                                        ;
                BTFSC   EN_MEM_SEL      ;       Was power meter memory selection disabled.
               _GOTO    KEYPAD_EXIT     ;       N. Clear flags & return.
                                        ;
                                        ;       Y. Power meter command
               _CALL    TX_COMMAND_7    ; (4)      7 = Saves changes made to the memory.
                MOVLW   1               ;          1 = Save changes to the setup
                                        ;              made by commands 0, 4 or 5.
FINISH_7:      _CALL    TX_BYTE_END     ; (4)      Send the byte & finish the command.
                                        ;
                                        ;          Wait until the power meter has updated
              P_CALL    WAIT_100MS      ; (2)      its EEPROM.
                                        ;
               _CALL    RX_COMMAND_14   ; (4)      PWR meter command 14 = Return the power.
;               _GOTO    KEYPAD_EXIT     ;          Clear flags & return.


;-----------------------------------------------------------------------------------------
;               Key 4 function.         ( Change the wobble direction )
;-----------------------------------------------------------------------------------------

KEY_4_FUNC:		BTFSC	SWEEP
			   _GOTO    KEYPAD_EXIT     ;
				MOVLW   SWEEP_DIR_MASK  ;
                XORWF   SWEEP_FLAGS,F   ;       Change the wobble direction.
			    BCF     SWEEP_STOPPED   ;
               _GOTO    KEYPAD_EXIT     ;

;-----------------------------------------------------------------------------------------
;               Key 5 function.         ( Start / Halt / Resume slow wobble )
;-----------------------------------------------------------------------------------------

KEY_5_FUNC:		BSF		WOBBLE			;		Wobble flag
				BCF		SWEEP			;		Retrace Sweep flag
				BTFSC   EN_MEM_SEL      ;       Is the mem select function active.
               _GOTO    KEYPAD_EXIT     ;       Y. Dont start sweeping.
               _CALL	DISP_MODE       ;
                BTFSS   SWEEP_ACTIVE    ;       Is sweep function enabled ?
               _GOTO    START_STEPPING  ;       N. Start stepping.

HALT_RESUME:    MOVLW   SWEEP_STP_MASK  ;       Y. Halt / Resume wobbling.
                XORWF   SWEEP_FLAGS,F   ;       Change the state of the sweep stopped flag.
	       	   _GOTO    KEYPAD_EXIT     ;

;-----------------------------------------------------------------------------------------
;               Key 6 function.         ( Start fast wobbling )
;-----------------------------------------------------------------------------------------

KEY_6_FUNC:		BSF		WOBBLE			;		Wobble mode
				BTFSS   EN_MEM_SEL      ;       Is the mem select function active or
                BTFSC   SWEEP_ACTIVE    ;       is the sweep function enabled ?
               _GOTO    KEYPAD_EXIT     ;       Y. Clear flags & return.
                BSF     FAST_WOBBLE     ;       Set the fast wobble mode.
			   _CALL	DISP_MODE       ;
			   
START_STEPPING:
			   _CALL    FREQ_TO_EE      ; (3)   Copy DDS freq's and step size to EEPROM.
                BSF     SWEEP_ACTIVE    ;       Turn on the sweep function.
                BCF     SWEEP_STOPPED   ;
                
;               Set the limits to S/F.
               _CALL    PUSH_ARG2       ; (1)   Dont destroy current sweep freq.
                MOVLW   S_DDS_FREQ      ;       Get the start frequency.
               _CALL    COPY_TO_ARG2    ; (1)
                MOVLW   F_DDS_FREQ      ;
               _CALL    RAM_TO_ARG3     ; (2)   Get the finish frequency.
                BSF     TEMP_FLAG       ;
               _CALL    COMPARE         ; (2)   Is it above the current freq ?
                BTFSC   CARRY           ;
                BCF     TEMP_FLAG       ;       N. Set wobble direction to down.
                
;               Set the lower sweep limit.
                BTFSC   TEMP_FLAG       ;       wobble up ?
               _CALL    ARG2_TO_ARG4    ; (1)   Y. The lower limit is the start freq.
               
                BTFSS   TEMP_FLAG       ;
               _CALL    ARG3_TO_ARG4    ; (1)   N. The lower limit is the finish freq.
                                        
;               Set the upper SWEEP limit.
                BTFSS   TEMP_FLAG       ;       Sweep up ?
               _CALL    ARG2_TO_ARG5    ; (1)   Y. The upper limit is the start freq.
                BTFSC   TEMP_FLAG       ;
               _CALL    ARG3_TO_ARG5    ; (1)   N. The upper limit is the finish freq.
               _CALL    PULL_ARG2       ; (1)   Recover current SWEEP freq.
                BCF     SWEEP_DIR       ;
                BTFSC   TEMP_FLAG       ;
                BSF     SWEEP_DIR       ;       Set sweep direction.
               _CALL    SETUP_SWEEP     ; (2)   Set the sweep variables.
                MOVF    SWEEP_RATE,W    ;
                MOVWF   SWEEP_COUNT     ;       Reset the SWEEP counter.
                BCF     SWEEP_CLOCK     ;
               _GOTO    KEYPAD_EXIT     ;
               
;-----------------------------------------------------------------------------------------
;               Key 7 function          ( Single Sweep )
;-----------------------------------------------------------------------------------------

KEY_7_FUNC:		BTFSS   SWEEP_STOPPED   ;		Is the sweep halted?
		       _GOTO    KEYPAD_EXIT     ;		N.
		       	BSF		SWEEP			;		Y. Enable sweep mode
  				BSF		SINGLE_SWEEP	;		   Enable single sweep mode
				BSF		EN_TRIGGER		;		   Enable the trigger pulse
				
;-----------------------------------------------------------------------------------------
;               Key 8 function.         ( Start Slow sweep / Halt / restart sweep )
;-----------------------------------------------------------------------------------------

KEY_8_FUNC:		BSF		SWEEP			;	    Set the Sweep flag
				BCF		WOBBLE			;	    Clear the Wobble flag
			  _CALL		DISP_MODE     	;
				BTFSC   EN_MEM_SEL      ;       Is the mem select function active.
               _GOTO    KEYPAD_EXIT     ;       Y. Dont start sweeping.

STOP_RESUME:    MOVLW   SWEEP_STP_MASK  ;       Y. Halt / Resume wobbling.
                XORWF   SWEEP_FLAGS,F   ;       Change the state of the sweep stopped flag.
				BTFSC   SWEEP_ACTIVE    ;       Is sweep function enabled ?
	           _GOTO    KEYPAD_EXIT     ;		N.
		

                BTFSS   SWEEP_ACTIVE    ;       Is the sweep function enabled ?
	           	BSF		EN_TRIGGER		;		Y.  Enable the trigger pulse
                BTFSS   SWEEP_ACTIVE    ;       Is sweep function enabled ?
               _GOTO    START_STEPPING  ;       Y. Start sweeping.

;-----------------------------------------------------------------------------------------
;               Key 9 function.         ( Fast sweep )
;-----------------------------------------------------------------------------------------               
               
KEY_9_FUNC:	
				BSF		SWEEP			;		Set the Sweep flag
				BTFSS   EN_MEM_SEL      ;       Is the mem select function active or
                BTFSC   SWEEP_ACTIVE    ;       is the sweep function enabled ?
               _GOTO    KEYPAD_EXIT     ;       Y. Clear flags & return.
                BSF     FAST_WOBBLE     ;       Set the fast wobble mode.
				BSF     FAST_SWEEP      ;       Set the fast sweep flag.
			   _CALL	DISP_MODE       ;
	           _GOTO    START_STEPPING  ;       Start sweeping.

;-----------------------------------------------------------------------------------------
;               Key # function.         ( Exit sweep mode )
;                                       ( quick press = return to VFO mode )
;                                       ( long press  = save sweep freq to active VFO )
;-----------------------------------------------------------------------------------------

KEY_#_FUNC:     BTFSS   MEM_ACTIVE      ;       Is the mem select function active or
                BTFSS   SWEEP_ACTIVE    ;       is the sweep function disabled ?
               _GOTO    KEYPAD_EXIT     ;       Y. Just exit.
                                        ;
               _CALL    WRITE_SWEEP     ; (2)   Save changes to the sweep rate to EEPROM.
                                        ;
                BTFSC   FAST_WOBBLE      ;       Is the fast sweep mode enabled ?
               _GOTO    FAST_SWP_EXIT   ;       Y. Just return to VFO mode.
                                        ;
KEY_#_LOOP:     BTFSC   KEYPAD_CNT,7    ;       Has the button been pressed for 1 sec ?
               _GOTO    TIME_OUT_#      ;
                MOVF    KEYPAD_CNT,W    ;
                BTFSS   ZERO            ;
               _GOTO    KEY_#_LOOP      ;
FAST_SWP_EXIT: _CALL    EE_TO_RAM       ; (3)   N. Get setup & cal info from EEPROM.
               _GOTO    KEY_EXIT        ;
                                        ;
TIME_OUT_#:     BSF     EEPROM_UPDATE   ;       Y. Set EEPROM update flag. (VFO = sweep freq)
                                        ;       Calculate the lower limit.
KEY_EXIT:      _CALL    CLEAR_ARG4      ; (1)   Save 0 Hz as the lower limit. (ARG4)
                                        ;
                MOVLW   MAX_DDS_FREQ    ;       Calculate the upper limit.
               _CALL    RAM_TO_ARG2     ; (2)   Get the upper DDS freq.
               _CALL    ARG2_TO_ARG5    ; (1)   Save it as the upper limit. (ARG5)
                CLRF    SWEEP_FLAGS     ;       Stop the sweep functions.
				BCF		WOBBLE
				BCF		SWEEP
               _GOTO    KEYPAD_EXIT     ;

KEY_._FUNC:    _GOTO    KEYPAD_EXIT     ;      Do nothing. Clear flags & return.	
