

;				SWEEPGEN v2.00 ENHANCEMENTS		Colin Purton VK7ZCP

;				KEY_4	change wobble direction 
;				KEY_5 	start / halt / resume slow wobble  
;				KEY-6 	fast wobble
;				KEY-7 	Single sweep 
;				KEY-8 	start / halt / restart sweep
;				KEY-9 	fast sweep
;				Trigger pulse on PortA,6
;				Mode Display on LCD line 1				

;	File Name:	SWEEPGEN+.asm

;*****************************************************************************************

#DEFINE         VER_TEXT        "SweepGen+ Beta_0"         ; Software version text.

;*****************************************************************************************

 LIST N=0, R=DEC, F=INHX8M
;*****************************************************************************************
;               Processor Definitions
;*****************************************************************************************

 PROCESSOR      16F648A
 __CONFIG       _WDT_OFF & _LVP_OFF & _INTRC_OSC_NOCLKOUT & _BODEN_ON & _PWRTE_ON

_BODEN_ON               EQU             H'3FFF'
_PWRTE_ON               EQU             H'3FF7'
_WDT_OFF                EQU             H'3FFB'
_LVP_OFF                EQU             H'3F7F'
_XT_OSC                 EQU             H'3FED'
_INTRC_OSC_NOCLKOUT     EQU             H'3FFC'

 __MAXRAM       H'01FF'
 __BADRAM       H'07'-H'09',H'0D',H'13'-H'14',H'1E'
 __BADRAM       H'87'-H'89', H'8D', H'8F'-H'91', H'93'-H'97', H'9E'
 __BADRAM       H'105', H'107'-H'109', H'10C'-H'11F'
 __BADRAM       H'185', H'187'-H'189', H'18C'-H'1EF'

;*****************************************************************************************
;               Register Definitions
;*****************************************************************************************

W               EQU             H'00'
F               EQU             H'01'
INDF            EQU             H'00'
TMR0            EQU             H'01'
PCL             EQU             H'02'
STATUS          EQU             H'03'
FSR             EQU             H'04'
PORTA           EQU             H'05'
PORTB           EQU             H'06'
PCLATH          EQU             H'0A'
INTCON          EQU             H'0B'
PIR1            EQU             H'0C'
CMCON           EQU             H'1F'

;*****************************************************************************************
;               Definitions that eliminate error 302
;*****************************************************************************************

OPTION_REG      EQU             H'81' & H'7F'
TRISA           EQU             H'85' & H'7F'
TRISB           EQU             H'86' & H'7F'
PIE1            EQU             H'8C' & H'7F'
EEDATA          EQU             H'9A' & H'7F'
EEADR           EQU             H'9B' & H'7F'
EECON1          EQU             H'9C' & H'7F'
EECON2          EQU             H'9D' & H'7F'
VRCON           EQU             H'9F' & H'7F'

;*****************************************************************************************
;               Definitions of commonly used register bits
;*****************************************************************************************

#DEFINE         CARRY           STATUS,0
#DEFINE         ZERO            STATUS,2
#DEFINE         RP0             STATUS,5
#DEFINE         RP1             STATUS,6
#DEFINE         IRP             STATUS,7
#DEFINE         T0IF            INTCON,2
#DEFINE         RD              EECON1,0
#DEFINE         WR              EECON1,1
#DEFINE         WREN            EECON1,2
#DEFINE         WRERR           EECON1,3
#DEFINE         EEIF            PIR1,7
#DEFINE         CIS             CMCON,3
#DEFINE         GIE             INTCON,7

;*****************************************************************************************
;               General equates
;*****************************************************************************************

TRUE            EQU             H'FF'
FALSE           EQU             H'00'
                                                ; Assembly options.
AD9851_CHIP     EQU             TRUE            ; TRUE for AD9851.
;AD9851_CHIP     EQU            FALSE            ; FALSE for AD9850.

SIMULATE        EQU            FALSE            ; FALSE if not using the MPLAB simulator.
;SIMULATE        EQU             TRUE            ; TRUE if using the MPLAB simulator.

RESETVECTOR     EQU           H'00'             ; PIC Reset vector.
INTVECTOR       EQU           H'04'             ; PIC Interrupt vector.

PAGE0           EQU         H'0000'             ; Code memory page 0. 0000-07FF
PAGE1           EQU         H'0800'             ; Code memory page 1. 0800-0FFF
                                                ;
VOLTS           EQU             14              ; Ref V = 3.44 V. 1.25+(14 x 0.15625)
TIMEOUT_NUM     EQU              5              ; Display timeout. 10 sec (5 x 2.1 sec)

#DEFINE         lo              H'7F' &         ; Used on opcodes that use registers/ram
                                                ; in bank 1, 2 & 3. Stops 302 message.

;-----------------------------------------------------------------------------------------
;               Button and keypad timing equates.
;-----------------------------------------------------------------------------------------

MIN_ON          EQU              6              ; Min valid button pressed time. (6 x 8mS)
MAX_ON          EQU             70              ; Max valid button pressed time.(70 x 8mS)

;-----------------------------------------------------------------------------------------
;   Wobble/Sweep function equates.         		( Step size  = 1 to 64000 Hz          )
;                                               ( Rate 		 = 20 to 255,  x 4.096 mS )
;-----------------------------------------------------------------------------------------

STEP_0 EQU                 5              ; Step size (Hz) for sweep.
RATE_0 EQU               122              ; Step rate for sweep. (5 Hz per 1/2 sec)
                                                ; (10 Hz per sec)

STEP_1 EQU                15              ; Step size (Hz) for sweep.
RATE_1 EQU               122              ; Step rate for sweep. (15 Hz per 1/2 sec)
                                                ; (30 Hz per sec)

STEP_2 EQU                25              ; Step size (Hz) for sweep.
RATE_2 EQU                61              ; Step rate for sweep. (25 Hz per 1/4 sec)
                                                ; (100 Hz per sec)

STEP_3 EQU                75              ; Step size (Hz) for sweep.
RATE_3 EQU                61              ; Step rate for sweep. (75 Hz per 1/4 sec)
                                                ; (300 Hz per sec)

STEP_4 EQU               125              ; Step size (Hz) for sweep.
RATE_4 EQU                30              ; Step rate for sweep. (125 Hz per 1/8 sec)
                                                ; (1 KHz per sec)

STEP_5 EQU               375              ; Step size (Hz) for sweep.
RATE_5 EQU                30              ; Step rate for sweep. (375 Hz per 1/8 sec)
                                                ; (3 KHz per sec)

STEP_6 EQU              1250              ; Step size (Hz) for sweep.
RATE_6 EQU                30              ; Step rate for sweep. (1.25 KHz per 1/8 sec)
                                                ; (10 KHz per sec)

STEP_7 EQU              3750              ; Step size (Hz) for sweep.
RATE_7 EQU                30              ; Step rate for sweep. (3.75 KHz per 1/8 sec)
                                                ; (30 KHz per sec)

STEP_8 EQU              6250              ; Step size (Hz) for sweep.
RATE_8 EQU                30              ; Step rate for sweep. (12.5 KHz per 1/8 sec)
                                                ; (50 KHz per sec)

STEP_9 EQU              8200              ; Step size (Hz) for sweep.
RATE_9 EQU                30              ; Step rate for sweep. (37.5 KHz per 1/8 sec)
                                                ; (67 KHz per sec)

;-----------------------------------------------------------------------------------------
;               Power meter and I2C.
;-----------------------------------------------------------------------------------------

I2C_WR_ADDR  EQU                B'01000010'     ; I2C slave write address.
I2C_RD_ADDR  EQU                B'01000011'     ; I2C slave read address.

DELAY_CONSTANT  EQU              2              ; I2C delay constant, for I2C clock rate.
METER_TIME      EQU             20              ; Power meter update time.  (40 x 8mS)

;*****************************************************************************************
;               General definitions
;*****************************************************************************************

;-----------------------------------------------------------------------------------------
;               PORT A pin definitions.
;-----------------------------------------------------------------------------------------

#DEFINE         RA0             PORTA,0         ; Pin 17, Analog I/P. Keyboard input from
#DEFINE         RA1             PORTA,1         ; Pin 18, Analog I/P. analog comparator.
#DEFINE         RA2             PORTA,2         ; Pin 1,  Analog I/P.         "
#DEFINE         RA3             PORTA,3         ; Pin 2,  Analog I/P.         "
#DEFINE         I2C_SDA         PORTA,4         ; Pin 3,  I/P. I2C data.
#DEFINE         RA5             PORTA,5         ; Pin 4,  I/P. MCLR, Vpp.
#DEFINE         RA6             PORTA,6         ; Pin 15, 0/P. Unused pic pin.
#DEFINE         RA7             PORTA,7         ; Pin 16, 0/P. Unused pic pin.

PORTA_DIR       EQU             B'00111111'     ; Initial pin directions. 0 = O/P, 1 = I/P.
PORTA_INIT      EQU             B'00000000'     ; Initial pin level.      0 = Low, 1 = High.

#DEFINE         I2C_SDA_DIR     TRISA,4         ; 0 = O/P, 1 = I/P.

;-----------------------------------------------------------------------------------------
;               PORT B pin definitions.
;-----------------------------------------------------------------------------------------

#DEFINE         DDS_LOAD        PORTB,0         ; Pin 6,  O/P. AD9850/51 update pin.
#DEFINE         LCD_RS          PORTB,1         ; Pin 7,  O/P. LCD Reg sel. 0 = inst 1 = data
#DEFINE         SW_COL_1        PORTB,1         ;   "      "   Column 1 enable, active low.
#DEFINE         I2C_SCL         PORTB,2         ; Pin 8,  I/P. I2C clock.
#DEFINE         LCD_E           PORTB,3         ; Pin 9,  O/P. LCD Enable. Latch on -edge.
#DEFINE         SW_COL_2        PORTB,4         ; Pin 10, O/P. Column 2 enable, active low.
#DEFINE         LCD_RB4         PORTB,4         ;   "      "   LCD data bit 4.
#DEFINE         DDS_CLOCK       PORTB,5         ; Pin 11, O/P. AD9850/51 write clock pin.
#DEFINE         SW_COL_3        PORTB,5         ;   "      "   Column 3 enable, active low.
#DEFINE         LCD_RB5         PORTB,5         ;   "      "   LCD data bit 5.
#DEFINE         SW_COL_4        PORTB,6         ; Pin 12, O/P. Column 4 enable, active low.
#DEFINE         LCD_RB6         PORTB,6         ;   "      "   LCD data bit 6.
#DEFINE         DDS_DATA        PORTB,7         ; Pin 13, O/P. AD9850/51 serial data input pin.
#DEFINE         SW_COL_5        PORTB,7         ;   "      "   Column 5 enable, active low.
#DEFINE         LCD_RB7         PORTB,7         ;   "      "   LCD data bit 7.

PORTB_DIR       EQU             B'00000100'     ; Initial pin directions. 0 = O/P, 1 = I/P.
PORTB_INIT      EQU             B'11111010'     ; Initial pin level.      0 = Low, 1 = High.

#DEFINE         I2C_SCL_DIR     TRISB,2         ; 0 = O/P, 1 = I/P.

;-----------------------------------------------------------------------------------------

#DEFINE         ROW_1           3               ; Bit position for row data in column.
#DEFINE         ROW_2           2               ;               "
#DEFINE         ROW_3           6               ;               "
#DEFINE         ROW_4           7               ;               "

#DEFINE         KP_1            COL_3,ROW_1     ; 1   3x4 or 4x4 Keypad.
#DEFINE         KP_2            COL_4,ROW_1     ; 2   (bit set if button is pressed)
#DEFINE         KP_3            COL_5,ROW_1     ; 3             "
#DEFINE         KP_4            COL_3,ROW_2     ; 4             "
#DEFINE         KP_5            COL_4,ROW_2     ; 5             "
#DEFINE         KP_6            COL_5,ROW_2     ; 6             "
#DEFINE         KP_7            COL_3,ROW_3     ; 7             "
#DEFINE         KP_8            COL_4,ROW_3     ; 8             "
#DEFINE         KP_9            COL_5,ROW_3     ; 9             "
#DEFINE         KP_.            COL_3,ROW_4     ; *             "
#DEFINE         KP_0            COL_4,ROW_4     ; 0             "
#DEFINE         KP_#            COL_5,ROW_4     ; #             "

#DEFINE         SW_STEP_SIZE    COL_1,ROW_1     ; A  4x4 Keypad or Ext button array.
#DEFINE         SW_CAL          COL_1,ROW_2     ; B  (bit set if button is pressed)
#DEFINE         SW_MEM          COL_1,ROW_3     ; C             "
#DEFINE         SW_SF           COL_1,ROW_4     ; D             "

#DEFINE         SW_xxx          COL_2,ROW_1     ; Ext button array, Unused.
#DEFINE         SW_yyy          COL_2,ROW_2     ; Ext button array, Unused.
#DEFINE         ENCODER_A       COL_2,ROW_3     ; Rotary encoder pin A.
#DEFINE         ENCODER_B       COL_2,ROW_4     ; Rotary encoder pin B.


#DEFINE         ENC_A_DEBOUNCED FLAG1,0         ; Encoder A state after debouncing.
#DEFINE         ENC_B_DEBOUNCED FLAG1,1         ; Encoder B state after debouncing.
#DEFINE         OLD_A_DEBOUNCED FLAG1,2         ; Old encoder a state after debouncing.
#DEFINE         TIMEOUT         FLAG1,3         ; Set if keypad or display timed out.
#DEFINE         EEPROM_UPDATE   FLAG1,4         ; Set if the EEPROM needs updating.
#DEFINE         WRITE_FLAG      FLAG1,5         ; Set if we want to do an EEPROM write.
#DEFINE         SAVE_FLAG       FLAG1,6         ; Set by SAVE if MEM button still pressed.
#DEFINE         TEMP_FLAG       FLAG1,7         ; Temporary flag bit.


#DEFINE         ANY_BUTTON      FLAG2,0         ; Set if any button is being pressed.
#DEFINE         PRESSED         FLAG2,1         ; Result flag from button pressed call.
#DEFINE         KEYPAD_PRESSED  FLAG2,2         ; Set if a keypad button is being pressed.
#DEFINE         MEM_1_SEC       FLAG2,3         ; Set if MEM button has been pressed for 1s.
#DEFINE         MEM_2_SEC       FLAG2,4         ; Set if MEM button has been pressed for 2s.
#DEFINE         WOBBLE      	FLAG2,5   		; Set = WOBBLE.
#DEFINE         SWEEP      		FLAG2,6   		; Set = SWEEP.
#DEFINE         EN_TRIGGER      FLAG2,7   		; Set = Trigger pulse enabled.


#DEFINE         NO_PWR_METER    FLAG3,0         ; Set if no power meter is fitted.
#DEFINE         RELATIVE_FLAG   FLAG3,1         ; Set if power meter mode is dBr.
#DEFINE         RX_I2C_OK       FLAG3,2         ; Set if the byte from the power meter was ok.
#DEFINE         EN_MEM_SEL      FLAG3,3         ; Set if enable power meter memory selection.
#DEFINE         VER_FLAG        FLAG3,4         ; Set if receiving power meter version text.
#DEFINE         I2C_ACK_FLAG    FLAG3,5         ; Set if the power meter acknowledged the byte.
#DEFINE         METER_FLAG      FLAG3,6         ; Set if time to update the power meter display.
#DEFINE         I2C_TX_ACK_FLAG FLAG3,7         ; If set the byte to the power meter will be
                                                ; followed by an acknowledge bit.

REL_MASK        EQU             B'00000010'     ; Mask for the power meter relative dB bit.
MEM_MASK        EQU             B'00001000'     ; Mask for power meter memory selection bit.


#DEFINE         LEADING_FLAG    DISP_FLAGS,0    ; Leading zero indicator.
#DEFINE         NEGATIVE        DISP_FLAGS,1    ; Set if number to display is negative.
#DEFINE         FULL_BLANKING   DISP_FLAGS,2    ; If set, use full leading 0 blanking.
#DEFINE         UNITS           DISP_FLAGS,3    ; If set, allow a larger step size range.
#DEFINE         SF_DISPLAYED    DISP_FLAGS,4    ; Set if finish frequency is displayed on LCD.
#DEFINE         MEM_ACTIVE      DISP_FLAGS,5    ; Set if the MEM function is active.
#DEFINE         CAL_FLAG        DISP_FLAGS,6    ; Set if in the cal screens.
#DEFINE         PWR_DIS_FLAG    DISP_FLAGS,7    ; Set if displaying the power.

SF_MASK         EQU             B'00010000'     ; Mask for S/F bit.


#DEFINE         SWEEP_CLOCK     SWEEP_FLAGS,0   ; Set if time to step the sweep freq.
#DEFINE         SWEEP_ACTIVE    SWEEP_FLAGS,1   ; Set if sweep is active.
#DEFINE         FAST_SWEEP      SWEEP_FLAGS,2   ; Set if fast wobble is active.
#DEFINE         SWEEP_STOPPED   SWEEP_FLAGS,3   ; Set if sweep has been stopped.
#DEFINE         SWEEP_DIR       SWEEP_FLAGS,4   ; Set if sweeping up.
#DEFINE         SWEEP_CHANGE    SWEEP_FLAGS,5   ; Set if sweep range has been exceeded.
#DEFINE         SINGLE_SWEEP    SWEEP_FLAGS,6   ; Set if in single sweep mode.
#DEFINE         FAST_WOBBLE     SWEEP_FLAGS,7   ; Set if fast retrace sweep is active.


SWEEP_STP_MASK  EQU             B'00001000'     ;
SWEEP_DIR_MASK  EQU             B'00010000'     ;


#DEFINE         KEYPAD_FLAG     KEYPAD_FLAGS,0  ; Set if the keypad is in use.
#DEFINE         KEYPAD_MAIN     KEYPAD_FLAGS,1  ; Set if keypad has to update the EEPROM.
#DEFINE         KEYPAD_DP       KEYPAD_FLAGS,2  ; Set if digits will be entered after DP.

#DEFINE         NOT_LTC5508_PWR ID_BYTE,0       ; Clear = LTC5508 power meter.

#DEFINE         X6_REF          0               ; Bit position of x6 REF CLK in DDS_OPT.

#DEFINE         ARG1            ARG1_7          ; Simpler definition for MSD of ARG's.
#DEFINE         ARG2            ARG2_3          ;               "
#DEFINE         ARG3            ARG3_3          ;               "
#DEFINE         ARG4            ARG4_3          ;               "
#DEFINE         ARG5            ARG5_3          ;               "

;*****************************************************************************************
;               MACRO'S
;*****************************************************************************************

        noexpand

;-----------------------------------------------------------------------------------------
;
;       S_PCL3          Help macro for P_CALL and P_GOTO.
;                       Set's/clears PCLATH bit 3 according to 'variable' PCLATH_3.
;
;-----------------------------------------------------------------------------------------

S_PCL3          MACRO   PCLATH_3
 IF (PCLATH_3 & 0x08)
        expand
                BSF     PCLATH,3
        noexpand
OFFSET_ADDRESS = 0 - H'800'
 ELSE
        expand
                BCF     PCLATH,3
        noexpand
OFFSET_ADDRESS = H'800'
 ENDIF
                ENDM

;-----------------------------------------------------------------------------------------
;
;       P_CALL          PAGE CALL, code for calling between page0<->1.
;                       Reset's PCLATH after call.
;                       W-reg is left unchanged.
;
;-----------------------------------------------------------------------------------------

P_CALL          MACRO   LABEL        
 LOCAL          DEST_HIGH, SOURCE_HIGH, DIFF_HIGH
DEST_HIGH       SET     (HIGH(LABEL) & 0x18)            ; Save bit's 4-5 of destination address.
SOURCE_HIGH     SET     (HIGH($) & 0x18)                ; Save bit's 4-5 of source address.
DIFF_HIGH       SET     DEST_HIGH ^ SOURCE_HIGH         ; Get difference. ( XOR )
 ERRORLEVEL     -306                                    ; Suppress page warning.

 IF (DIFF_HIGH == 0)                                    ; Same page, SHOULD generate no extra code.
 MESSG Call on same page, replace P_CALL with _CALL, LABEL
        expand
                NOP                                     ; Redundant NOP.
                CALL    LABEL                           ; Assemble the CALL.
                NOP                                     ; Redundant NOP.
        noexpand
 ELSE        
                                                        ; Difference in lowest bit only.
                S_PCL3  DEST_HIGH                       ; Set low bit of PCLATH.
        expand
                CALL    LABEL + OFFSET_ADDRESS
        noexpand
                S_PCL3  SOURCE_HIGH                     ; Reset low bit of PCLATH.
 ENDIF
 ERRORLEVEL     +306                                    ; Re-enable warning to catch real errors.
                ENDM

;-----------------------------------------------------------------------------------------
;
;         _CALL         Standard CALL, code for calling on same page.
;                       Outputs messages if page call must be used.
;
;-----------------------------------------------------------------------------------------

_CALL           MACRO   LABEL
 LOCAL          DEST_HIGH, SOURCE_HIGH, DIFF_HIGH
DEST_HIGH       SET     (HIGH(LABEL) & 0x18)            ; Save bit's 4-5 of destination address.
SOURCE_HIGH     SET     (HIGH($) & 0x18)                ; Save bit's 4-5 of source address.
DIFF_HIGH       SET     DEST_HIGH ^ SOURCE_HIGH         ; Get difference. ( XOR )
 ERRORLEVEL     -306                                    ; Suppress page warning.

 IF (DIFF_HIGH == 0)                                    ; Same page, CALL ok.
        expand
                CALL    LABEL
        noexpand
 ELSE        
                                                        ; Check if S_CALL is required.
  ERROR Replace _CALL with P_CALL, LABEL
        expand
                CALL    LABEL                           ; INCORRECT CALL !!!
        noexpand
 ENDIF
 ERRORLEVEL     +306                                    ; Re-enable warning to catch real errors.
                ENDM

;-----------------------------------------------------------------------------------------
;
;       P_GOTO          PAGE GOTO, code for jumping between page0<->1.
;                       W-reg is left unchanged.
;
;-----------------------------------------------------------------------------------------

P_GOTO          MACRO   LABEL        
 LOCAL          DEST_HIGH, SOURCE_HIGH, DIFF_HIGH
DEST_HIGH       SET     (HIGH(LABEL) & 0x18)            ; Save bit's 4-5 of destination address.
SOURCE_HIGH     SET     (HIGH($) & 0x18)                ; Save bit's 4-5 of source address.
DIFF_HIGH       SET     DEST_HIGH ^ SOURCE_HIGH         ; Get difference. ( XOR )
 ERRORLEVEL     -306                                    ; Suppress page warning.

 IF (DIFF_HIGH == 0)                                    ; Same page, SHOULD generate no extra code.
 MESSG Call on same page, replace P_GOTO with _GOTO, LABEL
        expand
                NOP                                     ; Redundant NOP.
                GOTO    LABEL                           ; Assemble the GOTO.
        noexpand
 ELSE        
                                                        ; Difference in lowest bit only.
                S_PCL3  DEST_HIGH                       ; Set low bit of PCLATH.
        expand
                GOTO    LABEL + OFFSET_ADDRESS
        noexpand
 ENDIF
 ERRORLEVEL     +306                                    ; Re-enable warning to catch real errors.
                ENDM

;-----------------------------------------------------------------------------------------
;
;         _GOTO         Standard GOTO, code for jumping on same page.
;                       Outputs messages if page GOTO must be used.
;
;-----------------------------------------------------------------------------------------

_GOTO           MACRO   LABEL
 LOCAL          DEST_HIGH, SOURCE_HIGH, DIFF_HIGH
DEST_HIGH       SET     (HIGH(LABEL) & 0x18)            ; Save bit's 4-5 of destination address.
SOURCE_HIGH     SET     (HIGH($) & 0x18)                ; Save bit's 4-5 of source address.
DIFF_HIGH       SET     DEST_HIGH ^ SOURCE_HIGH         ; Get difference. ( XOR )
 ERRORLEVEL     -306                                    ; Suppress page warning.

 IF (DIFF_HIGH == 0)                                    ; Same page, GOTO ok.
        expand
                GOTO    LABEL
        noexpand
 ELSE        
                                                        ; Check if S_GOTO is required.
  ERROR Replace _GOTO with P_GOTO, LABEL
        expand
                GOTO    LABEL                           ; INCORRECT GOTO !!!
        noexpand
 ENDIF
 ERRORLEVEL     +306                                    ; Re-enable warning to catch real errors.
                ENDM

;-----------------------------------------------------------------------------------------

#DEFINE         LABEL_OFFSET    ((($-TABLE_START)& H'FF')+($ & H'FF00'))& H'07FF'   

TEXT_STRING     MACRO   NAME,TEXT,TERM  ;       Text string macro.
NAME            EQU     LABEL_OFFSET    ;       Set the label offset.
        expand
                DT TEXT, TERM
        noexpand
                ENDM

;-----------------------------------------------------------------------------------------

_MOVLW          MACRO VAL               ;       _MOVLW macro.
PG =            VAL / H'100'            ;       Set PG to the page address of the text.
        expand
                MOVLW   VAL & H'FF'
        noexpand
                ENDM                    ;

;-----------------------------------------------------------------------------------------

_CALL____LCD_TEXT       MACRO           ;       LCD_TEXT macro, standard CALL.
 IF PG == 0                             ;
               _CALL    LCD_TEXT_0      ; (2)
 ELSE                                   ;
               _CALL    LCD_TEXT_1      ; (2)
 ENDIF                                  ;
                ENDM                    ;

;-----------------------------------------------------------------------------------------

P_CALL____LCD_TEXT      MACRO           ;       LCD_TEXT macro, SHORT CALL.
 IF PG == 0                             ;
              P_CALL    LCD_TEXT_0      ; (2)
 ELSE                                   ;
              P_CALL    LCD_TEXT_1      ; (2)
 ENDIF                                  ;
                ENDM                    ;

;-----------------------------------------------------------------------------------------

_CALL____MODE_TEXT      MACRO           ;       MODE_TEXT macro, standard CALL.
PG =            MODE0 / H'100'          ;       Set PG to the page address of the text.
               _CALL    MODE_TEXT       ; (1)
                ENDM                    ;

;-----------------------------------------------------------------------------------------

;*****************************************************************************************
;               Bank 0 RAM variables
;
;               H'21' to H'6F' cleared on power up.
;*****************************************************************************************

        CBLOCK  H'20'                           ; Bank 0 ram 80 bytes. 20-6F hex.

                ENC_HISTORY                     ; Last 4 samples of encoder state.
                                                ;
                SW_CAL_CNT                      ; CAL switch pressed counter. (x 8mS)
                SW_MEM_CNT                      ; MEM switch pressed counter. (x 8mS)
                SW_SF_CNT                       ; S/F switch pressed counter. (x 8mS)
                SW_xxx_CNT                      ; xxx switch pressed counter. (x 8mS)
                SW_yyy_CNT                      ; yyy switch pressed counter. (x 8mS)
                KEYPAD_CNT                      ; Keypad pressed counter.     (x 8mS)
                                                ; (Don't move locations of above 7 variables)
                                                ;
                DIGIT_CNT                       ; Keypad digit counter.
                KEYPAD_NUM                      ; Last keypad button pressed.
                                                ;
                FLASHER_COUNT                   ; LCD flasher. Incremented every 8 mS.
                INT_COUNT                       ; 8 to 1. Decremented every 1.024 mS.
                STEP_RATE                       ; Encoder step size. Dec every 1mS.
                ENC_COUNTER                     ; Encoder position counter,
                                                ; 1 to 38 = CW.  -1 to -38 = CCW.
                                                ;
                TIMEOUT_L                       ; 16 bit display timeout down counter,
                TIMEOUT_H                       ; return to main display when = 0.
                EEPROM_WRITE                    ; EEPROM update up counter.
                DP_POSITION                     ; Decimal point position.
                DIGITS                          ; Number of digits to display.
                SWEEP_COUNT                     ; Sweep rate counter.
                STACK_PTR                       ; Parameter stack pointer.
                                                ;
                COL_1                           ; Switch and keypad data from column 1.
                COL_2                           ;               "                    2.
                COL_3                           ;               "                    3.
                COL_4                           ;               "                    4.
                COL_5                           ;               "                    5.
                                                ;
                FLAG1                           ; 8 misc flags.
                FLAG2                           ;       "
                FLAG3                           ;       "
                DISP_FLAGS                      ;       "
                KEYPAD_FLAGS                    ;       "
                SWEEP_FLAGS                     ;       "
                                                ;
                TEMP_1                          ; Temporary storage for subroutines.
                TEMP_2                          ;               "
                TEMP_3                          ;               "
                                                ;
                MEM_H                           ; Memory number display.
                MEM_L                           ;               "
                MEM_NUMBER                      ;
                                                ;
                SWP_RATE_NUM                    ; Current sweep rate number.
                SWEEP_RATE                      ; Current sweep rate.
                SWP_STEP_SIZE:4                 ; Current sweep step size.
                                                ;
                                                ; I2C POWER METER RAM.
                I2C_DATA_RD                     ; Data read from I2C bus.
                I2C_DATA_WR                     ; Data to write to I2C bus.
                METER_COUNT                     ; Power meter update counter.
                PWR_MEM                         ; Power meter memory that is in use.
                MODE                            ; Power meter display mode. (1 to 3)
                MEM_FREQ                        ; Freq of the 7Ghz power meter memory.
                                                ; (0 to 99 =0.0G to 9.9G)
                ID_BYTE                         ; Power meter ID byte.
                                                ;
                DDS_OLD:4                       ; 32 bit previous DDS frequency.
                                                ;
                ARG1_9                          ;           10 byte ASCII buffer.
                ARG1_8                          ;
                ARG1_7                          ; MSD. \ \
                ARG1_6                          ;       | | ARG1, 32 bit maths buffer.
                ARG1_5                          ;       | |
                ARG1_4                          ; LSD.  |/
                ARG1_3                          ;       |
                ARG1_2                          ;       |   ARG1, 64 bit maths buffer.
                ARG1_1                          ;       |   (result from MULTIPLY)
                ARG1_0                          ; LSD. /
                                                ;
                ARG2_3                          ; MSD. \
                ARG2_2                          ;       |   ARG2, 32 bit maths buffer.
                ARG2_1                          ;       |
                ARG2_0                          ; LSD. /
                                                ;
                ARG3_3                          ; MSD. \
                ARG3_2                          ;       |   ARG3, 32 bit maths buffer.
                ARG3_1                          ;       |
                ARG3_0                          ; LSD. /
                                                ;
                ARG4_3                          ; MSD. \
                ARG4_2                          ;       |   ARG4, Lower limit.
                ARG4_1                          ;       |
                ARG4_0                          ; LSD. /
                                                ;
                ARG5_3                          ; MSD. \
                ARG5_2                          ;       |   ARG5, Upper limit.
                ARG5_1                          ;       |
                ARG5_0                          ; LSD. /
        ENDC

;*****************************************************************************************
;               Bank 0-3 common RAM variables
;               H'70' to H'7F'  Do not change the location of any of these variables.
;               H'F0' to H'FF'
;               H'170' to H'17F'
;               H'1F0' to H'1FF'
;*****************************************************************************************

        CBLOCK  H'70'                           ; Shared ram 16 bytes (all 4 banks)

                RESERVED_COMMON_BANK_ICD2       ; Reserved for ICD2.(16F648A-ICD Header)

                COUNT_0                         ; General counter for subroutines.
                COUNT_1                         ;               "
                COUNT_2                         ;               "
                COUNT_3                         ;               "
                COUNT_4                         ;               "

                SAVE_STATUS                     ; Interrupt context is saved here.
                SAVE_W_REG                      ;               "
                SAVE_PORTB                      ;               "
                SAVE_FSR                        ;               "
                SAVE_PCLATH                     ;               "
        ENDC

;*****************************************************************************************
;               Bank 1 RAM variables
;               H'A0' to H'EF'  Do not change the location of any of these variables.
;*****************************************************************************************

        CBLOCK  H'A0'                           ; Bank 1 ram, 80 bytes, A0-EF hex.

                STACK:40                        ; Parameter stack. 40 bytes. (10 ARG's)
        ENDC


        CBLOCK  H'DD'                           ; Bank 1 ram continued.

                MAX_DDS_FREQ:4                  ; Copy of EEPROM variables.
                DDS_CAL:4                       ;
                DDS_OPT                         ;
                STEP_NUM                        ;
                S_DDS_FREQ:4                    ;
                F_DDS_FREQ:4                    ;
                SWEEP_NUM                       ;
        ENDC

;*****************************************************************************************
;               Bank 2 RAM variables
;               H'120' to H'16F'
;*****************************************************************************************

        CBLOCK  H'120'                          ; Bank 2 ram, 80 bytes, 120-16F hex.

                ENC_HISTORY_DUMMY               ; Unused, just used to align SW counters.
                SW_CAL_OLD                      ; State of old CAL switch count.
                SW_MEM_OLD                      ; State of old MEM switch count.
                SW_SF_OLD                       ; State of old S/F switch count.
                SW_xxx_OLD                      ; State of old xxx switch count.
                SW_yyy_OLD                      ; State of old yyy switch count.
                KEYPAD_OLD                      ; State of old keypad count.
                                                ; (Don't move locations of above 7 variables)
        ENDC


        CBLOCK  H'127'                          ; 37 bytes, 127-164 hex.

                CAL_TEMP                        ; Temporary storage for CAL subroutines.
                CAL_NUM                         ; CAL 7Ghz power mem number.
                CAL_PTR                         ; CAL 7Ghz power table pointer.
                CAL_TBL:18                      ; CAL 7Ghz power lookup table.

        ENDC

        CBLOCK  H'165'                          ; Bank 2 ram.

                RESERVED_BANK2_IDC2:11          ; Reserved for ICD2.(16F648A-ICD Header)
        ENDC

;*****************************************************************************************
;               Start of PAGE 0 tables.
;*****************************************************************************************

                ORG     PAGE0 + H'10'

;-----------------------------------------------------------------------------------------
;               Branch table for normal key routines.
;-----------------------------------------------------------------------------------------

KEY_SUB:        MOVF    KEYPAD_NUM,W            ; Get the key pad number.
                ANDLW   B'00001111'             ; Ignore any high order bits.
                ADDWF   PCL,F                   ;
               _GOTO    KEY_FREQ                ; Enter a frequency. First digit = 0
               _GOTO    KEY_1_FUNC              ; GOTO subroutine for keypad 1 press.
               _GOTO    KEY_2_FUNC              ;               "            2
               _GOTO    KEY_3_FUNC              ;               "            3
               _GOTO    KEY_4_FUNC              ;               "            4
               _GOTO    KEY_5_FUNC              ;               "            5
               _GOTO    KEY_6_FUNC              ;               "            6
               _GOTO    KEY_7_FUNC              ;               "            7
               _GOTO    KEY_8_FUNC              ;               "            8
               _GOTO    KEY_9_FUNC              ;               "            9
               _GOTO    KEY_._FUNC              ;               "            *
               _GOTO    KEY_#_FUNC              ;               "            #

;*****************************************************************************************

CON2            MACRO   VAL                     ; Unnamed 2 Byte constant definition.
                expand                          ;
                DT (VAL>>8) & H'FF'             ;
                DT (VAL) & H'FF'                ;
                noexpand                        ;
                ENDM                            ;

;*****************************************************************************************
;                       CONSTANT MACRO
;*****************************************************************************************

CON             MACRO   NAME, VAL               ; Named 2 byte constant definition.
NAME            EQU $ - CONST_START             ;
                DT (VAL>>8) & H'FF'             ;
                DT (VAL) & H'FF'                ;
                ENDM                            ;

;*****************************************************************************************
;                       TABLE OF CONSTANT DEFINITIONS (+ numbers only. 2 byte)
;*****************************************************************************************

CONST:          ADDWF   PCL,F

CONST_START:

 CON2           STEP_0                    ; Step size 2 byte constants.
 CON2           STEP_1
 CON2           STEP_2
 CON2           STEP_3
 CON2           STEP_4
 CON2           STEP_5
 CON2           STEP_6
 CON2           STEP_7
 CON2           STEP_8
 CON2           STEP_9



 CON            ATTEN_LIM       , 2000          ; +200.0 dB (amplifier) to -200.0 dB limit.
 CON            MEM_OFFSET_LIM  ,  999          ; +99.9 dB to -99.9 dB limit.
 CON            CAL_LIM         ,32767          ; +3276.7 dB to -3276.7 dB limit.
                                                ; For slope and offset
 CON            CAL_30_DBM      ,  300          ; +30.0 dBm.
 CON            CON_1023        , 1023          ; Max A/D result.

;*****************************************************************************************
;                       TABLE OF CONSTANT DEFINITIONS (+ numbers only. 1 byte)
;*****************************************************************************************

GET_RATE: 		ADDWF   PCL,F
                RETLW   RATE_0            ; Step rate 1 byte constants.
                RETLW   RATE_1
                RETLW   RATE_2
                RETLW   RATE_3
                RETLW   RATE_4
                RETLW   RATE_5
                RETLW   RATE_6
                RETLW   RATE_7
                RETLW   RATE_8
                RETLW   RATE_9

;*****************************************************************************************
;                       Get the number of bytes returned by the I2C RX commands 8 to 15.
;*****************************************************************************************

RX_BYTES:       ADDLW   -8                      ;
                ADDWF   PCL,F                   ;
                RETLW   3                       ; Commands 8.
                RETLW   4                       ; Commands 9.
                RETLW   0                       ; Commands 10. (unused)
                RETLW   0                       ; Commands 11. (unused)
                RETLW   0                       ; Commands 12. (unused)
                RETLW   0                       ; Commands 13. Use the I2C_TO_LCD routine.
                RETLW   2                       ; Commands 14. (if command 1, mode is 0)
                RETLW   4                       ; Commands 15.

;-----------------------------------------------------------------------------------------

TABLE_0_END

 IF $ >= PAGE0 + H'100'
 ERROR "Page 0 tables too big. More than 256 bytes long"
 ENDIF

;*****************************************************************************************
;               End of PAGE 0 tables.
;*****************************************************************************************


;*****************************************************************************************
;               Start of PAGE 1 tables.
;
;*****************************************************************************************

                ORG     PAGE1                   ;

;*****************************************************************************************
;
;               Text table.
;
;               Each string terminates when a 0 is reached.
;               Strings with "" (nothing) terminate at the end of the next string.
;
;*****************************************************************************************

TABLE_0:        ADDWF   PCL,F                   ; Jump to character pointed to in W reg.

;               LABLE           STRING                  TERMINATION
TABLE_START:

 TEXT_STRING    DDS:      ,     "DDS REF MULT"          ,0
 TEXT_STRING    REF:      ,     " REFCLK"               ,0
 TEXT_STRING    DDS_SYS:  ,     " SYSTEM CLK"           ,0

 TEXT_STRING    SAVE_TO:  ,     "SAVE TO MEM "          ,0
 TEXT_STRING    VFO:      ,     "VFO"                   ,0

 TEXT_STRING    MAX:      ,     "MAX "                  ,""             ; \
 TEXT_STRING    DDS_FREQ: ,     "DDS "                  ,""             ;  |   Do not
 TEXT_STRING    FREQ:     ,     "FREQ "                 ,0              ; /   separate

 TEXT_STRING    SAVING:   ,     "SAVING"                ,0

 TEXT_STRING    FAST_:    ,     "FAST "      			,""				; \   Do not
 TEXT_STRING    SWEEP_: 	  ,     "SWEEP       "     		,0				; /   separate

 TEXT_STRING    FAST:     ,     "FAST "      			,""				; \   Do not
 TEXT_STRING    WOBBLE_:   ,     "WOBBLE      "      	,0				; /   separate

 TEXT_STRING    SINGLE:   ,     "SINGLE SWEEP"      	,0


 TEXT_STRING    MHZ:      ,     "MHz"                   ,0              ; \   Do not
 TEXT_STRING    KEY_TXT:  ,     "#  "                   ,0              ; /   separate

 TEXT_STRING    ATT_TEXT: ,     " ATTENUATOR"           ,0
 TEXT_STRING    DBM_TEXT: ,     "dBm"                   ,0
 TEXT_STRING    DB_TEXT:  ,     "dB"                    ,0
 TEXT_STRING    CAL_TEXT: ,     "CAL "                  ,0
 TEXT_STRING    PWR_TEXT: ,     "POWER "                ,0
 TEXT_STRING    OFF_TEXT: ,     "OFFSET "               ,0
 TEXT_STRING    SLP_TEXT: ,     "SLOPE  "               ,0
 TEXT_STRING    MEM_TEXT: ,     "CAL PWR MEMORY ?"      ,0
 TEXT_STRING    EXIT_TEXT:,     "EXIT"                  ,0
 TEXT_STRING    SEL_TEXT: ,     "MEM   TEXT"            ,0
 TEXT_STRING    SAVE_TEXT:,     " SAVE CHANGES ?"       ,0

 TEXT_STRING    YES_TEXT: ,     "YES"                   ,0              ; \   Do not
 TEXT_STRING    NO_TEXT:  ,     " NO"                   ,0              ; /   separate



 TEXT_STRING    VER:      ,     VER_TEXT                ,0

TABLE_END

;-----------------------------------------------------------------------------------------

TABLE_1_END

 IF $ >= PAGE1 + H'100'
 ERROR "Page 1 tables too big. More than 256 bytes long"
 ENDIF

;*****************************************************************************************
;               End of PAGE 1 tables.
;*****************************************************************************************




;*****************************************************************************************
;
;               Start of the program. Initialise the hardware.
;
;*****************************************************************************************

                ORG     RESETVECTOR     ;

                NOP                     ;       Required by ICD2.
               _GOTO    CONTINUE        ;

;*****************************************************************************************
;
;               Redirect the Interrupt service routine.
;
;*****************************************************************************************

                ORG     INTVECTOR       ;

                MOVWF   SAVE_W_REG      ;       Save W reg.
                SWAPF   STATUS,W        ;
                CLRF    STATUS          ;       Make sure we are addressing bank 0.
                MOVWF   SAVE_STATUS     ;       Save STATUS reg.
                MOVF    PORTB,W         ;
                MOVWF   SAVE_PORTB      ;       Save PORT B reg.
                MOVF    PCLATH,W        ;
                MOVWF   SAVE_PCLATH     ;       Save PCLATH reg.
                MOVF    FSR,W           ;
                MOVWF   SAVE_FSR        ;       Save FSR reg.
                                        ;
              P_GOTO    INT_SERVICE     ;       Goto interrupt.

;*****************************************************************************************
;
;               Continue with the program.
;
;*****************************************************************************************

                ORG     TABLE_0_END     ;
                                        ;
CONTINUE:     P_CALL    INIT_REGS       ; (2)   Set up any registers & ram that need it.
                                        ;
               _CALL    EE_TO_RAM       ; (3)   Get setup & cal info from EEPROM.
                                        ;
              P_CALL    INIT_LCD        ; (4)   Initialise the LCD display.
                                        ;
               _CALL    SETUP_SWEEP     ; (2)   Setup the sweep variables.
                                        ;
              P_CALL    WAIT_100MS      ; (2)
                                        ;
               _MOVLW   VER             ;
              P_CALL____LCD_TEXT        ; (3)   Display version num. ie  SweepGen v1.0
                                        ;
 IF AD9851_CHIP                         ;       Is the assembly for the AD9851 ?
                MOVLW   DDS_OPT         ;       Y. Get the address of DDS_OPT.
                MOVWF   FSR             ;
                MOVLW   "1"             ;          Is the x6 ref in use ?
                BTFSC   INDF,X6_REF     ;          N. Display '1'
                MOVLW   "2"             ;          Y. Display '2'
 ELSE                                   ;
                MOVLW   "0"             ;       N. Display '0'
 ENDIF                                  ;
              P_CALL    LCD_CHR         ; (2)   Display the last digit of the version num.
                                        ;
                MOVLW   H'C1'           ;
              P_CALL    LCD_CMD         ; (2)   Goto LCD line 2 position 2.
                                        ;
                MOVLW   13              ;       Power meter command
               _CALL    SET_RX_COMMAND  ; (4)   13 = Return software ver.
                BTFSC   I2C_ACK_FLAG    ;       Did the power meter acknowledge the address.
                BSF     NO_PWR_METER    ;       N. Flag that the power meter is not connected.
                                        ;
                MOVLW   15              ;       14 characters in version text + ID byte.
                BSF     VER_FLAG        ;       Set the version text flag.
               _CALL    I2C_TO_LCD      ; (4)   Display the power meter version text.
                BCF     VER_FLAG        ;       Clear the version text flag.
                                        ;
 IF AD9851_CHIP                         ;       Is the assembly for the AD9851 ?
                MOVLW   161             ;
                MOVWF   COUNT_2         ;
VER_LOOP:      _CALL    CLEAR_ARG1      ; (1)
                BTFSS   COUNT_2,0       ;       Y. Toggle the AD9851 x6 bit.
                BSF     ARG1_7,0        ;          (Set W32 when count is even)
               _CALL    DDS_SEND        ; (1)      Initialise the DDS chip.
              P_CALL    WAIT_8MS        ; (2)      Wait a while.
                DECFSZ  COUNT_2,F       ;          Loop until count = 0.
               _GOTO    VER_LOOP        ;
 ELSE                                   ;
               _CALL    DDS_SEND        ; (1)   N. Initialise the AD9850 DDS chip.
VER_LOOP:       BTFSS   EEPROM_WRITE,7  ;          Wait 1 sec. (until EEPROM_WRITE > 127)
               _GOTO    VER_LOOP        ;
 ENDIF                                  ;
                CLRF    ENC_COUNTER     ;       Reset the rotary encoder position count.
                                        ;
                BTFSS   SW_CAL_CNT,7    ;       N. Has CAL button been pressed for 1 sec ?
               _GOTO    RELEASE_LOOP    ;
              P_GOTO    CAL_SCRNS       ;          Y. Display the calibration screens.
                                        ;
RELEASE_LOOP:   BTFSC   ANY_BUTTON      ;       Wait until all buttons are released.
               _GOTO    RELEASE_LOOP    ;

;*****************************************************************************************
;
;       Main display screen.
;
;               Get the Start and Finish frequencies, sweep rate and step size from EEPROM.
;
;               Continuously check the rotary encoder for movement, make sure the
;               frequency stays within set limits. If the power meter it is fitted
;               display the power reading. Check the keypad buttons and perform the
;               function assigned to that button.
;
;               If the step size button is pressed, allow the user to alter the step size.
;
;               If the frequency or step size have been changed update the EEPROM
;               2 seconds after the rotary encoder stops moving.
;
;               If the CAL button is pressed for 1 second, display the setup screens.
;
;               If the MEM button is pressed briefly, activate the MEM function.
;               If the MEM button is pressed for 1 second, activate the MEM SAVE function.
;
;               If the S/F button is pressed briefly, change the frequency displayed.
;               (Start or Finish)
;
;*****************************************************************************************

;-----------------------------------------------------------------------------------------
;               Initialise the MAIN screen.
;-----------------------------------------------------------------------------------------

RS_PWR_METER: P_CALL    LCD_CLEAR       ; (3)   Clear the display.
               _CALL    RS_METER        ; (5)   Power meter command 7 FF = Reset the
                                        ;       power meter software.
                                        ;
MAIN_DISPLAY:   BTFSS   MEM_ACTIVE      ;       Is the MEM function active ?
               _CALL    EE_TO_RAM       ; (3)   N. Get setup & cal info from EEPROM.
                                        ;
                MOVLW   8               ;       Power meter command 8 = Return the memory
               _CALL    RX_I2C_BYTES    ; (4)   number and display mode.
                BTFSS   RX_I2C_OK       ;       Were the bytes received ok.
               _GOTO    RX_FAULTY       ;
                MOVF    ARG3_3,W        ;       Y. Get the 1st RX byte.
                MOVWF   PWR_MEM         ;          Save the memory number byte.
                MOVF    ARG3_2,W        ;          Get the 2nd RX byte.
                MOVWF   MODE            ;          Save the display mode byte.
                MOVF    ARG3_1,W        ;          Get the 3rd RX byte.
                MOVWF   MEM_FREQ        ;          Save the mem freq byte.
RX_FAULTY:                              ;
               _CALL    RX_COMMAND_14   ; (4)   Power meter command 14 = Return the power.
                                        ;
                                        ;       ***** Calculate the lower limit *****
SET_LIMITS:    _CALL    CLEAR_ARG4      ; (1)   Save 0 Hz as the lower limit. (ARG4)
                                        ;
                MOVLW   MAX_DDS_FREQ    ;       ***** Calculate the upper limit *****
               _CALL    RAM_TO_ARG2     ; (2)   Get the upper DDS freq.
               _CALL    ARG2_TO_ARG5    ; (1)   Save it as the upper limit. (ARG5)

;-----------------------------------------------------------------------------------------
;               Used by the MAIN functions.
;-----------------------------------------------------------------------------------------

MAIN_LOOP:      MOVLW   STACK           ;       Reset the ARG stack. (Clear any ARG'S left
                MOVWF   STACK_PTR       ;       by a timeout during the setup screens)
                                        ;
              P_CALL    DISP_LINE_1     ; (5)   Display line 1 LCD text.
               _CALL    SF_TO_ARG2      ; (2)   Get the DDS start or finish freq.

;-----------------------------------------------------------------------------------------

               _CALL    CHK_SF_BRIEF    ; (1)
                BTFSS   PRESSED         ;       Was the S/F button pressed briefly ?
               _GOTO    SKIP_SF         ;
               _CALL    FREQ_TO_EE      ; (3)   Y. Copy start & finish freq, sweep
                                        ;          rate & step size to EEPROM.
                MOVLW   SF_MASK         ;          Change the state of the S/F flag.
                XORWF   DISP_FLAGS,F    ;          Go back to the start and get the
               _GOTO    MAIN_DISPLAY    ;          and get new frequency.
SKIP_SF:                                ;

;*****************************************************************************************
;               POWER METER memory selection function.
;               Check if the POWER METER memory needs changing.
;*****************************************************************************************

                BTFSS   EN_MEM_SEL      ;       Is the select power meter mem active ?
               _GOTO    SKIP_MEM_SEL    ;       N. Skip the meter memory selection.
                                        ;
               _CALL    CLEAR_ARG4      ; (1)   ***** Calculate the lower limit *****
                INCF    ARG4_0,F        ;       Save 1 as the lower limit. (ARG4)
                                        ;
               _CALL    CLEAR_ARG5      ; (1)   ***** Calculate the upper limit *****
                                        ;
                MOVLW   5               ;
                BTFSC   NOT_LTC5508_PWR ;       Is the LTC5508 power meter fitted ?
                MOVLW   8               ;       N. 8 memories. (LTC5508 has only 5)
                MOVWF   ARG5_0          ;       Save 5 or 8 as the upper limit. (ARG5)
                                        ;
               _CALL    CLEAR_ARG2      ; (1)   ***** Get the MEMORY number.        *****
                MOVF    PWR_MEM,W       ;
                MOVWF   TEMP_1          ;       Save a copy.
                MOVWF   ARG2_0          ;       ***** Put it in ARG2 for adjustment *****
                                        ;
               _CALL    UP_DN_1         ; (5)   Chk encoder. (inc/dec ARG2, & chk limits)
                                        ;
               _CALL    CHK_KEYPAD      ; (5)   Check for keypad functions.
                                        ;       (no frequency entry)
                MOVF    ARG2_0,W        ;
                SUBWF   PWR_MEM,W       ;
                BTFSC   ZERO            ;       Any memory number changes ?
               _GOTO    SET_LIMITS      ;       N. Return to the main loop.
                                        ;
                MOVLW   0               ;       Y. Power meter command
               _CALL    SET_TX_COMMAND  ; (4)      0 = Set the memory number to use.
                MOVF    ARG2_0,W        ;
                MOVWF   PWR_MEM         ;          Update the MEM number.
               _CALL    TX_BYTE_END     ; (4)      Send the byte & finish the command.
                                        ;
                MOVLW   40              ;          Wait 40mS. 
              P_CALL    MS_WAIT         ;          Until meter has changed memories.
                                        ;
               _GOTO    MAIN_DISPLAY    ;          Return to the main loop.
SKIP_MEM_SEL:                           ;

;-----------------------------------------------------------------------------------------
;               Check if the SWEEP function is in use.
;-----------------------------------------------------------------------------------------

                BTFSS   SWEEP_ACTIVE    ;       Is the sweep function active ?
               _GOTO    MAIN_FUNCT      ;       N. Skip the sweep function.

;*****************************************************************************************
;               SWEEP function.
;*****************************************************************************************

                BTFSS   SWEEP_STOPPED   ;       Is the sweep running, and is it
                BTFSS   SWEEP_CLOCK     ;       time to change the DDS freq ?
               _GOTO    SWEEP_NO_CNG    ;       N. No change or frequency requred.

FAST_SWP_START:	BTFSS   SWEEP      		;		Sweep mode?
			   _GOTO	NO_TRIGGER		;		N.
				BTFSS   SWEEP_DIR		;       Is the direction down ?		
  			   	BSF		EN_TRIGGER		;		Y.  Set the trigger enable flag.   
NO_TRIGGER:		BTFSS   SWEEP_CLOCK     ;       For fast sweep, wait until its
               _GOTO    FAST_SWP_START  ;       time to change the DDS freq.
                BCF     SWEEP_CLOCK     ;       Y. Clear the sweep clock bit.
                MOVLW   SWP_STEP_SIZE   ;
               _CALL    COPY_TO_ARG3    ; (1)   Get the sweep constant.
                BTFSS   SWEEP_DIR       ;       Is the direction down ?
               _CALL    NEG_ARG3        ; (1)   Y. Make step size negative.
               _CALL    ADD             ; (3)   Inc ARG2 by the step size.
                BCF     SWEEP_CHANGE    ;       Clear the change direction flag.
                BTFSS   SWEEP_DIR       ;       Is the direction down ?
               _CALL    CHK_MIN         ; (3)   Y. Make sure we dont go below ARG4.
 	         	BTFSC   SWEEP_DIR       ;       Is the direction up ?
               _CALL    CHK_MAX         ; (3)   Y. Make sure we dont go above ARG5.
                MOVLW   SWEEP_DIR_MASK  ;
                BTFSC   SWEEP_CHANGE    ;       End of the sweep ?
                XORWF   SWEEP_FLAGS,F   ;       Y. Change the sweep direction.
				BTFSC	WOBBLE			;		Wobble mode?
               _GOTO	SWEEP_NO_CNG	;		Y.   
				BTFSS   SWEEP_DIR       ;		Is the direction down ? 
			   _CALL	ARG4_TO_ARG2	;		Y. Flyback to the start frequency
				BTFSS	SWEEP_DIR		;		Is the direction down ?
			   _CALL	STOP_SWEEP 
SWEEP_NO_CNG:	BTFSS	SWEEP_STOPPED	;		Is the sweep halted?
		       _GOTO	NOT_STOPPED		;		N.
		        BTFSC	SWEEP			;		Y. Sweep mode?
               _CALL	ARG4_TO_ARG2	;		Y. Load the start frequency
		
NOT_STOPPED:   _CALL    CHK_SWEEP_RATE  ; (3)   Chk encoder (inc/dec sweep rate)
               _CALL    ARG2_TO_SF      ; (1)   Save the start or finish frequency.
   			   	BTFSC   EN_TRIGGER		;		Is the trigger enabled?
               _CALL    TRIG_PULSE		;		Y.  
			   _CALL    FREQ_TO_LCD     ; (4)   Update the DDS, and display it.
                                        ;       (don't display it in fast sweep mode)
               _CALL    CHK_KEYPAD      ; (5)   Check for keypad functions.
                                        ;       (no frequency entry)
                BTFSC   FAST_WOBBLE     ;       Are we doing a fast sweep.
               _GOTO    FAST_SWP_START  ;       Y. Repeat the sweep.
                BTFSS   SWEEP_ACTIVE    ;       Is the sweep function active ?
               _GOTO    SET_LIMITS      ;       N. Reset the limits & return to the main loop.
               _GOTO    MAIN_LOOP       ;       Y. Return to the main loop.
               
;-----------------------------------------------------------------------------------------
               
TRIG_PULSE: 	BSF		PORTA,6			;       Trigger pulse positive edge.
	       	  P_CALL	WAIT_8MS
				BCF		PORTA,6			;       Trigger pulse negstive edge.
				BCF		EN_TRIGGER		;		Clear the trigger enable flag
				RETURN


STOP_SWEEP:		BTFSS	SINGLE_SWEEP	;		Single sweep mode?
				RETURN					;		N.
				MOVLW   SWEEP_STP_MASK  ;       Y.  Stop sweeping
                XORWF   SWEEP_FLAGS,F   ;       Change the state of the sweep stopped flag.
				BCF		SINGLE_SWEEP	;		Clear the single sweep flag
				RETURN	

;-----------------------------------------------------------------------------------------
;               Used by the MAIN functions.
;-----------------------------------------------------------------------------------------

MAIN_FUNCT:     BTFSS   MEM_ACTIVE      ;       Is the MEM function active ?
               _GOTO    MAIN_NO_MEM     ;
                                        ;
               _CALL    LIMIT           ; (4)   Y. Check the frequency limits.
               _CALL    ARG2_TO_SF      ; (1)      Save the start or finish frequency.
               _GOTO    MAIN_UPDATE     ;          Update the display & DDS.
                                        ;
MAIN_NO_MEM:    BSF     KEYPAD_MAIN     ;       N. Flag that the main routine called KEYPAD.
               _CALL    CHK_KEYPAD      ; (5)      Chk for keypad I/P.
                                        ;
               _CALL    UP_DOWN         ; (5)      Chk enc (inc/dec ARG2, & chk limits)
                                        ;          and adjust step size.
               _CALL    ARG2_TO_SF      ; (1)      Save the start or finish frequency.
                                        ;
                BTFSC   EEPROM_UPDATE   ;          Is the EEPROM update flag set?
               _CALL    FREQ_TO_EE      ; (3)      Y. Copy start & finish freq, sweep
                                        ;             rate & step size to EEPROM.
                                        ;
MAIN_UPDATE:   _CALL    FREQ_TO_LCD     ; (4)   Update the DDS and display it.

;-----------------------------------------------------------------------------------------

                BTFSC   MEM_ACTIVE      ;       Is the MEM function active ?
               _GOTO    MEM_FUNCT       ;       Y. Goto the MEM function.

;-----------------------------------------------------------------------------------------
;               Used by the MAIN functions. Check the MEM button.
;-----------------------------------------------------------------------------------------

               _CALL    CHK_MEM_BRIEF   ; (1)
                BTFSS   PRESSED         ;       Was the MEM button pressed briefly ?
               _GOTO    CHK_MEM_SAVE    ;
                                        ;
               _CALL    FREQ_TO_EE      ; (3)   Y. Copy start & finish freq, sweep
                                        ;          rate & step size to EEPROM.
                BSF     MEM_ACTIVE      ;          Set the MEM func flag.
               _GOTO    MEM_LIMITS      ;          Initialise the MEM display,
                                        ;          then return to main loop.
                                        ;
CHK_MEM_SAVE:   BTFSC   SW_MEM_CNT,7    ;       N. Was the MEM button pressed for 1 sec ?
               _GOTO    MEM_SAVE        ;          Y. Goto the MEM SAVE display,
                                        ;             then return to main loop.

;-----------------------------------------------------------------------------------------
;               Used by the MAIN functions. Check the CAL button.
;-----------------------------------------------------------------------------------------

                BTFSS   NO_PWR_METER    ;       Is the power meter fitted and
                BTFSS   SW_CAL_CNT,7    ;       was the CAL button pressed for 1 sec ?
               _GOTO    MAIN_LOOP       ;       N. Continue updating the display.
                                        ;
               _CALL    FREQ_TO_EE      ; (3)   Y. Copy start & finish freq, sweep rate &
                                        ;          step size to EEPROM.
               _CALL    RS_METER        ; (5)      Power meter command 7 FF = Reset the
                                        ;          power meter software.
                                        ;
                BCF     RELATIVE_FLAG   ;          Clear the relative dBr flag.
              P_GOTO    SETUP_SCRNS     ;          Goto setup screens.

;-----------------------------------------------------------------------------------------
;               Used by the MAIN functions. When the MEM function is active.
;-----------------------------------------------------------------------------------------

MEM_FUNCT:     _CALL    CHK_MEM_BRIEF   ; (1)   Was the MEM button pressed briefly ?
                BTFSC   PRESSED         ;       Y. Deactivate the MEM function,
               _GOTO    MEM_END_1       ;          but keep the original setup.
                                        ;
                BTFSC   SW_MEM_CNT,7    ;       Was the MEM button pressed for 1 sec ?
               _GOTO    MEM_END_2       ;       Y. Deactivate the MEM function,
                                        ;          and copy the MEM to the setup.
               _GOTO    MEM_LIMITS      ;       Use part of the MEM SAVE function to
                                        ;       select the memory.
MEM_FUNCT_CONT: MOVF    ARG1_1,W        ;
                BTFSS   ARG1_1,4        ;       Save the MEM display number.
                MOVF    ARG1_0,W        ;       (if first chr is blank, skip it)
                MOVWF   MEM_H           ;
                MOVF    ARG1_0,W        ;
                BTFSS   ARG1_1,4        ;
                MOVLW   " "             ;
                MOVWF   MEM_L           ;
                                        ;
               _CALL    EE_TO_RAM       ; (3)   Get the current setup from EEPROM.
               _CALL    RECALL          ; (4)   Recall the selected memory.
               _GOTO    MAIN_DISPLAY    ;       Return to the main display.
                                        ;
MEM_END_1:     _CALL    EE_TO_RAM       ; (3)   Get setup & cal info from EEPROM.
MEM_END_2:      BCF     MEM_ACTIVE      ;       Deactivate the MEM function.
               _CALL    RAM_TO_EE       ; (3)   Save any RAM changes to EEPROM.
              P_CALL    DISP_LINE_1     ; (5)   Display line 1 LCD text.
               _GOTO    RELEASE_MEM     ;       Wait for the MEM button to be released.
                                        ;       Return to the main display.

;*****************************************************************************************
;               End of the MAIN screen.
;*****************************************************************************************

;*****************************************************************************************
;
;       MEM SAVE function display.
;
;               Display 'SAVE TO MEM' on the LCD.
;               Continuously check the rotary encoder for movement, (selects the mem num)
;               On the 2nd line of the LCD display the start freq of the selected memory.
;               If the MEM button is pressed briefly, or time out is reached, return to
;               the previous screen. Do not save any changes to EEPROM.
;               If the MEM button is pressed for 1 second, save the frequency
;               to the selected memory.
;               Display the "SAVING" message, then return the previous screen.
;
;*****************************************************************************************

MEM_SAVE:      _CALL    FREQ_TO_EE      ; (3)   Copy start & finish freq, sweep rate &
                                        ;       step size to EEPROM.
                                        ;
              P_CALL    LCD_CLEAR       ; (3)   *********************************
               _MOVLW   SAVE_TO         ;       *  Clear the display.           *
              P_CALL____LCD_TEXT        ; (3)   *  Display 'SAVE TO MEM'.       *
                                        ;       *********************************
                BSF     SAVE_FLAG       ;
                                        ;
MEM_LIMITS:    _CALL    CLEAR_ARG4      ; (1)   ***** Calculate the lower limit *****
                INCF    ARG4_0,F        ;       Save 1 as the lower limit. (ARG4)
                                        ;
               _CALL    CLEAR_ARG5      ; (1)   ***** Calculate the upper limit *****
                MOVLW   11              ;       11 Memories.
                MOVWF   ARG5_0          ;       Save 11 as the upper limit. (ARG5)
                                        ;
                BCF     TIMEOUT         ;       Clear the display timeout flag.
MEM_SAVE_LOOP:  BTFSC   TIMEOUT         ;       Has the display timeout been reached ?
               _GOTO    MEM_SAVE_EXIT   ;       Y. Ignore changes, and return.
                                        ;
               _CALL    CLEAR_ARG2      ; (1)   ***** Get the MEMORY number.        *****
                MOVF    MEM_NUMBER,W    ;
                MOVWF   ARG2_0          ;       ***** Put it in ARG2 for adjustment *****
                                        ;
               _CALL    UP_DN_1         ; (5)   Chk encoder. (inc/dec ARG2, & chk limits)
                                        ;
               _CALL    BIN_TO_DEC_FB   ; (2)   Convert the number to ASCII decimal.
                                        ;       (full leading 0 blanking)
                MOVF    ARG2_0,W        ;
                MOVWF   MEM_NUMBER      ;       Update the MEM number.
                                        ;
                BTFSC   MEM_ACTIVE      ;       Is the MEM recall function active?
               _GOTO    MEM_FUNCT_CONT  ;       Y. Go back to the MEM recall function.
                                        ;
                MOVLW   7               ;
                MOVWF   DP_POSITION     ;       Save the position of the DP.
               _CALL    DISP_4_DIGITS   ; (3)   Display the number.
                                        ;
               _CALL    RECALL          ; (4)   Recall the selected memory.
               _CALL    SF_TO_ARG2      ; (2)   Get the DDS start or finish freq.
                                        ;
               _CALL    DISPLAY_FREQ    ; (4)   Display it.
                                        ;
               _CALL    CHK_MEM_BRIEF   ; (1)
                BTFSS   PRESSED         ;       Was the MEM button pressed briefly ?
               _GOTO    SKIP_RESTORE    ;
MEM_SAVE_EXIT:P_GOTO    RESTORE_SETUP   ; (3)   Y. Ignore changes, and return.
                                        ;
SKIP_RESTORE:   BTFSS   SW_MEM_CNT,7    ;
                BCF     SAVE_FLAG       ;
                                        ;
                BTFSS   SAVE_FLAG       ;
                BTFSS   SW_MEM_CNT,7    ;       Was the MEM button pressed for 1 sec ?
               _GOTO    MEM_SAVE_LOOP   ;       N. Continue updating the MEM display.
                                        ;
               _CALL    EE_TO_RAM       ; (3)   Restore the original setup.
               _CALL    SAVE            ; (4)   Save to selected MEM in EEPROM.
SAVE_MESSAGE: P_CALL    LCD_CLEAR       ; (3)   *********************************
                MOVLW   H'85'           ;       *  Clear the display.           *
              P_CALL    LCD_CMD         ; (2)   *  Goto LCD line 1 position 6.  *
               _MOVLW   SAVING          ;       *  Display 'SAVING'.            *
              P_CALL____LCD_TEXT        ; (3)   *********************************
                                        ;
WAIT:           BTFSC   TIMEOUT_L,7     ;       Wait 1 second.
               _GOTO    WAIT            ;
                                        ;
RELEASE_MEM:    BTFSC   SW_MEM_CNT,7    ;       Is the MEM button pressed ?
               _GOTO    RELEASE_MEM     ;       N. Wait until it is released.
              P_GOTO    RESTORE_SETUP   ;       Return to the main display.

;*****************************************************************************************
;               Start of major subroutines
;*****************************************************************************************


;*****************************************************************************************
;
;      NAME:    UP_DOWN
;
;   PURPOSE:    If the rotary encoder has been moved, inc or dec the variable in ARG2,
;               using step size.
;               Ensure that ARG2 remains within limits.
;               Lower limit (ARG4) and upper limit (ARG5)
;               Also allow the user to change the step size.
;
;      NAME:    UP_DN_1
;
;   PURPOSE:    If the rotary encoder has been moved, inc or dec the variable in ARG2
;               by 1, ignore the contents of step size, and dont use the step rate.
;               Ensure that ARG2 remains within limits.
;
;     INPUT:    ARG2.
;
;    OUTPUT:    ARG2 = ARG2 +- N.
;
; STACK USE:    4
;
;*****************************************************************************************

UP_DN_1:        BSF     UNITS           ;       Set the units flag.
               _GOTO    UP_DN           ;       Skip the step size check.

UP_DOWN:       _CALL    CHK_STEP_SIZE   ; (3)   See if the user want to change step size.
UP_DN:          MOVF    ENC_COUNTER,W   ;
                BTFSC   ZERO            ;       Has the rotary encoder moved ?
               _GOTO    LIMIT           ;
                                        ;
                MOVWF   COUNT_2         ;       Y. Save the encoder count.
                CLRF    ENC_COUNTER     ;          Reset the encoder position count.
                                        ;
                MOVLW   STEP_NUM        ;          Get the address of STEP_NUM.
                MOVWF   FSR             ;
                MOVF    INDF,W          ;          Get the STEP_NUM.
                BTFSC   UNITS           ;          Is units flag set ?
                CLRW                    ;          Y. Use a step size of 1Hz instead.
               _CALL    TEN_TO_POWER_W  ; (4)      Convert to step size, place in ARG3.
                                        ;
                BTFSC   COUNT_2,7       ;          Is the encoder movement CCW ?
               _CALL    NEG_ARG3        ; (1)      Y. Make step size negative.
                BCF     COUNT_2,7       ;          Make encoder count positive.
                                        ;
STEP_UP:       _CALL    ADD             ; (3)      Inc ARG2 by the step size.
                BTFSC   UNITS           ;          If LIMIT is set, only add once.
               _GOTO    LIMIT           ;
                DECFSZ  COUNT_2,F       ;          Dec the loop count.
               _GOTO    STEP_UP         ;          Inc ARG2 until count = 0.
                                        ;
LIMIT:          BCF     UNITS           ;       Check that ARG2 remains within limits.
               _CALL    CHK_MIN         ; (3)   Make sure we dont go below ARG4.
               _GOTO    CHK_MAX         ; (2)   Make sure we dont go above ARG5.
                                        ;       Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    CHK_SWEEP_RATE
;
;   PURPOSE:    If the user wants to change the sweep rate.
;
;     INPUT:    None.
;
;    OUTPUT:    None.
;
; STACK USE:    2
;
;*****************************************************************************************

CHK_SWEEP_RATE: MOVF    ENC_COUNTER,W   ;       Get rotary encoder position.
                BTFSC   ZERO            ;       Was there any movement ?
                RETURN                  ;       N. All done.
                MOVLW   SWEEP_NUM       ;       Get the address of SWEEP_NUM.
                MOVWF   FSR             ;
                BTFSS   ENC_COUNTER,7   ;       Was the encoder movement CCW ?
                INCF    INDF,F          ;       Y. Inc the position.
                MOVLW   -9 - 1          ;
                ADDWF   INDF,W          ;       Has the position gone above 9 ?
                BTFSC   CARRY           ;
                DECF    INDF,F          ;       Y. Return it to 9.
                BTFSC   ENC_COUNTER,7   ;       Was the encoder movement CW ?
                DECF    INDF,F          ;       Y. Dec the position.
                BTFSC   INDF,7          ;       Has the position gone below 0 ?
                INCF    INDF,F          ;       Y. Return it to 0.
                CLRF    ENC_COUNTER     ;       Reset the encoder position.
               _CALL    SETUP_SWEEP     ; (2)   Reset the sweep variables.
              P_CALL    LCD_HOME        ; (2)   Goto LCD line 1 position 1.
DISP_RATE:      MOVF    SWP_RATE_NUM,W  ;       Get sweep rate number.
                ADDLW   '0'             ;
              P_CALL    LCD_CHR         ; (2)   Display the sweep rate number.
				RETURN
              
DISP_MODE:		MOVLW	H'84'
	      	  P_CALL    LCD_CMD         ;
				BTFSS	SWEEP
				BTFSS	FAST_WOBBLE
	      	   _MOVLW   WOBBLE_        	;       Display 'WOBBLE'.
				BTFSC	FAST_WOBBLE
			   _MOVLW   FAST        	;       Display 'FAST WOBBLE'.
				BTFSC	SWEEP
	      	   _MOVLW   SWEEP_       	;       Display 'SWEEP'.
				BTFSC	FAST_SWEEP
			   _MOVLW   FAST_        	;       Display 'FAST SWEEP'.
				BTFSS	SINGLE_SWEEP
			   _GOTO	DISP_TXT
			   _MOVLW	SINGLE
DISP_TXT:     P_CALL____LCD_TEXT        ; 
                RETURN                  ;




;*****************************************************************************************
;
;      NAME:    CHK_STEP_SIZE
;
;   PURPOSE:    If the user wants to change the step size, enable the cursor on the
;               frequency display, and set the STEP_NUM variable.
;
;     INPUT:    None.
;
;    OUTPUT:    None.
;
; STACK USE:    2
;
;*****************************************************************************************

CHK_STEP_SIZE:  BTFSS   SW_STEP_SIZE    ;       Is the step size button pressed ?
                RETURN                  ;       N. All done.
                MOVLW   B'00001110'     ;       Turn on the cursor.
              P_CALL    LCD_CMD         ; (2)
CHK_STEP_LOOP:  MOVLW   STEP_NUM        ;       Get the address of STEP_NUM.
                MOVWF   FSR             ;
                MOVLW   6               ;
                SUBWF   INDF,W          ;       Is cursor before the decimal point ?
                BTFSC   CARRY           ;
                ADDLW   1               ;       Y. Shift cursor left 1 digit.
                SUBLW   H'C5'           ;       Subtract it from the 1Hz char position.
              P_CALL    LCD_CMD         ; (2)   Position the cursor on frequency display.
                MOVF    ENC_COUNTER,W   ;       Get rotary encoder position.
                BTFSC   ZERO            ;
               _GOTO    NO_MOVEMENT     ;       Was there any movement ?
                BTFSC   ENC_COUNTER,7   ;       Y. Was the encoder movement CCW ?
                INCF    INDF,F          ;          Y. Inc the position.
                BTFSC   INDF,3          ;          Has the position gone above 7 ?
                DECF    INDF,F          ;          Y. Return it to 7.
                BTFSS   ENC_COUNTER,7   ;          Was the encoder movement CW ?
                DECF    INDF,F          ;          Y. Dec the position.
                BTFSC   INDF,7          ;          Has the position gone below 0 ?
                INCF    INDF,F          ;          Y. Return it to 0.
NO_MOVEMENT:    CLRF    ENC_COUNTER     ;       Reset the encoder position.
                BTFSC   SW_STEP_SIZE    ;       Is the step size button still pressed ?
               _GOTO    CHK_STEP_LOOP   ;       Y. Keep checking rotary encoder.
              P_CALL    LCD_NORM        ; (2)   N. Turn off the cursor.
                RETURN                  ;

;*****************************************************************************************
;
;      NAME:    KEY_x_FUNC
;
;   PURPOSE:    Run a subroutine based on which keypad button is pressed.
;
;     INPUT:    None.
;
;    OUTPUT:    None.
;
;*****************************************************************************************

;-----------------------------------------------------------------------------------------
;               Key 1 function.	        ( Stop/start relative power measurements )
;-----------------------------------------------------------------------------------------

KEY_1_FUNC:     BTFSC   FAST_WOBBLE      ;       Is the fast sweep function enabled ?
               _GOTO    KEYPAD_EXIT     ;       Y. Clear flags & return.
                                        ;
                MOVLW   6               ;       Power meter command
               _CALL    SET_TX_COMMAND  ; (4)   6 = Stop/start relative power measurements
                                        ;
                MOVLW   REL_MASK        ;       Change the state of the relative flag.
                XORWF   FLAG3,F         ;
                CLRW                    ;
                BTFSC   RELATIVE_FLAG   ;
                MOVLW   1               ;
               _CALL    TX_BYTE_END     ; (4)   Send the byte & finish the command.
                                        ;
               _CALL    RX_COMMAND_14   ; (4)   Power meter command 14 = Return the power.
                                        ;
               _GOTO    KEYPAD_EXIT     ;       Clear flags & return.

;-----------------------------------------------------------------------------------------
;               Key 2 function.	        ( Change the power display mode )
;-----------------------------------------------------------------------------------------

KEY_2_FUNC:     BTFSC   FAST_WOBBLE      ;       Is the fast sweep function enabled ?
               _GOTO    KEYPAD_EXIT     ;       Y. Clear flags & return.
                                        ;
                MOVLW   1               ;       Power meter command
               _CALL    SET_TX_COMMAND  ; (4)   1 = Set the power display mode.
                                        ;
                INCF    MODE,F          ;       Inc the display mode.
                MOVLW   1               ;
                BTFSC   MODE,2          ;
                MOVWF   MODE            ;
                MOVF    MODE,W          ;
               _CALL    TX_BYTE_END     ; (4)   Send the byte & finish the command.
                                        ;
                                        ;       Power meter command
               _CALL    TX_COMMAND_7    ; (4)   7 = Saves changes made to the memory.
                MOVLW   0               ;       0 = Save changes to the setup
               _GOTO    FINISH_7        ;           made by commands 1, 2 or 3.

;-----------------------------------------------------------------------------------------
;               Key 3 function.	        ( Select power meter memory )
;-----------------------------------------------------------------------------------------

KEY_3_FUNC:     BTFSS   SWEEP_ACTIVE    ;       Is the sweep function enabled or
                BTFSC   NO_PWR_METER    ;       is the power meter not fitted ?
               _GOTO    KEYPAD_EXIT     ;       Y. Clear flags & return.
                                        ;
                MOVLW   MEM_MASK        ;       Change the state of the mem select flag.
                XORWF   FLAG3,F         ;
                                        ;
                BTFSC   EN_MEM_SEL      ;       Was power meter memory selection disabled.
               _GOTO    KEYPAD_EXIT     ;       N. Clear flags & return.
                                        ;
                                        ;       Y. Power meter command
               _CALL    TX_COMMAND_7    ; (4)      7 = Saves changes made to the memory.
                MOVLW   1               ;          1 = Save changes to the setup
                                        ;              made by commands 0, 4 or 5.
FINISH_7:      _CALL    TX_BYTE_END     ; (4)      Send the byte & finish the command.
                                        ;
                                        ;          Wait until the power meter has updated
              P_CALL    WAIT_100MS      ; (2)      its EEPROM.
                                        ;
               _CALL    RX_COMMAND_14   ; (4)      PWR meter command 14 = Return the power.
;               _GOTO    KEYPAD_EXIT     ;          Clear flags & return.


;-----------------------------------------------------------------------------------------
;               Key 4 function.         ( Change the wobble direction )
;-----------------------------------------------------------------------------------------

KEY_4_FUNC:		BTFSC	SWEEP
			   _GOTO    KEYPAD_EXIT     ;
				MOVLW   SWEEP_DIR_MASK  ;
                XORWF   SWEEP_FLAGS,F   ;       Change the wobble direction.
			    BCF     SWEEP_STOPPED   ;
               _GOTO    KEYPAD_EXIT     ;

;-----------------------------------------------------------------------------------------
;               Key 5 function.         ( Start / Halt / Resume slow wobble )
;-----------------------------------------------------------------------------------------

KEY_5_FUNC:		BSF		WOBBLE			;		Wobble flag
				BCF		SWEEP			;		Retrace Sweep flag
				BTFSC   EN_MEM_SEL      ;       Is the mem select function active.
               _GOTO    KEYPAD_EXIT     ;       Y. Dont start sweeping.
               _CALL	DISP_MODE       ;
                BTFSS   SWEEP_ACTIVE    ;       Is sweep function enabled ?
               _GOTO    START_STEPPING  ;       N. Start stepping.

HALT_RESUME:    MOVLW   SWEEP_STP_MASK  ;       Y. Halt / Resume wobbling.
                XORWF   SWEEP_FLAGS,F   ;       Change the state of the sweep stopped flag.
	       	   _GOTO    KEYPAD_EXIT     ;

;-----------------------------------------------------------------------------------------
;               Key 6 function.         ( Start fast wobbling )
;-----------------------------------------------------------------------------------------

KEY_6_FUNC:		BSF		WOBBLE			;		Wobble mode
				BTFSS   EN_MEM_SEL      ;       Is the mem select function active or
                BTFSC   SWEEP_ACTIVE    ;       is the sweep function enabled ?
               _GOTO    KEYPAD_EXIT     ;       Y. Clear flags & return.
                BSF     FAST_WOBBLE     ;       Set the fast wobble mode.
			   _CALL	DISP_MODE       ;
			   
START_STEPPING:
			   _CALL    FREQ_TO_EE      ; (3)   Copy DDS freq's and step size to EEPROM.
                BSF     SWEEP_ACTIVE    ;       Turn on the sweep function.
                BCF     SWEEP_STOPPED   ;
                
;               Set the limits to S/F.
               _CALL    PUSH_ARG2       ; (1)   Dont destroy current sweep freq.
                MOVLW   S_DDS_FREQ      ;       Get the start frequency.
               _CALL    COPY_TO_ARG2    ; (1)
                MOVLW   F_DDS_FREQ      ;
               _CALL    RAM_TO_ARG3     ; (2)   Get the finish frequency.
                BSF     TEMP_FLAG       ;
               _CALL    COMPARE         ; (2)   Is it above the current freq ?
                BTFSC   CARRY           ;
                BCF     TEMP_FLAG       ;       N. Set wobble direction to down.
                
;               Set the lower sweep limit.
                BTFSC   TEMP_FLAG       ;       wobble up ?
               _CALL    ARG2_TO_ARG4    ; (1)   Y. The lower limit is the start freq.
               
                BTFSS   TEMP_FLAG       ;
               _CALL    ARG3_TO_ARG4    ; (1)   N. The lower limit is the finish freq.
                                        
;               Set the upper SWEEP limit.
                BTFSS   TEMP_FLAG       ;       Sweep up ?
               _CALL    ARG2_TO_ARG5    ; (1)   Y. The upper limit is the start freq.
                BTFSC   TEMP_FLAG       ;
               _CALL    ARG3_TO_ARG5    ; (1)   N. The upper limit is the finish freq.
               _CALL    PULL_ARG2       ; (1)   Recover current SWEEP freq.
                BCF     SWEEP_DIR       ;
                BTFSC   TEMP_FLAG       ;
                BSF     SWEEP_DIR       ;       Set sweep direction.
               _CALL    SETUP_SWEEP     ; (2)   Set the sweep variables.
                MOVF    SWEEP_RATE,W    ;
                MOVWF   SWEEP_COUNT     ;       Reset the SWEEP counter.
                BCF     SWEEP_CLOCK     ;
               _GOTO    KEYPAD_EXIT     ;
               
;-----------------------------------------------------------------------------------------
;               Key 7 function          ( Single Sweep )
;-----------------------------------------------------------------------------------------

KEY_7_FUNC:		BTFSS   SWEEP_STOPPED   ;		Is the sweep halted?
		       _GOTO    KEYPAD_EXIT     ;		N.
		       	BSF		SWEEP			;		Y. Enable sweep mode
  				BSF		SINGLE_SWEEP	;		   Enable single sweep mode
				BSF		EN_TRIGGER		;		   Enable the trigger pulse
				
;-----------------------------------------------------------------------------------------
;               Key 8 function.         ( Start Slow sweep / Halt / restart sweep )
;-----------------------------------------------------------------------------------------

KEY_8_FUNC:		BSF		SWEEP			;	    Set the Sweep flag
				BCF		WOBBLE			;	    Clear the Wobble flag
			   _CALL	DISP_MODE     	;
				BTFSC   EN_MEM_SEL      ;       Is the mem select function active.
               _GOTO    KEYPAD_EXIT     ;       Y. Dont start sweeping.

STOP_RESUME:    MOVLW   SWEEP_STP_MASK  ;       Y. Halt / Resume wobbling.
                XORWF   SWEEP_FLAGS,F   ;       Change the state of the sweep stopped flag.
				BTFSC   SWEEP_ACTIVE    ;       Is sweep function enabled ?
	           _GOTO    KEYPAD_EXIT     ;		N.
		

                BTFSS   SWEEP_ACTIVE    ;       Is the sweep function enabled ?
	           	BSF		EN_TRIGGER		;		Y.  Enable the trigger pulse
                BTFSS   SWEEP_ACTIVE    ;       Is sweep function enabled ?
               _GOTO    START_STEPPING  ;       Y. Start sweeping.

;-----------------------------------------------------------------------------------------
;               Key 9 function.         ( Fast sweep )
;-----------------------------------------------------------------------------------------               
               
KEY_9_FUNC:	
				BSF		SWEEP			;		Set the Sweep flag
				BTFSS   EN_MEM_SEL      ;       Is the mem select function active or
                BTFSC   SWEEP_ACTIVE    ;       is the sweep function enabled ?
               _GOTO    KEYPAD_EXIT     ;       Y. Clear flags & return.
                BSF     FAST_WOBBLE     ;       Set the fast wobble mode.
				BSF     FAST_SWEEP      ;       Set the fast sweep flag.
			   _CALL	DISP_MODE       ;
	           _GOTO    START_STEPPING  ;       Start sweeping.

;-----------------------------------------------------------------------------------------
;               Key # function.         ( Exit sweep mode )
;                                       ( quick press = return to VFO mode )
;                                       ( long press  = save sweep freq to active VFO )
;-----------------------------------------------------------------------------------------

KEY_#_FUNC:     BTFSS   MEM_ACTIVE      ;       Is the mem select function active or
                BTFSS   SWEEP_ACTIVE    ;       is the sweep function disabled ?
               _GOTO    KEYPAD_EXIT     ;       Y. Just exit.
                                        ;
               _CALL    WRITE_SWEEP     ; (2)   Save changes to the sweep rate to EEPROM.
                                        ;
                BTFSC   FAST_WOBBLE      ;       Is the fast sweep mode enabled ?
               _GOTO    FAST_SWP_EXIT   ;       Y. Just return to VFO mode.
                                        ;
KEY_#_LOOP:     BTFSC   KEYPAD_CNT,7    ;       Has the button been pressed for 1 sec ?
               _GOTO    TIME_OUT_#      ;
                MOVF    KEYPAD_CNT,W    ;
                BTFSS   ZERO            ;
               _GOTO    KEY_#_LOOP      ;
FAST_SWP_EXIT: _CALL    EE_TO_RAM       ; (3)   N. Get setup & cal info from EEPROM.
               _GOTO    KEY_EXIT        ;
                                        ;
TIME_OUT_#:     BSF     EEPROM_UPDATE   ;       Y. Set EEPROM update flag. (VFO = sweep freq)
                                        ;       Calculate the lower limit.
KEY_EXIT:      _CALL    CLEAR_ARG4      ; (1)   Save 0 Hz as the lower limit. (ARG4)
                                        ;
                MOVLW   MAX_DDS_FREQ    ;       Calculate the upper limit.
               _CALL    RAM_TO_ARG2     ; (2)   Get the upper DDS freq.
               _CALL    ARG2_TO_ARG5    ; (1)   Save it as the upper limit. (ARG5)
                CLRF    SWEEP_FLAGS     ;       Stop the sweep functions.
				BCF		WOBBLE
				BCF		SWEEP
               _GOTO    KEYPAD_EXIT     ;

KEY_._FUNC:    _GOTO    KEYPAD_EXIT     ;      Do nothing. Clear flags & return.	

;*****************************************************************************************
;
;      NAME:    CHK_KEYPAD
;
;   PURPOSE:    Checks to see if any keypad button has been pressed.
;               Then converts the digits entered into a number.
;               Any other digits will be entered in front of the MHz decimal point.
;               If the * is entered, any following digits will be entered after the
;               MHz decimal point.
;               The # is then used to accept the number entered.
;
;     INPUT:    None.
;
;    OUTPUT:    ARG2 = freq.
;               (ARG3 destroyed)
;
; STACK USE:    4
;
;*****************************************************************************************

CHK_KEYPAD:    _CALL    CHK_KEY_BRIEF   ; (1)
                BTFSS   PRESSED         ;       Has any keypad button been pressed ?
               _GOTO    KEYPAD_EXIT     ;       N. All done.
                                        ;
                BTFSC   CAL_FLAG        ;       Did the cal screens call CHK_KEYPAD.
               _GOTO    KEY_FREQ        ;       Y. Skip the key functions.
                                        ;
               _GOTO    KEY_SUB         ;       Goto function based on the key pressed.
                                        ;       ( 0 goes KEY_FREQ. )
                                        ;       ( 1 goes to KEY_1_FUNC.     )
                                        ;       ( 2 goes to KEY_2_FUNC. etc )
                                        ;
KEY_FREQ:		BTFSC	SWEEP_ACTIVE	;
			   _GOTO	KEYPAD_EXIT		;		Skip Frequency entry if sweep active.		
		       	BTFSS   EN_MEM_SEL      ;       Is the select power meter mem function
                BTFSC   MEM_ACTIVE      ;       active ?, or is the MEM function active ?
               _GOTO    KEYPAD_EXIT     ;       Y. All done.
                                        ;
                BCF     TIMEOUT         ;       Clear the display timeout flag.
                BSF     KEYPAD_FLAG     ;       Indicate that the keypad is in use.
               _CALL    PUSH_ARG2       ; (1)   Save ARG2, used if disp timeout reached.
               _CALL    CLEAR_ARG2      ; (1)   ARG2 will hold the number as it is built.
               _CALL    DISPLAY_FREQ    ; (4)   Display the number on line 2.
                                        ;
DIGIT_0_LOOP:   BTFSS   TIMEOUT         ;       Has the display timeout been reached ?
               _GOTO    NOT_TIME_OUT_0  ;
               _CALL    PULL_ARG2       ; (1)   Y. Recover the original ARG2,
               _GOTO    KEYPAD_EXIT     ;          and exit.
NOT_TIME_OUT_0:_CALL    CHK_KEY_BRIEF   ; (1)
                BTFSS   PRESSED         ;       Has next keypad button been pressed ?
               _GOTO    DIGIT_0_LOOP    ;       N. Wait for a digit.
                                        ;
                MOVLW   2               ;       Allow 2 digits before the decimal point.
                MOVWF   DIGIT_CNT       ;
               _GOTO    LOOP_IN         ;       Insert the digit.

;-----------------------------------------------------------------------------------------
;               Wait for a keypad button to be pressed, then insert the digit into ARG2.
;-----------------------------------------------------------------------------------------

UPDATE_L2:     _CALL    DISPLAY_FREQ    ; (4)   Display the number on line 2.
                                        ;
DIGIT_LOOP:     BTFSS   TIMEOUT         ;       Has the display timeout been reached ?
               _GOTO    NOT_TIME_OUT    ;
               _CALL    PULL_ARG2       ; (1)   Y. Recover the original ARG2,
               _GOTO    KEYPAD_EXIT     ;          and exit.
NOT_TIME_OUT:  _CALL    CHK_KEY_BRIEF   ; (1)
                BTFSS   PRESSED         ;       Any keypad button pressed ?
               _GOTO    DIGIT_LOOP      ;       N. Wait for a digit.
                BTFSC   KEYPAD_NUM,6    ;       Was it the # button ?
               _GOTO    KEYPAD_DONE     ;       Y. All done.
LOOP_IN:        BTFSS   KEYPAD_NUM,5    ;
               _GOTO    NOT_DP          ;       Was it the * button ?
                BTFSC   KEYPAD_DP       ;       Y. Has the DP already been pressed.
               _GOTO    DIGIT_LOOP      ;          Y. Ignore the *. Wait for next digit.
                BSF     KEYPAD_DP       ;          N. Set the DP flag.
                MOVLW   6               ;
                MOVWF   DIGIT_CNT       ;             Allow 6 Digits after the DP.
               _GOTO    UPDATE_L2       ;             Disp the num, wait for next digit.
NOT_DP          MOVF    DIGIT_CNT,W     ;
                BTFSC   ZERO            ;       Have all digits been entered ?
               _GOTO    DIGIT_LOOP      ;       Y. Wait for the #.

;-----------------------------------------------------------------------------------------
;               Insert the digit into ARG2.
;-----------------------------------------------------------------------------------------

                DECF    DIGIT_CNT,W     ;       Get the digit-1 count. (digit position)
                BTFSC   KEYPAD_DP       ;       Place the digit before the DP ?
               _GOTO    POST_DIGIT      ;
PRE_DIGIT:     _CALL    CLEAR_ARG3      ; (1)   Y. Make room for new digit by shifting
                MOVLW   10              ;          all digits left one position.
                MOVWF   ARG3_0          ;          (ARG2 x 10)
               _CALL    MULTIPLY        ; (3)
                MOVLW   ARG1_3          ;
               _CALL    COPY_TO_ARG2    ; (1)      ARG2 = ARG2 x 10.
                MOVLW   6               ;          Set the digit position. 6 = MHz
POST_DIGIT:    _CALL    TEN_TO_POWER_W  ; (4)   Convert digit position to a power of 10.
                MOVF    KEYPAD_NUM,W    ;       Get the keypad digit.
                ANDLW   B'00001111'     ;       Strip off any high order bits.
               _CALL    ARG3_TIMES_W    ; (3)   Multiply it by the power of ten.
               _CALL    ADD             ; (3)   Add in the new digit.
                BTFSC   KEYPAD_DP       ;       Are we before the DP ?
               _GOTO    SKIP_0_CHK      ;
               _CALL    CLEAR_ARG3      ; (1)
               _CALL    COMPARE         ; (2)   Y. Is the result = 0 ?
                BTFSS   ZERO            ;          Y. Ignore the leading zero digits.
SKIP_0_CHK:     DECF    DIGIT_CNT,F     ;       N. Dec the digit count.
               _GOTO    UPDATE_L2       ;       Display the num, then wait for next digit.

;-----------------------------------------------------------------------------------------
;               The # button has been pressed.
;-----------------------------------------------------------------------------------------

KEYPAD_DONE:   _CALL    PULL_ARG3       ; (1)   Discard the original ARG2.
                BTFSC   KEYPAD_MAIN     ;       Does the EEPROM need updating ?
KEYPAD_END:     BSF     EEPROM_UPDATE   ;       Y. Flag that EEPROM needs updating.
KEYPAD_EXIT:    CLRF    KEYPAD_FLAGS    ;       Clear all the keypad flags.
                RETURN                  ;

;*****************************************************************************************
;
;      NAME:    CHK_DDS
;
;   PURPOSE:    Update the AD9850/51 DDS chip if the frequency has changed.
;
;               Sends a 40 bit control word to the AD9850/51 DDS chip in serial form.
;               32 bit frequency + 8 bit refclk/phase/power down.
;               LSB of frequency is sent first.
;
;     INPUT:    ARG2 = Frequency.
;               ARG3 = DDS cal constant.  (2^56 / XTAL FREQ)
;
;    OUTPUT:    None.
;               (ARG2 and ARG3 unchanged)
;
; STACK USE:    3
;
;*****************************************************************************************

CHK_DDS:       _CALL    PUSH_ARG3       ; (1)   Dont destroy ARG3.
                MOVLW   DDS_OLD         ;       Get the previous DDS freq.
               _CALL    COPY_TO_ARG3    ; (1)
                                        ;
               _CALL    COMPARE         ; (2)   Is it the same as the current freq ?
                BTFSC   ZERO            ;       Y. Do not update the DDS chip.
               _GOTO    PULL_ARG3       ; (0)      Restore ARG3 then return via "GOTO".
                                        ;
               _CALL    PULL_ARG3       ; (1)   N. Recover the original value of ARG3.
                MOVLW   DDS_OLD         ;
               _CALL    COPY_FROM_ARG2  ; (1)      Save it as the new DDS freq.
                                        ;
SET_DDS:       _CALL    MULTIPLY        ; (3)      Multiply DDS freq by cal constant.
                                        ;          (result ARG1_6..3)
                                        ;
SEND_DDS_WORD:  CLRF    ARG1_7          ;       Clear the refclk/phase/power down byte.
 IF AD9851_CHIP                         ;       Is the assembly for the AD9851 ?
                MOVLW   DDS_OPT         ;       Y. Get the address of DDS_OPT.
                MOVWF   FSR             ;
                BTFSC   INDF,X6_REF     ;          Is the x6 ref in use ?
                BSF     ARG1_7,0        ;          Y. Enable the 9851 x6 ref.
 ENDIF                                  ;
DDS_SEND:       MOVLW   40              ;
                MOVWF   COUNT_1         ;       40 bits to send.
                BCF     GIE             ;       Disable interrupts.
NEXT_BIT:       RRF     ARG1_7,F        ;       Shift right the 40 bits,
                RRF     ARG1_6,F        ;       to move the LSB into the carry.
                RRF     ARG1_5,F        ;
                RRF     ARG1_4,F        ;
                RRF     ARG1_3,F        ;
                BTFSS   CARRY           ;       Is the carry bit 1 ?
                BCF     DDS_DATA        ;       N. Send a 0.
                BTFSC   CARRY           ;       Is the carry bit 1 ?
                BSF     DDS_DATA        ;       Y. Send a 1.
                BSF     DDS_CLOCK       ;       Toggle the DDS word load clock pin.
                BCF     DDS_CLOCK       ;
                DECFSZ  COUNT_1,F       ;       Have all 40 bits been sent?
               _GOTO    NEXT_BIT        ;       N. Keep going.
                BSF     DDS_LOAD        ;       Toggle the DDS frequency update pin.
                BCF     DDS_LOAD        ;
                BSF     GIE             ;       Enable interrupts.
                RETURN                  ;

;*****************************************************************************************
;
;      NAME:    FREQ_TO_LCD
;
;   PURPOSE:    Update the DDS chip if required, then display it on the LCD.
;
;     INPUT:    ARG2 = frequency.
;
;    OUTPUT:    None.
;               (ARG1 & ARG3 destroyed)
;
; STACK USE:    4
;
;*****************************************************************************************
;
;      NAME:    DISPLAY_FREQ
;
;   PURPOSE:    Display the frequency in ARG2 on the LCD.
;               Including negative sign, decimal point and leading zero blanking,
;               and MHz text. ( # if keypad is in use)
;               The DP_POSITION defines the position of the decimal point.
;               ie, 2 =  1 chr after the DP.
;
;     INPUT:    ARG2 = frequency.
;
;    OUTPUT:    None.
;
; STACK USE:    3
;
;*****************************************************************************************

FREQ_TO_LCD:    MOVLW   DDS_CAL         ;
               _CALL    RAM_TO_ARG3     ; (2)   Get the DDS CAL constant.
                BTFSC   FAST_WOBBLE      ;       Are we doing a fast sweep.
               _GOTO    SET_DDS         ;       Y. Always Update the DDS chip.
                                        ;          Return via "GOTO".
		       _CALL    CHK_DDS         ; (4)   Update the DDS chip if required.
DISPLAY_FREQ:   MOVLW   8               ;
                MOVWF   DIGITS          ;       Set the number of digits to display.
                MOVLW   H'C0'           ;       Position the cursor, line 2 pos 1.
              P_CALL    LCD_CMD         ; (2)
                                        ;
                MOVLW   "F"             ;
                BTFSS   SF_DISPLAYED    ;       Is start or finish frequency displayed.
                MOVLW   "S"             ;
                BTFSS   SWEEP_ACTIVE    ;
               _GOTO    DISPLAY_IT      ;
                MOVLW   "^"             ;       '^'
                BTFSS   SWEEP_DIR       ;
                MOVLW   "v"             ;       'v'
                BTFSC   SWEEP_STOPPED   ;
                MOVLW   "-"             ;       '-'
DISPLAY_IT:     BTFSC   CAL_FLAG        ;       Was this function called by the cal screen ?
               _GOTO    Y_CAL_SCRN      ;
              P_CALL    LCD_CHR         ; (2)   N. Display the character.
               _GOTO    END_CAL_TEST    ;
Y_CAL_SCRN:     INCF    DIGITS,F        ;       Y. One extra digit to display.
END_CAL_TEST: P_CALL    LCD_SPACE       ; (2)
              P_CALL    LCD_SPACE       ; (2)
                                        ;
                MOVLW   7               ;       Set the position of the DP.
               _CALL    DISPLAY_NUMBER  ; (3)   Display the number.
                                        ;
              P_CALL    LCD_SPACE       ; (2)   Display a space.
               _MOVLW   MHZ             ;       Is the keypad in use ?
                BTFSC   KEYPAD_FLAG     ;       N. Display 'MHz'.
               _MOVLW   KEY_TXT         ;       Y. Display a # after the freq.
              P_CALL____LCD_TEXT        ; (3)
                RETURN                  ;

;-----------------------------------------------------------------------------------------

DISP_NUM_DP2:   MOVWF   DIGITS          ;       Set the number of digits to display.
                MOVLW   2               ;
DISPLAY_NUMBER: MOVWF   DP_POSITION     ;       Set the position of the DP.
                BCF     NEGATIVE        ;       Clear the negative flag.
                BTFSS   ARG2,7          ;       Is ARG2 a negative number?
               _GOTO    DISP_NUMBER     ;       N. Just continue.
                BSF     NEGATIVE        ;       Y. Set the negative flag.
               _CALL    NEG_ARG2        ; (1)      Convert it to a + number.
DISP_NUMBER:   _CALL    BIN_TO_DEC      ; (2)   Convert the number to ASCII decimal.
                                        ;
                COMF    DIGITS,W        ;
                ADDLW   ARG1_0+2        ;       Point to first chr to display.
                MOVWF   FSR             ;
                MOVF    DIGITS,W        ;       Set the number of digits to display.
                MOVWF   COUNT_1         ;
                                        ;
DISP_LOOP:      MOVF    INDF,W          ;       Get char pointed to by FSR.
              P_CALL    LCD_CHR         ; (2)   Display the char.
                MOVF    DP_POSITION,W   ;       Get the position of the DP.
                SUBWF   COUNT_1,W       ;       Y. Is it time to insert a decimal point?
                MOVLW   H'A5'           ;
                BTFSS   ZERO            ;
               _GOTO    NO_DP           ;
              P_CALL    LCD_CHR         ; (2)      Y. Display a decimal point.
NO_DP:          INCF    FSR,F           ;       Inc the pointer.
                DECFSZ  COUNT_1,F       ;       All done ?
               _GOTO    DISP_LOOP       ;       N. Continue.
                                        ;
                BTFSC   NEGATIVE        ;       Was ARG2 negative ?
               _CALL    NEG_ARG2        ; (1)   Y. Return it to a negative number.
                BCF     NEGATIVE        ;       Clear the negative flag.
                RETURN                  ;

;-----------------------------------------------------------------------------------------

DISP_4_DIGITS:  MOVLW   4               ;       4 digits to display.
                MOVWF   COUNT_1         ;
                MOVLW   ARG1_3          ;       Starting at ARG1_3.
                MOVWF   FSR             ;
                MOVLW   H'8C'           ;       Position the cursor, line 1 pos 14.
              P_CALL    LCD_CMD         ; (2)
               _GOTO    DISP_LOOP       ;       Display the digits.

;*****************************************************************************************
;               Start of primitive subroutines
;               RAM variables usage for each subroutine is noted in the comments.
;*****************************************************************************************

;*****************************************************************************************
;
;      NAME:    BIN_TO_DEC
;
;   PURPOSE:    Convert a 32 bit binary number to a 10 digit ASCII decimal number.
;               First the 32 bit binary number is converted to 10 digit decimal.
;               It is then converted to ASCII, including decimal point, leading zero
;               blanking and negative sign (if NEGATIVE flag set), ready for display on
;               the LCD.
;               (if FULL_BLANKING flag is set, dont stop blanking at the decimal point.)
;               The DP_POSITION defines the position of the decimal point.
;               ie, 2 =  1 chr after the DP.
;
;     INPUT:    ARG2 = frequency. (+ numbers only )
;
;    OUTPUT:    10 digit ASCII decimal number in ARG1_9..0. (MSD..LSD)
;               (ARG2 unchanged)
;
; VARIABLES:    FSR, COUNT_1, COUNT_2, TEMP_1.
;
; STACK USE:    1
;
;*****************************************************************************************

BIN_TO_DEC_FB:  BSF     FULL_BLANKING   ;       Enable full leading blanking.
BIN_TO_DEC:    _CALL    CLEAR_ARG1      ; (1)
                MOVLW   32              ;
                MOVWF   COUNT_2         ;       32 bits to process.
               _GOTO    SHIFT_TO_DEC    ;
                                        ;
LOOP32:        _CALL    SET_ARG1        ; (1)   Set FSR = ARG1_9, COUNT_1 = 10.
DEC_LOOP:       MOVLW   3               ;
                ADDWF   INDF,W          ;       If num + 3 > 7 then num = num + 3.
                MOVWF   TEMP_1          ;       Add 3 to num. (adds 6, after next shift)
                BTFSS   TEMP_1,3        ;       Is result > 7 ? (bit 3 set)
               _GOTO    NO_ADJ          ;
                ADDLW   B'01111000'     ;       Y. Move bit 3 to bit 7.
                MOVWF   INDF            ;          Put the number back in the buffer.
NO_ADJ:         INCF    FSR,F           ;       Point to next byte in buffer.
                DECFSZ  COUNT_1,F       ;
               _GOTO    DEC_LOOP        ;       Loop until all 11 bytes done.
                                        ;
SHIFT_TO_DEC:   RLF     ARG2_3,W        ;       Shift 32 bit number left by one bit
                RLF     ARG2_0,F        ;       into dec buffer.
                RLF     ARG2_1,F        ;
                RLF     ARG2_2,F        ;       After 40 shifts ARG2 unchanged.
                RLF     ARG2_3,F        ;       MSD of ARG2
                RLF     ARG1_0,F        ;       LSD of ARG1
                RLF     ARG1_1,F        ;
                RLF     ARG1_2,F        ;
                RLF     ARG1_3,F        ;
                RLF     ARG1_4,F        ;
                RLF     ARG1_5,F        ;
                RLF     ARG1_6,F        ;
                RLF     ARG1_7,F        ;
                RLF     ARG1_8,F        ;
                RLF     ARG1_9,F        ;       MSD of ARG1.
                DECFSZ  COUNT_2,F       ;
               _GOTO    LOOP32          ;       Loop until all 32 bits processed.
                                        ;
                BCF     LEADING_FLAG    ;       Blank leading zeros until flag is set.
                                        ;
               _CALL    SET_ARG1        ; (1)   Set FSR = ARG1_9, COUNT_1 = 10.
ASCII_LOOP:     BTFSS   KEYPAD_DP       ;       Has * button on the keypad been pressed ?
               _GOTO    NON_KEYPAD_DP   ;
                MOVF    COUNT_1,W       ;       Y. Blank any digits after the DP that
                SUBWF   DIGIT_CNT,W     ;          have not been entered yet.
                BTFSC   CARRY           ;
               _GOTO    BLANK_IT        ;
NON_KEYPAD_DP:  MOVF    INDF,W          ;       Get the number.
                BTFSC   ZERO            ;       Is it = 0 ?
               _GOTO    BLANK           ;
NO_BLANK        ADDLW   H'30'           ;       N. Convert it to ASCII.
                MOVWF   INDF            ;
                BSF     LEADING_FLAG    ;          Set the leading blank flag,
               _GOTO    NUM_DONE        ;          following zeros will not be blanked.
BLANK:          BTFSC   LEADING_FLAG    ;       Y. Is the leading blank flag set.
               _GOTO    NO_BLANK        ;          Y. Convert it to ASCII.
                MOVF    FSR,W           ;          N. Save the location so that a - sign
                MOVWF   COUNT_2         ;             can be inserted.
BLANK_IT:       MOVLW   ' '             ;
                MOVWF   INDF            ;             Blank the byte.
NUM_DONE:       INCF    FSR,F           ;       Point to the next byte.
                INCF    DP_POSITION,W   ;       Get the position of the DP.
                BTFSC   FULL_BLANKING   ;
                MOVLW   1               ;
                SUBWF   COUNT_1,W       ;       Time to cancel leading blanking?
                BTFSC   ZERO            ;
                BSF     LEADING_FLAG    ;       Y. Always display char before DP.
                DECFSZ  COUNT_1,F       ;
               _GOTO    ASCII_LOOP      ;
                                        ;
                BCF     FULL_BLANKING   ;       Clear the full blanking flag
                MOVF    COUNT_2,W       ;       Get position of last blank.
                MOVWF   FSR             ;
                MOVLW   '-'             ;       Get a - sign.
                BTFSC   NEGATIVE        ;       Was this a negative number ?
                MOVWF   INDF            ;       Y. Place - sign in front of the number.
                RETURN                  ;

;-----------------------------------------------------------------------------------------

SET_ARG1:       MOVLW   ARG1_9          ;       Set FSR = ARG1_9, COUNT_1 = 10.
                MOVWF   FSR             ;
                MOVLW   10              ;
                MOVWF   COUNT_1         ;
                RETURN                  ;

;*****************************************************************************************
;
;     NAMES:    ADD                     ARG2 = ARG2 + ARG3.     (all routines are 32 bit)
;
; VARIABLES:    FSR, COUNT_1, TEMP_1, TEMP_2, TEMP_3.
;
; STACK USE:    2
;
;*****************************************************************************************

ADD:           _CALL    ARG2_TO_ARG1    ; (1)   ARG1 = ARG2.
               _CALL    ADD_ARG1_ARG3   ; (2)   ARG1 = ARG2 + ARG3.
               _GOTO    ARG1_TO_ARG2    ; (0)   ARG2 = ARG2 + ARG3. Return via "GOTO".

;*****************************************************************************************
;
;     NAMES:    SUB                     ARG2 = ARG2 - ARG3.     (all routines are 32 bit)
;
; VARIABLES:    FSR, COUNT_1, TEMP_1, TEMP_2, TEMP_3.
;
; STACK USE:    1
;
;*****************************************************************************************

SUB:           _CALL    ARG2_TO_ARG1    ; (1)   ARG1 = ARG2.
               _CALL    SUB_ARG1_ARG3   ; (1)   ARG1 = ARG2 - ARG3.
               _GOTO    ARG1_TO_ARG2    ; (0)   ARG2 = ARG2 - ARG3. Return via "GOTO".

;*****************************************************************************************
;
;     NAMES:    ADD_ARG1_ARG3           ARG1 = ARG1 + ARG3.     (all routines are 32 bit)
;
; VARIABLES:    FSR
;
; STACK USE:    1
;
;*****************************************************************************************

ADD_ARG1_ARG3: _CALL    NEG_ARG3        ; (1)   ARG3 = -ARG3.
               _CALL    SUB_ARG1_ARG3   ; (1)   ARG1 = ARG2 + ARG3.
               _GOTO    NEG_ARG3        ; (0)   ARG3 = ARG3. Return via "GOTO".

;*****************************************************************************************
;
;     NAMES:    SUB_ARG1_ARG3           ARG1 = ARG1 - ARG3.     (all routines are 32 bit)
;
; VARIABLES:    FSR
;
; STACK USE:    0
;
;*****************************************************************************************

SUB_ARG1_ARG3:  MOVF    ARG3_0,W        ;
                SUBWF   ARG1_4,F        ;       Sub the 1st (LSD) bytes.
                MOVF    ARG3_1,W        ;
                BTFSS   CARRY           ;
                INCFSZ  ARG3_1,W        ;       If there was a carry, inc the next byte.
                                        ;
                SUBWF   ARG1_5,F        ;       Sub the 2nd bytes.
                MOVF    ARG3_2,W        ;
                BTFSS   CARRY           ;
                INCFSZ  ARG3_2,W        ;       If there was a carry, inc the next byte.
                                        ;
                SUBWF   ARG1_6,F        ;       Sub the 3rd bytes.
                MOVF    ARG3_3,W        ;
                BTFSS   CARRY           ;
                INCFSZ  ARG3_3,W        ;       If there was a carry, inc the next byte.
                                        ;
                SUBWF   ARG1_7,F        ;       Sub the 4th bytes.
                RETURN                  ;

;*****************************************************************************************
;
;     NAMES:    NEG_ARG2                ARG2 = -ARG2.           (all routines are 32 bit)
;               NEG_ARG3                ARG3 = -ARG3.
;
; VARIABLES:    FSR.
;
; STACK USE:    0
;
;*****************************************************************************************

NEG_ARG3:       MOVLW   ARG3            ;
               _GOTO    NEG_X           ;
NEG_ARG2:       MOVLW   ARG2            ;
NEG_X:          MOVWF   FSR             ;       Place address of ARG in FSR
                COMF    INDF,F          ;       Compliment the arg.
                INCF    FSR,F           ;
                COMF    INDF,F          ;
                INCF    FSR,F           ;
                COMF    INDF,F          ;
                INCF    FSR,F           ;
                COMF    INDF,F          ;
                                        ;
                INCFSZ  INDF,F          ;       Increment the arg.
                RETURN                  ;
                DECF    FSR,F           ;
                INCFSZ  INDF,F          ;
                RETURN                  ;
                DECF    FSR,F           ;
                INCFSZ  INDF,F          ;
                RETURN                  ;
                DECF    FSR,F           ;
                INCF    INDF,F          ;
                RETURN                  ;       ARGx = -ARGx.

;*****************************************************************************************
;
;     NAMES:    CLEAR_ARG1              ARG1 = 0.
;               CLEAR_ARG2              ARG2 = 0.
;               CLEAR_ARG3              ARG3 = 0.
;               CLEAR_ARG4              ARG4 = 0.
;               CLEAR_ARG5              ARG5 = 0.
;
; VARIABLES:    FSR, COUNT_1.
;
; STACK USE:    0
;
;*****************************************************************************************

CLEAR_ARG1:     MOVLW   ARG1_9          ;       Clear all 10 bytes.
                MOVWF   FSR             ;
                MOVLW   10              ;
               _GOTO    CLEAR_BYTES     ;

CLEAR_ARG5:     MOVLW   ARG5            ;
               _GOTO    CLEAR           ;
CLEAR_ARG4:     MOVLW   ARG4            ;
               _GOTO    CLEAR           ;
CLEAR_ARG3:     MOVLW   ARG3            ;
               _GOTO    CLEAR           ;
CLEAR_ARG2:     MOVLW   ARG2            ;
CLEAR:          MOVWF   FSR             ;
                MOVLW   4               ;
CLEAR_BYTES:    MOVWF   COUNT_1         ;       Save the count.
CLEAR_LOOP:     CLRF    INDF            ;       Clear the RAM byte.
                INCF    FSR,F           ;       Inc the RAM address pointer.
                DECFSZ  COUNT_1,F       ;       Dec the count.
               _GOTO    CLEAR_LOOP      ;       Loop until all bytes cleared.
                RETURN                  ;

;*****************************************************************************************
;
;      NAME:    COMPARE
;
;   PURPOSE:    32 bit compare.  ARG2 - ARG3
;
;     INPUT:    32 bit argment ARG2.
;               32 bit argment ARG3.
;
;    OUTPUT:    None.
;               Carry flag clear (borrow) if result was negative.
;               Zero flag set if equal.
;               (ARG2 & ARG3 unchanged)
;
; VARIABLES:    FSR, COUNT_1, TEMP_1, TEMP_2, TEMP_3.
;
; STACK USE:    1
;
;*****************************************************************************************

COMPARE:       _CALL    OFFSET_ARGS     ; (1)   Add a large offset to ARG2 & ARG3.
                                        ;       (makes sure we are dealing with)
               _CALL    ARG2_TO_ARG1    ; (1)   (positive numbers only         )
               _CALL    SUB_ARG1_ARG3   ; (1)   Do the compare. Result in ARG1.
OFFSET_ARGS:    MOVLW   B'10000000'     ;       Remove the offset from ARG2 and ARG3.
                XORWF   ARG2_3,F        ;
                XORWF   ARG3_3,F        ;
                MOVF    ARG1_7,W        ;       Get the result for zero check.
                IORWF   ARG1_6,W        ;       (sets zero flag if result was 0)
                IORWF   ARG1_5,W        ;
                IORWF   ARG1_4,W        ;
                RETURN                  ;

;*****************************************************************************************
;
;      NAME:    MINIMUM
;
;   PURPOSE:    32 bit minimum. Return the smallest of two argments.
;
;       INPUT:  32 bit argment in ARG2.
;               32 bit argment in ARG3.
;
;       OUTPUT: The smallest argment is returned in ARG2.
;               (ARG3 unchanged)
;
; VARIABLES:    FSR, COUNT_1, TEMP_1, TEMP_2, TEMP_3.
;
; STACK USE:    2
;
;*****************************************************************************************

CHK_MAX:        MOVLW   ARG5            ;
               _CALL    COPY_TO_ARG3    ; (1)   Make sure we dont go above ARG5.

MINIMUM:       _CALL    COMPARE         ; (2)   ARG2 - ARG3
                BTFSC   CARRY           ;       Is ARG2 >= ARG3 ?
                BSF     SWEEP_CHANGE    ;       Y. Set the change direction flag.
                BTFSC   CARRY           ;
               _CALL    ARG3_TO_ARG2    ; (1)      Set ARG2 the same as ARG3.
                RETURN                  ;       N. No change required.

;*****************************************************************************************
;
;      NAME:    MAXIMUM
;
;   PURPOSE:    32 bit maximum. Return the largest of two argments.
;
;       INPUT:  32 bit argment in ARG2.
;               32 bit argment in ARG3.
;
;       OUTPUT: The largest argment is returned in ARG2.
;               (ARG3 unchanged)
;
; VARIABLES:    FSR, COUNT_1, TEMP_1, TEMP_2, TEMP_3.
;
; STACK USE:    2
;
;*****************************************************************************************

CHK_MIN:        MOVLW   ARG4            ;
               _CALL    COPY_TO_ARG3    ; (1)   Make sure we dont go below ARG4.

MAXIMUM:       _CALL    COMPARE         ; (2)   ARG2 - ARG3
                BTFSS   ZERO            ;
                BTFSS   CARRY           ;       Is ARG2 <= ARG3 ?
                BSF     SWEEP_CHANGE    ;       Y. Set the change direction flag.
                BTFSS   CARRY           ;       Is ARG2 > ARG3 ?
               _CALL    ARG3_TO_ARG2    ; (1)   N. Set ARG2 the same as ARG3.
                RETURN                  ;       Y. No change required.

;*****************************************************************************************
;
;      NAME:    MULTIPLY
;
;   PURPOSE:    32 bit * 32 bit unsigned multiply.    (ARG1 = ARG2 x ARG3)
;
;     INPUT:    ARG2  (+ or - numbers )
;               ARG3  (+ numbers only )
;
;    OUTPUT:    ARG1_7..0  (MSD..LSD)
;               (ARG2 and ARG3 unchanged)
;
; VARIABLES:    FSR, COUNT_1, COUNT_3, TEMP_1, TEMP_2, TEMP_3.
;
; STACK USE:    2
;
;*****************************************************************************************

MULTIPLY:      _CALL    PUSH_ARG2       ; (1)   Save ARG2.
                BTFSC   ARG2,7          ;       Is ARG2 negative ?
               _CALL    NEG_ARG2        ; (1)   Y. Convert to positive.
                                        ;
               _CALL    CLEAR_ARG1      ; (1)
                MOVLW   32              ;       32 bits to multiply.
                MOVWF   COUNT_3         ;
MLOOP:          RRF     ARG2_0,W        ;       Rotate the multiplier right 1 bit
                RRF     ARG2_3,F        ;       into the carry bit.
                RRF     ARG2_2,F        ;       ( after 32 rotates ARG2 will be )
                RRF     ARG2_1,F        ;       ( returned to its original value )
                RRF     ARG2_0,F        ;
                                        ;
                BTFSC   CARRY           ;       If carry bit is set, add ARG3 to partial
               _CALL    ADD_ARG1_ARG3   ; (2)   product. ARG1_7..4 = ARG1_7..4 + ARG3_3..0
                                        ;
                RRF     ARG1_7,F        ;       Right shift the partial product.
                RRF     ARG1_6,F        ;
                RRF     ARG1_5,F        ;
                RRF     ARG1_4,F        ;
                RRF     ARG1_3,F        ;
                RRF     ARG1_2,F        ;
                RRF     ARG1_1,F        ;
                RRF     ARG1_0,F        ;
                                        ;
                DECFSZ  COUNT_3,F       ;       All 40 bits done ?
               _GOTO    MLOOP           ;       N. Keep going.
               _GOTO    PULL_ARG2       ; (0)   Y. All done. Get the original ARG2 back.
                                        ;          Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    DIVIDE
;
;   PURPOSE:    32 bit / 32 bit unsigned divide.    (ARG2 = ARG2 / ARG3)
;
;     INPUT:    ARG2 (+ numbers only )
;               ARG3 (+ numbers only )
;
;    OUTPUT:    ARG2 = result
;               ARG1 = remainder (ARG1_9..5  MSD..LSD)
;               (ARG3 unchanged)
;
; STACK USE:    2
;
;*****************************************************************************************
;
;      NAME:    TWO_56_DIVIDE
;
;   PURPOSE:    ARG2 = 2^56 / ARG2      (ARG3 destroyed)
;
; VARIABLES:    FSR, COUNT_1, COUNT_3, TEMP_1, TEMP_2, TEMP_3.
;
; STACK USE:    2
;
;*****************************************************************************************

TWO_56_DIVIDE: _CALL    ARG2_TO_ARG3    ; (1)
               _CALL    CLEAR_ARG2      ; (1)
               _CALL    CLEAR_ARG1      ; (1)
                BSF     ARG1_7,0        ;       Set ARG1,ARG2 to 2^56.
               _GOTO    DIV             ;

DIVIDE:        _CALL    CLEAR_ARG1      ; (1)
DIV:            MOVLW   32              ;       32 bits to divide.
                MOVWF   COUNT_3         ;
                                        ;
DLOOP:          BCF     CARRY           ;
                RLF     ARG2_0,F        ;       Rotate dividend left 1 bit position.
                RLF     ARG2_1,F        ;
                RLF     ARG2_2,F        ;
                RLF     ARG2_3,F        ;
                RLF     ARG1_4,F        ;       Rotate remainder left 1 bit position.
                RLF     ARG1_5,F        ;
                RLF     ARG1_6,F        ;
                RLF     ARG1_7,F        ;
                                        ;
                BTFSC   CARRY           ;       Invert carry and exclusive or with the
               _GOTO    CLR_LSB         ;       MSB of the divisor then move this bit
                BTFSS   ARG3_3,7        ;       into the LSB of the dividend.
                INCF    ARG2_0,F        ;
               _GOTO    CONT            ;
CLR_LSB:        BTFSC   ARG3_3,7        ;
                INCF    ARG2_0,F        ;
                                        ;
CONT:           BTFSS   ARG2_0,0        ;       Is the LSB of the dividend =0 ?
               _CALL    ADD_ARG1_ARG3   ; (2)   Y. ARG1_4..0 = ARG1_4..0 + ARG3_4..0
                BTFSC   ARG2_0,0        ;
               _CALL    SUB_ARG1_ARG3   ; (1)   N. ARG1_4..0 = ARG1_4..0 - ARG3_4..0
                                        ;
                DECFSZ  COUNT_3,F       ;       Do 32 times.
               _GOTO    DLOOP           ;
                                        ;
                BCF     CARRY           ;
                RLF     ARG2_0,F        ;       Shift lower 32 bits of dividend 1 bit
                RLF     ARG2_1,F        ;       Position left
                RLF     ARG2_2,F        ;
                RLF     ARG2_3,F        ;
                BTFSC   ARG1_7,7        ;       Exclusive or the inverse of the MSB of
               _GOTO    CHK_1           ;       the dividend with the MSB of the divisor
                BTFSS   ARG3_3,7        ;       store in the LSB of the dividend.
                INCF    ARG2_0,F        ;
               _GOTO    DIV_END         ;
CHK_1:          BTFSC   ARG3_3,7        ;
                INCF    ARG2_0,F        ;
DIV_END:        RETURN                  ;

;*****************************************************************************************
;
;      NAME:    CON_TO_ARG3
;
;   PURPOSE:    Place the 2 byte constant pointed to by W, into ARG3.
;
;     INPUT:    W = Pointer.
;
;    OUTPUT:    ARG3 = Requested constant
;
; VARIABLES:    TEMP_1.
;
; STACK USE:    1
;
;*****************************************************************************************

SWEEP_TO_ARG3:  MOVWF   TEMP_1          ;       Save the pointer to the required const.
                ADDWF   TEMP_1,W        ;       Multiply the pointer by 2.

CON_TO_ARG3:    MOVWF   TEMP_1          ;       Save the pointer to the required const.
                CLRF    ARG3_3          ;       Clear the msd's of ARG3.
                CLRF    ARG3_2          ;
               _CALL    CONST           ; (1)   Get the 1st byte from the constant table.
                MOVWF   ARG3_1          ;       Place the byte in ARG3.
                INCF    TEMP_1,W        ;
               _CALL    CONST           ; (1)   Get the 2nd byte from the constant table.
                MOVWF   ARG3_0          ;       Place the byte in ARG3.
                RETURN                  ;

;*****************************************************************************************
;
;      NAME:    ARG3_TIMES_W
;
;   PURPOSE:    Calculate ARG3 x W (0 to 255 times)
;
;     INPUT:    W = Multiplier.
;
;    OUTPUT:    ARG3 = ARG3 x W.
;
;  RAM USED:    FSR, COUNT_0, COUNT_1, TEMP_1, TEMP_2, TEMP_3.
;
; STACK USE:    2
;
;*****************************************************************************************

ARG3_TIMES_W:   ADDLW   1               ;       W = multiplier.
                MOVWF   COUNT_0         ;       Save the multiplier + 1.
               _CALL    CLEAR_ARG1      ; (1)   ARG1 = 0.
               _GOTO    X_LOOP_IP       ;
X_LOOP:        _CALL    ADD_ARG1_ARG3   ; (2)   Add ARG3 to ARG1, W times.
X_LOOP_IP:      DECFSZ  COUNT_0,F       ;
               _GOTO    X_LOOP          ;
                MOVLW   ARG1            ;       Place the result into ARG3.
               _GOTO    COPY_TO_ARG3    ; (0)   Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    TEN_TO_POWER_W
;                            (W)
;   PURPOSE:    Calculate 10^
;
;     INPUT:    W = Power.
;                         (W)
;    OUTPUT:    ARG3 = 10^
;
; VARIABLES:    FSR, COUNT_0, COUNT_1. COUNT_3, TEMP_1, TEMP_2, TEMP_3.
;
; STACK USE:    3
;
;*****************************************************************************************

TEN_MEG:        MOVLW   7               ;
TEN_TO_POWER_W: ADDLW   1               ;
                MOVWF   COUNT_3         ;       Save the power + 1.
               _CALL    CLEAR_ARG3      ; (1)
                INCF    ARG3_0,F        ;       Set ARG3 to 1.
               _GOTO    END_TEST        ;
POWER_LOOP:     MOVLW   10              ;
               _CALL    ARG3_TIMES_W    ; (3)
END_TEST:       DECFSZ  COUNT_3,F       ;       Keep mult by ten until done.
               _GOTO    POWER_LOOP      ;
                RETURN                  ;

;*****************************************************************************************
;
;     NAMES:    CHK_KEY_BRIEF   Return a true flag, if any keypad button is pressed.
;               CHK_MEM_BRIEF   Return a true flag in PRESSED, if the
;               CHK_CAL_BRIEF   button was pressed briefly.
;
;     INPUT:    None.
;
;    OUTPUT:    None. (PRESSSED flag set if button pressed)
;
; VARIABLES:    FSR, TEMP_1.
;
; STACK USE:    0
;
;*****************************************************************************************


CHK_KEY_BRIEF:  BTFSC   KEYPAD_PRESSED  ;       Was a button pressed last time ?
               _GOTO    WAIT_RELEASE    ;
                MOVLW   MIN_ON          ;       N. Use the minimum on time limit.
                SUBWF   KEYPAD_CNT,W    ;          Has any button been pressed A while ?
                BTFSC   CARRY           ;
                BSF     KEYPAD_PRESSED  ;          Y. Set the keypad pressed flag.
               _GOTO    RETURN_FLAG     ;          Return the pressed flag.


WAIT_RELEASE:   MOVF    KEYPAD_CNT,W    ;       Y. Clear the keypad pressed flag
                BTFSC   ZERO            ;          when the button is released.
                BCF     KEYPAD_PRESSED  ;
               _GOTO    FALSE_FLAG      ;


CHK_SF_BRIEF:   MOVLW   SW_SF_CNT       ;
               _GOTO    CHECK_TIMING    ;

CHK_CAL_BRIEF:  MOVLW   SW_CAL_CNT      ;
               _GOTO    CHECK_TIMING    ;

CHK_MEM_BRIEF:  MOVLW   SW_MEM_CNT      ;
CHECK_TIMING:   MOVWF   FSR             ;
                BSF     IRP             ;
                MOVF    INDF,W          ;       Get old button count.
                MOVWF   TEMP_1          ;       Save the old button state.
                BCF     IRP             ;       Select current count.
                SUBWF   INDF,W          ;       Has button been released ? (current-old)
                MOVF    INDF,W          ;
                BSF     IRP             ;          Select old count.
                MOVWF   INDF            ;          Copy current state to old state.
                BCF     IRP             ;
                BTFSC   CARRY           ;
               _GOTO    FALSE_FLAG      ;       N. Return a false flag.
                MOVF    TEMP_1,W        ;       Y. Was the button on less than min ?
                SUBLW   MIN_ON          ;          (min on period - old)
                BTFSC   CARRY           ;
               _GOTO    FALSE_FLAG      ;          Y. Return a false flag.
                MOVF    TEMP_1,W        ;          N. Was the button on less than max ?
                SUBLW   MAX_ON          ;             (max on period - old)
RETURN_FLAG:    BSF     PRESSED         ;             Y. Return true flag.
                BTFSS   CARRY           ;
FALSE_FLAG:     BCF     PRESSED         ;             N. Return false flag.
                RETURN                  ;

;*****************************************************************************************
;
;     NAMES:    PUSH_ARG2               Push ARG2 to the ARG parameter stack.
;               PUSH_ARG3               Push ARG3 to the ARG parameter stack.
;               PULL_ARG2               Pop the ARG parameter stack to ARG2.
;               PULL_ARG3               Pop the ARG parameter stack to ARG3.
;
; VARIABLES:    FSR, COUNT_1, TEMP_1, TEMP_2, TEMP_3.
;
;               TEMP_1 = destination address.
;               TEMP_2 = source address.
;               TEMP_3 = byte to move.
;
; STACK USE:    0
;
;*****************************************************************************************

PULL_ARG3:      MOVLW   ARG3            ;
               _GOTO    PULL_ARG        ;
PULL_ARG2:      MOVLW   ARG2            ;
PULL_ARG:       MOVWF   TEMP_1          ;       Set destination address
                MOVLW   4               ;       Dec the stack pointer by 4.
                SUBWF   STACK_PTR,F     ;
                MOVF    STACK_PTR,W     ;       Get the stack pointer.
                MOVWF   TEMP_2          ;       Use it as the source address.
               _GOTO    COPY_4          ; (0)   Do the move. Return via "GOTO".

;-----------------------------------------------------------------------------------------

PUSH_ARG3:      MOVLW   ARG3            ;
               _GOTO    PUSH_ARG        ;
PUSH_ARG2:      MOVLW   ARG2            ;
PUSH_ARG:       MOVWF   TEMP_2          ;       Set source address
                MOVF    STACK_PTR,W     ;       Get the stack pointer.
                MOVWF   TEMP_1          ;       Use it as the destination address.
                MOVLW   4               ;       Inc the stack pointer by 4.
                ADDWF   STACK_PTR,F     ;
               _GOTO    COPY_4          ; (0)   Do the move. Return via "GOTO".

;*****************************************************************************************
;
;     NAMES:    RAM_TO_ARG2             Copy 4 bytes of RAM pointed to by W to ARG2.
;               RAM_TO_ARG3                                "                   ARG3.
;
;   PURPOSE:    Move 4 bytes of RAM pointed to by W to ARG.
;
;     INPUT:    W = source pointer.
;
; VARIABLES:    FSR, COUNT_1, TEMP_1, TEMP_2, TEMP_3.
;
;               TEMP_1 = destination address.
;               TEMP_2 = source address.
;               COUNT_1 = bytes to move.
;
; STACK USE:    1
;
;*****************************************************************************************

RAM_TO_ARG3:    MOVWF   TEMP_2          ;       Save the source pointer.
                MOVLW   ARG3            ;       The destination address is ARG3.
               _GOTO    RAM_TO_ARG      ; (1)


SF_TO_ARG2:     MOVLW   F_DDS_FREQ      ;
                BTFSS   SF_DISPLAYED    ;       Is start or finish frequency displayed.
                MOVLW   S_DDS_FREQ      ;

RAM_TO_ARG2:    MOVWF   TEMP_2          ;       Save the source pointer.
                MOVLW   ARG2            ;       The destination address is ARG2.

RAM_TO_ARG:     MOVWF   TEMP_1          ;       Save the address of the ARG.
               _CALL    CLEAR           ; (1)   Clear the ARG.
                MOVF    TEMP_2,W        ;
                MOVWF   FSR             ;
               _GOTO    COPY_4          ; (0)   Move 4 bytes. Return via "GOTO".

;*****************************************************************************************
;
;     NAMES:    ARG2_TO_RAM             Copy ARG2 to the RAM address pointed to by W.
;
;   PURPOSE:    Move ARG2 to RAM address pointed to by W.
;
;     INPUT:    W = destination pointer.
;
; VARIABLES:    FSR, COUNT_1, TEMP_1, TEMP_2, TEMP_3.
;
;               TEMP_1 = destination address.
;               TEMP_2 = source address.
;               COUNT_1 = bytes to move.
;
; STACK USE:    1
;
;*****************************************************************************************

ARG2_TO_SF:     MOVLW   F_DDS_FREQ      ;
                BTFSS   SF_DISPLAYED    ;       Is the start or finish frequency in use.
                MOVLW   S_DDS_FREQ      ;

ARG2_TO_RAM:    MOVWF   TEMP_1          ;       Save the RAM destination pointer.
                MOVLW   ARG2            ;       The source address is ARG2.
                MOVWF   TEMP_2          ;       Save the source address.
               _GOTO    COPY_4          ; (0)   Move 4 bytes. Return via "GOTO".

;*****************************************************************************************
;
;     NAMES:    COPY_TO_ARG2            Copy 5 bytes pointed to by W to ARG2.
;               COPY_TO_ARG3            Copy 5 bytes pointed to by W to ARG3.
;
;               COPY_FROM_ARG2          Copy ARG2 to the address pointed to by W.
;               COPY_FROM_ARG3          Copy ARG3 to the address pointed to by W.
;
;               ARG2_TO_ARG1            Copy ARG2 to ARG1.
;               ARG2_TO_ARG3            Copy ARG2 to ARG3.
;               ARG2_TO_ARG4            Copy ARG2 to ARG4.
;               ARG2_TO_ARG5            Copy ARG2 to ARG5.
;
;               ARG1_TO_ARG2            Copy ARG1 to ARG2.
;               ARG3_TO_ARG2            Copy ARG3 to ARG2.
;
;               ARG3_TO_ARG4            Copy ARG3 to ARG4.
;               ARG3_TO_ARG5            Copy ARG3 to ARG5.
;
;               ARG4_TO_ARG3            Copy ARG4 to ARG3.
;
;   PURPOSE:    Move 4 bytes of RAM from source address to dest address.
;               (starting at source and dest, then source+1 dest+1, etc.)
;
; VARIABLES:    FSR, COUNT_1, TEMP_1, TEMP_2, TEMP_3.
;
;               TEMP_1 = destination address.
;               TEMP_2 = source address.
;               COUNT_1 = bytes to move.
;
; STACK USE:    0
;
;*****************************************************************************************

ARG2_TO_ARG5:   MOVLW   ARG5            ;       \
               _GOTO    COPY_FROM_ARG2  ;        \
ARG2_TO_ARG4:   MOVLW   ARG4            ;         \
               _GOTO    COPY_FROM_ARG2  ;          |
ARG2_TO_ARG1:   MOVLW   ARG1            ;          |
COPY_FROM_ARG2: MOVWF   TEMP_1          ;          |
                MOVLW   ARG2            ;          |
               _GOTO    COPY_B          ;          |
                                        ;          |
ARG3_TO_ARG5:   MOVLW   ARG5            ;          |
               _GOTO    COPY_FROM_ARG3  ;          |
ARG3_TO_ARG4:   MOVLW   ARG4            ;          |
COPY_FROM_ARG3: MOVWF   TEMP_1          ;          |
                MOVLW   ARG3            ;           \  set the various source and
COPY_B:         MOVWF   TEMP_2          ;           /  destination addresses.
               _GOTO    COPY_4          ;          |
                                        ;          |
ARG3_TO_ARG2:   MOVLW   ARG3            ;          |
               _GOTO    COPY_TO_ARG2    ;          |
ARG1_TO_ARG2:   MOVLW   ARG1            ;          |
COPY_TO_ARG2:   MOVWF   TEMP_2          ;          |
                MOVLW   ARG2            ;          |
               _GOTO    COPY            ;          |
ARG4_TO_ARG2:   MOVLW   ARG4            ;          |
               _GOTO    COPY_TO_ARG2    ;          |                                        ;          |
ARG4_TO_ARG3:   MOVLW   ARG4            ;          |
               _GOTO    COPY_TO_ARG3    ;          |
ARG2_TO_ARG3:   MOVLW   ARG2            ;         /
COPY_TO_ARG3:   MOVWF   TEMP_2          ;        /
                MOVLW   ARG3            ;       /
                                        ;
COPY:           MOVWF   TEMP_1          ;
COPY_4:         MOVLW   4               ;       4 bytes to move.
                MOVWF   COUNT_1         ;
COPY_LOOP:      MOVF    TEMP_2,W        ;       Get source address.
                MOVWF   FSR             ;              "
                MOVF    INDF,W          ;       Get the byte from the source address.
                MOVWF   TEMP_3          ;                      "
                MOVF    TEMP_1,W        ;       Get destination address.
                MOVWF   FSR             ;                "
                MOVF    TEMP_3,W        ;       Get the source byte.
NO_MONITOR:     MOVWF   INDF            ;       Place the byte at destination address.
                INCF    TEMP_1,F        ;       Inc dest address.
                INCF    TEMP_2,F        ;       Inc source address.
                DECFSZ  COUNT_1,F       ;
               _GOTO    COPY_LOOP       ;       Loop until all bytes copied.
                RETURN                  ;       ARG2 and destination.

;*****************************************************************************************
;
;     NAMES:    SAVE            Copy the current setup to the selected memory in EEPROM.
;               RECALL          Copy the selected memory in EEPROM to RAM.
;
;     INPUT:    ARG2 = Memory number. (1 to 12)
;
; VARIABLES:    FSR, COUNT_1, COUNT_2, TEMP_1, TEMP_2, TEMP_3.
;
; STACK USE:    3
;
;*****************************************************************************************

SAVE:           BSF     WRITE_FLAG      ;       Indicate we want to write to EEPROM.
RECALL:        _CALL    CLEAR_ARG3      ; (1)
                                        ;
                DECF    MEM_NUMBER,W    ;       ARG3 = mem number. (0 to max-1)
                MOVWF   ARG3_0          ;
                MOVLW   9               ;       9 Bytes per memory.
               _CALL    ARG3_TIMES_W    ; (3)   ARG3 = bytes per mem x mem num = offset.
                MOVLW   EE_MEM_START    ;       W = base address of memories.
                ADDWF   ARG3_0,W        ;       W = base address + offset = EEPROM addr.
                BSF     RP0             ;       Select bank 1 for EEPROM reg access.
                MOVWF   EEADR           ;       Save the EEPROM address.
                BCF     RP0             ;       Return to bank 0.
                                        ;
                MOVLW   9               ;       9 Bytes per memory.
                MOVWF   COUNT_1         ;       Save the number of bytes to move.
                MOVLW   S_DDS_FREQ      ;
                MOVWF   FSR             ;       Set the RAM address.
                                        ;
               _CALL    EEPROM_LOOP     ; (3)   Do the move.
                                        ;
SETUP_SWEEP:    MOVLW   SWEEP_NUM       ;
                MOVWF   FSR             ;
                MOVF    INDF,W          ;       Get the SWEEP_NUM.
                MOVWF   SWP_RATE_NUM    ;       Save the sweep rate number. 0 to 9.
               _CALL    GET_RATE  ; (1)   Convert the number to the sweep rate.
                MOVWF   SWEEP_RATE      ;       Save it.
                                        ;
                MOVF    SWP_RATE_NUM,W  ;       Get the sweep rate number.
               _CALL    SWEEP_TO_ARG3   ; (2)   Get the 2 byte sweep constant.
                                        ;
                MOVLW   10              ;
                BTFSC   FAST_WOBBLE      ;       Is the fast sweep function active ?
               _CALL    ARG3_TIMES_W    ; (3)   Y. Multiply it by 10.
                                        ;
                MOVLW   SWP_STEP_SIZE   ;
               _GOTO    COPY_FROM_ARG3  ; (0)   Save it. Return via "GOTO".

;*****************************************************************************************
;
;     NAMES:    EE_TO_RAM               Copy 19 bytes of EEPROM to RAM.
;                                         (all cal & setup)
;               RAM_TO_EE               Copy 19 bytes of RAM to EEPROM.
;                                         (all cal & setup)
;               SETUP_TO_EE             Copy 9 bytes of RAM to EEPROM.
;                                         (all cal & setup except the following )
;                                         (STEP_NUM, S_DDS_FREQ, F_DDS_FREQ & SWEEP_NUM)
;               FREQ_TO_EE              Copy 10 bytes of RAM to EEPROM.
;                                         (STEP_NUM, S_DDS_FREQ, F_DDS_FREQ & SWEEP_NUM)
;
;     INPUT:    none.
;
; VARIABLES:    FSR, COUNT_1, TEMP_1, TEMP_2, TEMP_3.
;
;               TEMP_1 = destination address.
;               TEMP_2 = source address.
;               COUNT_1 = bytes to move.
;
; STACK USE:    2
;
;*****************************************************************************************

SETUP_TO_EE:    BSF     WRITE_FLAG      ;       Indicate we want to write to EEPROM.
                MOVLW   9               ;       9 bytes to move.
               _GOTO    EE_MOVE         ; (2)   Do the move. Return via "GOTO".

RAM_TO_EE:      BSF     WRITE_FLAG      ;       Indicate we want to write to EEPROM.
EE_TO_RAM:      MOVLW   19              ;
EE_MOVE:        MOVWF   COUNT_1         ;       19 bytes to move.
                MOVLW   MAX_DDS_FREQ    ;       Set start address in RAM.
                MOVWF   FSR             ;
                MOVLW   EE_MAX_DDS_FREQ ;       Set start address in EEPROM.
               _GOTO    EEPROM_MOVE     ; (2)   Do the move. Return via "GOTO".

FREQ_TO_EE:     BSF     WRITE_FLAG      ;       Indicate we want to write to EEPROM.
                MOVLW   10              ;
                MOVWF   COUNT_1         ;       10 bytes to move.
                MOVLW   STEP_NUM        ;       Set start address in RAM.
                MOVWF   FSR             ;
                MOVLW   EE_STEP_NUM     ;       Set start address in EEPROM.
               _GOTO    EEPROM_MOVE     ; (2)   Do the move. Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    EEPROM_MOVE
;
;   PURPOSE:    Move several bytes, between RAM and EEPROM, or EEPROM and RAM.
;               If the RAM and EEPROM data are the same, the EEPROM is not reprogrammed.
;               Assumes the number of bytes to move COUNT_1 has been set,
;               and the RAM source/destination address FSR has been set,
;               and the EEPROM source/destination address is in W,
;               Setting the WRITE_FLAG before calling EEPROM_MOVE indicates that the
;               direction will be from RAM to EEPROM, otherwise it will be EEPROM to RAM.
;
;     INPUT:    WRITE_FLAG = direction. (Set = RAM to EEPROM)
;                  COUNT_1 = bytes to move.
;                      FSR = RAM address.
;                        W = EEPROM address.
;
;    OUTPUT:    None.
;
; VARIABLES:    FSR,  COUNT_1.
;
; STACK USE:    2
;
;*****************************************************************************************

EEPROM_MOVE:    BSF     RP0             ;       Select bank 1 for EEPROM reg access.
                MOVWF   EEADR           ;       Save the EEPROM address.
                BCF     RP0             ;       Return to bank 0.
EEPROM_LOOP:    BTFSC   WRITE_FLAG      ;       Do we want to write ? (WRITE_FLAG set)
               _CALL    WRITE_EEPROM    ; (2)   Y. Write a byte from RAM to EEPROM.
               _CALL    READ_EEPROM     ; (1)   Get a byte from EEPROM.
                MOVWF   INDF            ;       Place it in RAM.
                BSF     RP0             ;       Select bank 1 for EEPROM reg access.
                INCF    EEADR,F         ;       Inc the EEPROM address.
                BCF     RP0             ;       Return to bank 0.
                INCF    FSR,F           ;       Inc the RAM address.
                DECFSZ  COUNT_1,F       ;
               _GOTO    EEPROM_LOOP     ;       Loop until all bytes moved.
                BCF     WRITE_FLAG      ;
                RETURN                  ;

;*****************************************************************************************
;
;      NAME:    WRITE_EEPROM
;
;   PURPOSE:    Write the byte of RAM pointed to by FSR,
;               to the EEPROM address pointed to by EEADR.
;               If the RAM and EEPROM data are the same, the EEPROM is not reprogrammed.
;               Assumes the source address FSR has been set,
;               and the destination address EEADR has been set.
;               Goes to sleep while programming a byte because my 16F648A-ICD Header
;               requires it
;
;     INPUT:    FSR, EEADR.
;
;    OUTPUT:    None.
;
; VARIABLES:    FSR.
;
; STACK USE:    1
;
;*****************************************************************************************
;
;      NAME:    WRITE_SWEEP             Copy the sweep rate to EEPROM.
;
; STACK USE:    1
;
;*****************************************************************************************

WRITE_SWEEP:    MOVLW   SWEEP_NUM       ;       Set start address in RAM.
                MOVWF   FSR             ;
                MOVLW   EE_SWEEP_NUM    ;       Set start address in EEPROM.
                BSF     RP0             ;       Select bank 1 for EEPROM reg access.
                MOVWF   EEADR           ;       Save the EEPROM address.
                BCF     RP0             ;       Return to bank 0.

WRITE_EEPROM:  _CALL    READ_EEPROM     ; (1)   Get the current byte from EEPROM.
                SUBWF   INDF,W          ;
                BTFSC   ZERO            ;       Is it the same as it is in RAM ?
               _GOTO    NO_CHANGE       ;
                MOVF    INDF,W          ;          N. Get the data byte from RAM.
                BCF     EEIF            ;             Clear the EEPROM write done flag.
                BSF     RP0             ;             Select bank 1 for EEPROM reg access.
                MOVWF   EEDATA          ;             Put the byte in the buffer.
                MOVLW   B'01000000'     ;
                MOVWF   INTCON          ;             Enable peripheral interrupt only.
                MOVLW   B'10000000'     ;
                MOVWF   PIE1            ;             Enable EE write complete only.
                BSF     WREN            ;             Enable EEPROM write.
                MOVLW   H'55'           ;             Perform required safety steps.
                MOVWF   EECON2          ;
                MOVLW   H'AA'           ;
                MOVWF   EECON2          ;
                BSF     WR              ;             Begin the write.
                SLEEP                   ;             Sleep until write is done.
                BCF     WREN            ;             Disable EEPROM write.
                BCF     RP0             ;             Return to bank 0.
                MOVLW   B'10100000'     ;             Re-enable TMR0 overflow interrupt.
                MOVWF   INTCON          ;
                MOVLW   B'00000000'     ;
                MOVWF   PIE1            ;             Disable EE write complete interrupt.
NO_CHANGE:      BCF     EEPROM_UPDATE   ;       Y. Clear the update required flag.
                RETURN                  ;

;*****************************************************************************************
;
;      NAME:    READ_EEPROM
;
;   PURPOSE:    Read the byte of EEPROM data pointed to by EEADR.
;               Assumes the read address EEADR has been set.
;
;     INPUT:    EEADR.
;
;    OUTPUT:    W = The byte read from EEPROM.
;
; VARIABLES:    None.
;
; STACK USE:    0
;
;*****************************************************************************************

READ_EEPROM:    BSF     RP0             ;       Select bank 1 for EEPROM reg access.
                BSF     RD              ;       Perform an EEPROM read.
                MOVF    EEDATA,W        ;       Get the read byte.
                BCF     RP0             ;       Return to ram bank 0.
                RETURN                  ;

;*****************************************************************************************
;
;      NAME:    I2C_TO_LCD
;
;   PURPOSE:    Display a string of W characters the power meter, The power meter command
;               must already be set by the SET_RX_COMMAND subroutine.
;
;     INPUT:    W = number of bytes to display.
;
;    OUTPUT:    none.
;
; VARIABLES:    COUNT_1, COUNT_2, TEMP_1, TEMP_3, I2C_DATA_RD.
;
; STACK USE:    3
;
;*****************************************************************************************

PWR_TO_LCD:     BSF     PWR_DIS_FLAG    ;       Set the power display flag.
                MOVLW   8               ;       8 characters in power text.

I2C_TO_LCD:     BTFSC   NO_PWR_METER    ;       Is the power meter fitted ?
                RETURN                  ;       N. All done. Just return.
                MOVWF   COUNT_2         ;       Get the number of bytes to display.
                BCF     METER_FLAG      ;       Clear the power meter flag.
               _CALL    I2C_START       ; (2)   Send an I2C start signal.
                MOVLW   I2C_RD_ADDR     ;       Get the read address.
               _CALL    I2C_TX_BYTE     ; (3)   Send the chip address byte.
                BTFSC   I2C_ACK_FLAG    ;       Did the slave acknowledge the address.
               _GOTO    I2C_LCD_END     ;
NEXT_I2C_BYTE:  BCF     I2C_TX_ACK_FLAG ;       Y. Last byte will be followed by a NACK.
                DECFSZ  COUNT_2,W       ;
                BSF     I2C_TX_ACK_FLAG ;          All other bytes will be followed by ACK.
               _CALL    I2C_RX_BYTE     ; (3)      Read the byte into I2C_DATA_RD.
                MOVF    I2C_DATA_RD,W   ;          Get the RX byte.
                BTFSS   I2C_TX_ACK_FLAG ;
                BTFSS   VER_FLAG        ;          Is this the last byte of version text ?
               _GOTO    SKIP_VER_ID     ;
                MOVWF   ID_BYTE         ;          Y. Save the ID byte.
               _GOTO    I2C_LCD_END     ;             All done.
                                        ;
SKIP_VER_ID:    BTFSC   PWR_DIS_FLAG    ;          Is this the power display,
                BTFSS   COUNT_2,3       ;          and is it chr 8 ? (first chr of power)
               _GOTO    SKIP_SPACE      ;
                MOVF    I2C_DATA_RD,W   ;          Y. Get the RX byte.
                SUBLW   ' '             ;             Is first chr a space,
                BTFSC   ZERO            ;
               _GOTO    SKIP_SPACE      ;
                ADDLW   '-' - ' '       ;             or is first chr a - ?
                BTFSC   ZERO            ;
               _GOTO    SKIP_SPACE      ;
                MOVLW   ' '             ;             N. Display an extra space.
               _CALL    CHR_LCD         ; (2)            Chr to LCD, no change to interupts. 
SKIP_SPACE:     BCF     PWR_DIS_FLAG    ;          Clear the power display flag.
                MOVF    I2C_DATA_RD,W   ;          Get the RX byte.
               _CALL    CHR_LCD         ; (2)      Chr to LCD no change to interupts. 
                DECFSZ  COUNT_2,F       ;
               _GOTO    NEXT_I2C_BYTE   ;
I2C_LCD_END:   _GOTO    I2C_STOP        ; (1)   Send the stop command. Return via "GOTO".


                                        ;       Character to LCD with no change to interupts. 
CHR_LCD:        MOVWF   TEMP_3          ;       Save the byte to send to the LCD.
                BSF     LCD_RS          ;       Select display data register.
              P_CALL    SEND_NIBBLE     ; (1)   Send the high nibble to the LCD.
                SWAPF   TEMP_3,F        ;
              P_CALL    SEND_NIBBLE     ; (1)   Send the low nibble to the LCD.
                RETURN

;*****************************************************************************************
;
;      NAME:    RX_I2C_BYTES
;
;   PURPOSE:    Receive upto 4 bytes from the power meter place results in ARG3 MSD first.
;
;     INPUT:    W = RX command number.
;
;    OUTPUT:    Flag RX_I2C_OK is set if all bytes have been received ok.
;
; VARIABLES:    FSR, COUNT_1, COUNT_2, TEMP_1, I2C_DATA_RD.
;
; STACK USE:    3
;
;*****************************************************************************************

RX_I2C_BYTES:   BCF     RX_I2C_OK       ;
                BTFSC   NO_PWR_METER    ;       Is the power meter fitted ?
                RETURN                  ;       N. All done. Just return.

                MOVWF   I2C_DATA_WR     ;       Save the command number.
                MOVWF   COUNT_2         ;               "
                                        ;
               _CALL    I2C_START       ; (2)   Send an I2C start signal.
                MOVLW   I2C_WR_ADDR     ;       Get the write address.
               _CALL    I2C_TX_BYTE     ; (3)   Send the address byte.
                MOVF    I2C_DATA_WR,W   ;       Get the data byte.
               _CALL    I2C_TX_BYTE     ; (3)   Send the byte.
               _CALL    I2C_STOP        ; (2)   Send the stop command.
                                        ;
                BCF     RX_I2C_OK       ;
               _CALL    I2C_START       ; (2)   Send an I2C start signal.
                MOVLW   I2C_RD_ADDR     ;       Get the read address.
               _CALL    I2C_TX_BYTE     ; (3)   Send the chip address byte.
                BTFSC   I2C_ACK_FLAG    ;       Did the slave acknowledge the address.
               _GOTO    I2C_RX_END      ;
                MOVF    COUNT_2,W       ;       Y. Get the command number.
               _CALL    RX_BYTES        ; (1)      Get the number of bytes to receive.
                MOVWF   COUNT_2         ;
                MOVLW   ARG3_3          ;          Address of the buffer to save the
                MOVWF   FSR             ;          bytes to.
NEXT_RX_BYTE:   BCF     I2C_TX_ACK_FLAG ;          Last byte will be followed by a NACK.
                DECFSZ  COUNT_2,W       ;
                BSF     I2C_TX_ACK_FLAG ;          All other bytes will be followed by ACK.
               _CALL    I2C_RX_BYTE     ; (3)      Read the byte into I2C_DATA_RD.
                MOVF    I2C_DATA_RD,W   ;          Get the RX byte.
                MOVWF   INDF            ;          Save the byte in the buffer.
                INCF    FSR,F           ;          Inc the buffer pointer.
                DECFSZ  COUNT_2,F       ;
               _GOTO    NEXT_RX_BYTE    ;          Loop until all bytes have been received.
                BSF     RX_I2C_OK       ;          Flag that bytes have been received ok.
I2C_RX_END:    _GOTO    I2C_STOP        ; (1)   Send the stop command. Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    SET_RX_COMMAND
;
;   PURPOSE:    Send the command number byte to the power meter.
;
;     INPUT:    W = byte to write. 8 to 15
;
;    OUTPUT:    None.
;
; VARIABLES:    COUNT_1, TEMP_1, I2C_DATA_WR.
;
; STACK USE:    3
;
;*****************************************************************************************

RX_COMMAND_14:  MOVLW   14              ;       Power meter command 14 = return the power.

SET_RX_COMMAND: BTFSC   NO_PWR_METER    ;       Is the power meter fitted ?
                RETURN                  ;       N. All done. Just return.
                                        ;
                MOVWF   I2C_DATA_WR     ;       Save the data byte.
               _CALL    I2C_START       ; (2)   Send an I2C start signal.
                MOVLW   I2C_WR_ADDR     ;       Get the write address.
               _CALL    I2C_TX_BYTE     ; (3)   Send the address byte.
                MOVF    I2C_DATA_WR,W   ;       Get the data byte.
               _CALL    I2C_TX_BYTE     ; (3)   Send the byte.
               _GOTO    I2C_STOP        ; (1)   Send the stop command. Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    RS_METER
;
;   PURPOSE:    Send command 7 FF to the power meter, that will reset the power meter.
;
;     INPUT:    None.
;
;    OUTPUT:    None.
;
; VARIABLES:    COUNT_1, TEMP_1, I2C_DATA_WR.
;
; STACK USE:    4
;
;*****************************************************************************************

RS_METER:       BTFSC   NO_PWR_METER    ;       Is the power meter fitted ?
                RETURN                  ;       N. All done. Just return.
                                        ;
               _CALL    TX_COMMAND_7    ; (4)   Power meter command 7.
                MOVLW   H'FF'           ;       FF = Reset the power meter software.
               _CALL    TX_BYTE_END     ; (4)   Send the byte & finish the command.
                                        ;
              P_CALL    WAIT_100MS      ; (2)   Wait until the power meter has restarted.
                RETURN                  ;

;*****************************************************************************************
;
;      NAME:    SET_TX_COMMAND
;
;   PURPOSE:    Send the command number byte to the power meter.
;
;     INPUT:    W = byte to write. 0 to 7
;
;    OUTPUT:    None.
;
; VARIABLES:    COUNT_1, TEMP_1, I2C_DATA_WR.
;
; STACK USE:    3
;
;*****************************************************************************************

TX_COMMAND_7:   MOVLW   7               ;       Power meter command 7 = save memories.

SET_TX_COMMAND: BTFSC   NO_PWR_METER    ;       Is the power meter fitted ?
                RETURN                  ;       N. All done. Just return.
                                        ;
                MOVWF   I2C_DATA_WR     ;       Save the data byte.
               _CALL    I2C_START       ; (2)   Send an I2C start signal.
                MOVLW   I2C_WR_ADDR     ;       Get the write address.
               _CALL    I2C_TX_BYTE     ; (3)   Send the address byte.
                MOVF    I2C_DATA_WR,W   ;       Get the data byte.
               _GOTO    I2C_TX_BYTE     ; (3)   Send the byte. Return via "GOTO".
                                        ;
TX_BYTE_END:    BTFSC   NO_PWR_METER    ;       Is the power meter fitted ?
                RETURN                  ;       N. All done. Just return.
               _CALL    I2C_TX_BYTE     ; (3)   Send the byte.
               _GOTO    I2C_STOP        ; (1)   Send the stop command. Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    I2C_TX_BYTE             Send the byte in W to the I2C bus.
;
; VARIABLES:    COUNT_1, TEMP_1.
;
; STACK USE:    2
;
;*****************************************************************************************

I2C_TX_BYTE:    MOVWF   TEMP_1          ;       Save the byte to TX.
                MOVLW   8               ;       Set the number of bits to transmit.
                MOVWF   COUNT_1         ;
TX_BIT_LOOP:    RLF     TEMP_1,F        ;       Move the bit (MSB first) into the carry.
                BTFSC   CARRY           ;       Carry set ?
               _CALL    LET_SDA_HIGH    ; (1)   Y. Let SDA float high. (Set SDA as I/P)
                BTFSS   CARRY           ;
               _CALL    SET_SDA_LOW     ; (1)   N. Set SDA low. (Set SDA as O/P)
               _CALL    I2C_CLOCK       ; (2)   Latch the data into the slave.
                DECFSZ  COUNT_1,F       ;
               _GOTO    TX_BIT_LOOP     ;       Loop until 8 bits are sent.
                                        ;
               _CALL    LET_SDA_HIGH    ; (1)   Let SDA float high. (Set SDA as I/P)
               _CALL    I2C_DELAY       ; (1)
               _CALL    LET_SCL_HIGH    ; (1)   Let SCL float high. (Set SCL as I/P)
               _CALL    I2C_DELAY       ; (1)
                                        ;
                BTFSS   I2C_SDA         ;
                BCF     I2C_ACK_FLAG    ;       Save the slaves acknowledge bit.
                BTFSC   I2C_SDA         ;
                BSF     I2C_ACK_FLAG    ;              "
                                        ;
               _CALL    SET_SCL_LOW     ; (1)   Set SCL low. (Set SCL as O/P)
               _GOTO    I2C_DELAY       ; (0)   Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    I2C_RX_BYTE             Read a byte into W from the I2C bus.
;
; VARIABLES:    COUNT_1, TEMP_1.
;
; STACK USE:    2
;
;*****************************************************************************************
;
;      NAME:    ACK                     Send an I2C acknowledge to the I2C bus.
;                                       (SDA low while generating a clock signal)
;               NACK                    Send an I2C negative acknowledge to the I2C bus.
;                                       (SDA high while generating a clock signal)
; VARIABLES:    None.
;
; STACK USE:    2
;
;*****************************************************************************************

I2C_RX_BYTE:    MOVLW   8               ;
                MOVWF   COUNT_1         ;       Set the number of bits to receive.
RX_BIT_LOOP:   _CALL    I2C_DELAY       ; (1)
               _CALL    LET_SCL_HIGH    ; (1)   Let SCL float high. (Set SCL as I/P)
               _CALL    I2C_DELAY       ; (1)
                                        ;
                RLF     I2C_DATA_RD,F   ;       Make room for the received bit.
                BTFSC   I2C_SDA         ;
                BSF     I2C_DATA_RD,0   ;       Save the received bit. (1)
                BTFSS   I2C_SDA         ;
                BCF     I2C_DATA_RD,0   ;       Save the received bit. (0)
                                        ;
               _CALL    SET_SCL_LOW     ; (1)   Set SCL low. (Set SCL as O/P)
                DECFSZ  COUNT_1,F       ;
               _GOTO    RX_BIT_LOOP     ;       Loop until 8 bits received.
                BTFSC   I2C_TX_ACK_FLAG ;
               _GOTO    ACK             ; (2)   Send an acknowledge, Return via "GOTO".
                                        ;       Or send a negative ack.
NACK:          _CALL    LET_SDA_HIGH    ; (1)   Let SDA float high. (Set SDA as I/P)
               _CALL    I2C_CLOCK       ; (2)   (Send an I2C negative acknowledge)
                                        ;       Short delay.
I2C_DELAY       MOVLW   DELAY_CONSTANT  ;          Load W with count for the delay.
I2C_DELAY_LOOP  ADDLW   -1              ;          1 CLK cycles \
                BTFSS   ZERO            ;          1     "       > 4 CLKs x 1uS
               _GOTO    I2C_DELAY_LOOP  ;          2     "      /
                RETURN                  ;

;-----------------------------------------------------------------------------------------

ACK:           _CALL    SET_SDA_LOW     ; (1)   Set SDA low. (Set SDA as O/P)
               _CALL    I2C_CLOCK       ; (2)   (Send an I2C acknowledge)
               _CALL    I2C_DELAY       ; (1)
LET_SDA_HIGH:   BSF     RP0             ;       Select bank 1.
                BSF     I2C_SDA_DIR     ;       Let SDA float high. (Set SDA as I/P)
               _GOTO    RET_BANK_0      ;       Return to bank 0. Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    I2C_STOP                Send an I2C stop signal on the bus.
;               I2C_START               Send an I2C start signal on the bus.
;
; VARIABLES:    None.
;
; STACK USE:    1
;
;*****************************************************************************************

I2C_STOP:      _CALL    SET_SDA_LOW     ; (1)   Set SDA low. (Set SDA as O/P)
               _CALL    I2C_DELAY       ; (1)
               _CALL    LET_SCL_HIGH    ; (1)   Let SCL float high. (Set SCL as I/P)
               _CALL    I2C_DELAY       ; (1)
               _CALL    LET_SDA_HIGH    ; (1)   Let SDA float high. (Set SDA as I/P)
               _CALL    I2C_DELAY       ; (1)
                BSF     GIE             ;       Enable interrupts.
                RETURN                  ;

;-----------------------------------------------------------------------------------------

I2C_START:      BCF     GIE             ;       Disable interrupts.
               _CALL    LET_SCL_HIGH    ; (1)   Let SCL float high. (Set SCL as I/P)
               _CALL    LET_SDA_HIGH    ; (1)   Let SDA float high. (Set SDA as I/P)
               _CALL    I2C_DELAY       ; (1)
               _CALL    SET_SDA_LOW     ; (1)   Set SDA low. (Set SDA as O/P)
               _GOTO    I2C_CLOCK2      ; (1)   Return via "GOTO".

;-----------------------------------------------------------------------------------------

I2C_CLOCK:     _CALL    I2C_DELAY       ; (1)
               _CALL    LET_SCL_HIGH    ; (1)   Let SCL float high. (Set SCL as I/P)
I2C_CLOCK2:    _CALL    I2C_DELAY       ; (1)
SET_SCL_LOW:    BCF     I2C_SCL         ;       SCL will be low when set as O/P.
                BSF     RP0             ;       Select bank 1.
                BCF     I2C_SCL_DIR     ;       Set SCL low. (Set SCL as O/P)
RET_BANK_0:     BCF     RP0             ;       Return to bank 0.
                RETURN                  ;

SET_SDA_LOW:    BCF     I2C_SDA         ;       SDA will be low when set as O/P.
                BSF     RP0             ;       Select bank 1.
                BCF     I2C_SDA_DIR     ;       Set SDA low. (Set SDA as O/P)
               _GOTO    RET_BANK_0      ;       Return to bank 0. Return via "GOTO".

;-----------------------------------------------------------------------------------------

LET_SCL_HIGH:                           ;
 IF SIMULATE                            ;
                RETURN                  ;
 ELSE                                   ;
                BSF     RP0             ;       Select bank 1.
                BSF     I2C_SCL_DIR     ;       Let SCL float high. (Set SCL as I/P)
                BCF     RP0             ;       Return to bank 0.
I2C_CLOCK_WAIT: BTFSS   I2C_SCL         ;
               _GOTO    I2C_CLOCK_WAIT  ;       Wait until SCL goes high.
                RETURN                  ;       (Slave may stretch the clock)
 ENDIF                                  ;

;*****************************************************************************************
;               ICD2 reserved code space. 700-7FF hex.
;*****************************************************************************************

 IF $ >= H'700' 
 ERROR "Code Space 700-7FF hex reserved for ICD2."
 ENDIF

;-----------------------------------------------------------------------------------------


 IF $ >= PAGE0 + H'800'
 ERROR "Page 0 to long"
 ENDIF

;*****************************************************************************************
;               End of PAGE 0 routines.
;*****************************************************************************************


;*****************************************************************************************
;               Start of PAGE 1 routines.
;*****************************************************************************************

                ORG     TABLE_1_END             ;



;*****************************************************************************************
;
;       Cal setup screen :-     DDS REF MULT. (for AD9851 only)
;
;               Range =  x1 REFCLK, x6 REFCLK.
;
;*****************************************************************************************

CAL_SCRNS:      BSF     CAL_FLAG        ;       Flag that the cal screens are in use.
                                        ;
              P_CALL    CLEAR_ARG4      ; (1)   ***** Calculate the lower limit *****
                                        ;       Save 0 as the lower limit. (ARG4)
                                        ;
              P_CALL    CLEAR_ARG5      ; (1)   ***** Calculate the upper limit *****
                INCF    ARG5_0,F        ;       Save 1 as the upper limit. (ARG5)
                                        ;
               _CALL    LCD_CLEAR       ; (3)   Clear the display.
                                        ;
 IF AD9851_CHIP                         ;       Assemble if using the AD9851.
               _MOVLW   DDS             ;       *********************************
               _CALL____LCD_TEXT        ; (3)   *  Display 'DDS REF MULT'.      *
                                        ;       *********************************
              P_CALL    CLEAR_ARG2      ; (1)
                                        ;
                MOVLW   DDS_OPT         ;       Get the address of DDS_OPT.
                MOVWF   FSR             ;
                BTFSC   INDF,X6_REF     ;          Get the x6 ref bit.
                BSF     ARG2_0,0        ;       ***** Put it in ARG2 for adjustment *****
                                        ;
                BCF     TIMEOUT         ;       Clear the display timeout flag.
CHIP_LOOP:      BTFSC   TIMEOUT         ;       Has the display timeout been reached ?
               _GOTO    RESTORE_SETUP   ; (3)   Y. Ignore changes, Go to main display.
                                        ;
              P_CALL    UP_DN_1         ; (5)   Chk encoder. (inc/dec ARG2, & chk limits)
                                        ;
                MOVLW   H'C0'           ;
               _CALL    LCD_CMD         ; (2)   Position the cursor, line 2 pos 1.
                                        ;
                MOVLW   "x"             ;
               _CALL    LCD_CHR         ; (2)      Display ' x'.
                                        ;
                MOVLW   "1"             ;
                BTFSC   ARG2_0,0        ;          Is bit 0 set ?
                MOVLW   "6"             ;          Y. Display '6'.
               _CALL    LCD_CHR         ; (2)      N. Display '1'.
                                        ;
               _MOVLW   REF             ;
               _CALL____LCD_TEXT        ; (3)      Display ' REFCLK'.
                                        ;
              P_CALL    CHK_CAL_BRIEF   ; (1)
                BTFSS   PRESSED         ;       Was the CAL button pressed briefly ?
               _GOTO    CHIP_LOOP       ;       N. Continue updating the display.
                                        ;
                MOVLW   DDS_OPT         ;       Y. Get the address of DDS_OPT.
                MOVWF   FSR             ;
                BCF     INDF,X6_REF     ;          Update the DDS REF bit in ram.
                BTFSC   ARG2_0,0        ;
                BSF     INDF,X6_REF     ;
 ENDIF                                  ;

;*****************************************************************************************
;
;       Cal setup screen :-     Adjust DDS SYSTEM CLK freq to give a DDS O/P of 10 MHz.
;
;
;               NOTES:- DDS SYSTEM CLK = DDS crystal freq, for AD9850 & AD9851 x1 REFCLK
;                                      = 6 x DDS crystal freq, for AD9851 x6 REFCLK
;
;                       The variable in EEPROM is DDS_CAL = 2^56 / DDS SYSTEM CLK
;
;                       There is no timeout on this display.
;
;               Range = 30 MHz to 130 MHz for AD9850.
;                       30 MHz to 190 MHz for AD9851.
;
;*****************************************************************************************

CAL_10MHZ:    P_CALL    TEN_MEG         ; (4)   ***** Calculate the lower limit *****
                MOVLW   3               ;       Set ARG3 to 10 MHz.
              P_CALL    ARG3_TIMES_W    ; (3)   ARG3 = 30 MHz.
              P_CALL    ARG3_TO_ARG4    ; (1)   Save it as the lower limit. (ARG4)
                                        ;
              P_CALL    TEN_MEG         ; (4)   ***** Calculate the upper limit *****
 IF AD9851_CHIP                         ;       Is the assembly for the AD9851 ?
                MOVLW   19              ;       Y. Set ARG3 to 190 MHz.
 ELSE                                   ;
                MOVLW   13              ;       N. Set ARG3 to 130 MHz.
 ENDIF                                  ;
              P_CALL    ARG3_TIMES_W    ; (3)
              P_CALL    ARG3_TO_ARG5    ; (1)   Save it as the upper limit. (ARG5)
                                        ;
                MOVLW   DDS_CAL         ;       ***** Get the current DDS_CAL       *****
              P_CALL    RAM_TO_ARG2     ; (2)   ***** Put it in ARG2 for adjustment *****
              P_CALL    TWO_56_DIVIDE   ; (3)         ARG2 = 2^56 / ARG2 = DDS SYSTEM CLK.
                                        ;
                MOVLW   H'83'           ;       *********************************
               _CALL    LCD_CMD         ; (2)   *  Goto LCD line 1 position 4.  *
               _MOVLW   DDS_SYS         ;       *  Display 'SYSTEM CLK'.        *
               _CALL____LCD_TEXT        ; (3)   *  ( displays 'DDS SYSTEM CLK') *
                                        ;       *********************************
                                        ;
CAL_DDS:      P_CALL    CHK_KEYPAD      ; (5)   Chk for keypad I/P.
                                        ;
              P_CALL    UP_DOWN         ; (5)   Chk encoder (inc/dec ARG2, & chk limits)
                                        ;       and adjust step size.
                                        ;
              P_CALL    PUSH_ARG2       ; (1)   Save the DDS SYSTEM CLK freq.
                                        ;
              P_CALL    DISPLAY_FREQ    ; (4)   Display the variable.
                                        ;
              P_CALL    TWO_56_DIVIDE   ; (3)   ARG2 = 2^56 / ARG2 = DDS CAL.
              P_CALL    TEN_MEG         ; (4)   Set ARG3 to 10 MHz.
              P_CALL    CHK_DDS         ; (4)   Update DDS if there were any changes.
                                        ;
              P_CALL    PULL_ARG2       ; (1)   Recover the DDS SYSTEM CLK freq.
                                        ;
              P_CALL    CHK_CAL_BRIEF   ; (1)
                BTFSS   PRESSED         ;       Was the CAL button pressed briefly ?
               _GOTO    CAL_DDS         ;       N. Continue updating the display.
                                        ;
              P_CALL    PUSH_ARG2       ; (1)   Y. Save the DDS SYS CLK. (for next screen)
                                        ;
              P_CALL    TWO_56_DIVIDE   ; (3)      ARG2 = 2^56 / ARG2 = DDS CAL.
                MOVLW   DDS_CAL         ;
              P_CALL    ARG2_TO_RAM     ; (1)      Save the CAL constant to RAM.

;*****************************************************************************************
;
;       Cal setup screen :-     Maximum DDS frequency. (MAX_DDS_FREQ)
;
;               Range = 0 MHz to .45 x DDS SYSTEM CLK.
;
;*****************************************************************************************

                                        ;       ***** Calculate the lower limit *****
              P_CALL    CLEAR_ARG4      ; (1)   Save 0 as the lower limit. (ARG4)
                                        ;
                                        ;       ***** Calculate the upper limit *****
              P_CALL    PULL_ARG3       ; (1)   Recover the DDS SYSTEM CLK freq.
                MOVLW   9               ;
              P_CALL    ARG3_TIMES_W    ; (3)   Multiply it by 9.
              P_CALL    ARG3_TO_ARG2    ; (1)
              P_CALL    CLEAR_ARG3      ; (1)
                MOVLW   20              ;
                MOVWF   ARG3_0          ;
              P_CALL    DIVIDE          ; (3)   Divide it by 20. = .45 x DDS SYS CLK freq.
              P_CALL    ARG2_TO_ARG5    ; (1)   Save it as the upper limit. (ARG5)
                                        ;
                MOVLW   MAX_DDS_FREQ    ;       ***** Get the current MAX_DDS_FREQ  *****
              P_CALL    RAM_TO_ARG2     ; (2)   ***** Put it in ARG2 for adjustment *****
                                        ;
               _CALL    LCD_CLEAR       ; (3)   *********************************
               _MOVLW   MAX             ;       *  Clear the display.           *
               _CALL____LCD_TEXT        ; (3)   *  Display 'MAX DDS FREQ '.     *
                                        ;       *********************************
                                        ;
                BCF     TIMEOUT         ;       Clear the display timeout flag.
MAX_DDS:        BTFSC   TIMEOUT         ;       Has the display timeout been reached ?
               _GOTO    RESTORE_SETUP   ; (3)   Y. Ignore changes, Go to main display.
                                        ;
              P_CALL    CHK_KEYPAD      ; (5)   Chk for keypad I/P.
                                        ;
              P_CALL    UP_DOWN         ; (5)   Chk encoder (inc/dec ARG2, & chk limits)
                                        ;       and adjust step size.
                                        ;
              P_CALL    DISPLAY_FREQ    ; (4)   Display the variable.
                                        ;
              P_CALL    CHK_CAL_BRIEF   ; (1)
                BTFSS   PRESSED         ;       Was the CAL button pressed briefly ?
               _GOTO    MAX_DDS         ;       N. Continue updating the display.
                                        ;
                MOVLW   MAX_DDS_FREQ    ;
              P_CALL    ARG2_TO_RAM     ; (1)   Y. Save the maximum DDS freq to RAM.

;*****************************************************************************************

              P_CALL    SETUP_TO_EE     ; (3)   Save any changes to EEPROM.
                                        ;
                BTFSC   NO_PWR_METER    ;       is a power meter fitted ?
               _GOTO    RESTORE_SETUP   ;       N. All done.
                                        ;
                BTFSS   NOT_LTC5508_PWR ;       Using the 7Ghz power meter ?
               _GOTO    LTC5508_CAL     ;       Y. Calibrate the LTC5508 power meter.
                                        ;       N. Calibrate the AD8307 power meter.

;*****************************************************************************************




;*****************************************************************************************
;
;       Setup screen :-         Calibrate the AD8307 POWER METER slope.
;
;               Uses power meter command 4.
;               There is no timeout on this display.
;
;*****************************************************************************************

                MOVLW   CAL_LIM         ;       Get the 2 byte limit constant.
              P_CALL    CON_TO_ARG3     ; (2)
                                        ;       ***** Calculate the upper limit   *****
              P_CALL    ARG3_TO_ARG5    ; (1)   Save it as the upper limit. (ARG5)
                                        ;
              P_CALL    NEG_ARG3        ; (1)   ***** Calculate the lower limit   *****
              P_CALL    ARG3_TO_ARG4    ; (1)   Save it as the lower limit. (ARG4)
                                        ;
              P_CALL    CLEAR_ARG2      ; (1)   ***** Get the current slope.     *****
                MOVLW   15              ;       Power meter command 15 = Return the
              P_CALL    RX_I2C_BYTES    ; (4)   calibration slope and offset.
                BTFSS   RX_I2C_OK       ;       Were the bytes received ok.
               _GOTO    RESTORE_SETUP   ;       N. Ignore changes, Go to main display.
                                        ;
                MOVF    ARG3_3,W        ;       Y.    Get the two slope bytes and,
                MOVWF   ARG2_1          ;       ***** put them in ARG2 for adjustment *****
                MOVF    ARG3_2,W        ;
                MOVWF   ARG2_0          ;
               _CALL    ARG2_EXTEND_2   ; (1)   Sign extend to the 2 MSD bytes.
                                        ;
               _CALL    LCD_CLEAR       ; (3)
               _MOVLW   SLP_TEXT        ;       *********************************
               _CALL____LCD_TEXT        ; (3)   *  Display 'SLOPE  '.           *
                                        ;       *********************************
                MOVLW   H'C0'           ;
               _CALL    LCD_CMD         ; (2)   Goto LCD line 2 position 1.
                                        ;
               _MOVLW   PWR_TEXT        ;       *********************************
               _CALL____LCD_TEXT        ; (3)   *  Display 'POWER '.            *
                                        ;       *********************************
                                        ;
SEL_SLOPE:    P_CALL    UP_DN_1         ; (5)   Chk encoder. (inc/dec ARG2, & chk limits)
                                        ;
                MOVLW   H'89'           ;       Position the cursor, line 1 pos 10.
               _CALL    LCD_CMD         ; (2)
                                        ;
                MOVLW   6               ;
                MOVWF   DIGITS          ;       Set the number of digits to display.
                MOVLW   3               ;       Set the position of the DP.
              P_CALL    DISPLAY_NUMBER  ; (3)   Display the cal slope.
                                        ;
                BTFSS   METER_FLAG      ;       Is the the power meter update flag set ?
               _GOTO    SKIP_METER_1    ;       N. Skip the power meter commands.
                                        ;
                MOVLW   4               ;       Power meter command
              P_CALL    SET_TX_COMMAND  ; (4)   4 = Set the cal slope.
                MOVF    ARG2_1,W        ;       Get the MSD cal slope.
              P_CALL    I2C_TX_BYTE     ; (3)   Send the hi byte to the power meter.
                MOVF    ARG2_0,W        ;       Get the LSD cal slope.
              P_CALL    TX_BYTE_END     ; (4)   Send the low byte & finish the command.
                                        ;
              P_CALL    RX_COMMAND_14   ; (4)   Power meter command 14 = Return the power.
                                        ;
                MOVLW   H'C7'           ;       Position the cursor, line 2 pos 8.
               _CALL    DISP_POWER      ; (4)   Display the power from the meter.
                                        ;
SKIP_METER_1: P_CALL    CHK_KEY_BRIEF   ; (1)
                BTFSS   PRESSED         ;       Has any keypad button been pressed ?
               _GOTO    SKIP_REL        ;       N. Skip the relative dBr check.
                                        ;
                DECF    KEYPAD_NUM,W    ;       Get the key pad number -1.
                BTFSS   ZERO            ;       Was it key 1 ?
               _GOTO    SKIP_REL        ;
              P_CALL    KEY_1_FUNC      ;       Y. Start/stop relative dBr.
                                        ;
SKIP_REL:     P_CALL    CHK_CAL_BRIEF   ; (5)
                BTFSS   PRESSED         ;       Was the CAL button pressed briefly ?
               _GOTO    SEL_SLOPE       ;       N. Continue updating the display.

;*****************************************************************************************
;
;       Setup screen :-         Calibrate the AD8307 POWER METER offset.
;
;               Uses power meter command 5.
;               There is no timeout on this display.
;
;*****************************************************************************************

                MOVLW   6               ;       Power meter command
              P_CALL    SET_TX_COMMAND  ; (4)   6 = Stop/start relative power measurements
                CLRW                    ;       Stop relative power measurements
              P_CALL    TX_BYTE_END     ; (4)   Send the byte & finish the command.
                                        ;
                BCF     RELATIVE_FLAG   ;       Clear the relative dBr flag.
                                        ;
              P_CALL    CLEAR_ARG2      ; (1)   ***** Get the current offset.     *****
                MOVLW   15              ;       Power meter command 15 = Return the
              P_CALL    RX_I2C_BYTES    ; (4)   calibration slope and offset.
                BTFSS   RX_I2C_OK       ;       Were the bytes received ok.
               _GOTO    RESTORE_SETUP   ;       N. Ignore changes, Go to main display.
                                        ;
                MOVF    ARG3_1,W        ;       Y.    Get the two offset bytes and,
                MOVWF   ARG2_1          ;       ***** put them in ARG2 for adjustment *****
                MOVF    ARG3_0,W        ;
                MOVWF   ARG2_0          ;
               _CALL    ARG2_EXTEND_2   ; (1)   Sign extend to the 2 MSD bytes.
                                        ;
               _CALL    LCD_HOME        ; (2)   Goto LCD line 1 position 1.
                                        ;
               _MOVLW   OFF_TEXT        ;       *********************************
               _CALL____LCD_TEXT        ; (3)   *  Display 'OFFSET '.           *
                                        ;       *********************************
                                        ;
SEL_OFFSET:   P_CALL    UP_DN_1         ; (5)   Chk encoder. (inc/dec ARG2, & chk limits)
                                        ;
                MOVLW   H'87'           ;       Position the cursor, line 1 pos 8.
               _CALL    LCD_CMD         ; (2)
                                        ;
                MOVLW   6               ;       Set the number of digits to display.
              P_CALL    DISP_NUM_DP2    ; (3)   Display the cal offset.
                                        ;
               _MOVLW   DBM_TEXT        ;
               _CALL____LCD_TEXT        ; (3)   Display 'dBm'.
                                        ;
                BTFSS   METER_FLAG      ;       Is the the power meter update flag set ?
               _GOTO    SKIP_METER_2    ;       N. Skip the power meter commands.
                                        ;
                MOVLW   5               ;       Power meter command
              P_CALL    SET_TX_COMMAND  ; (4)   5 = Set the cal offset.
                MOVF    ARG2_1,W        ;       Get the MSD cal offset.
              P_CALL    I2C_TX_BYTE     ; (3)   Send the hi byte to the power meter.
                MOVF    ARG2_0,W        ;       Get the LSD cal offset.
              P_CALL    TX_BYTE_END     ; (4)   Send the low byte & finish the command.
                                        ;
              P_CALL    RX_COMMAND_14   ; (4)   Power meter command 14 = Return the power.
                                        ;
                MOVLW   H'C7'           ;       Position the cursor, line 2 pos 8.
               _CALL    DISP_POWER      ; (4)   Display the power from the meter.
                                        ;
SKIP_METER_2: P_CALL    CHK_CAL_BRIEF   ; (5)
                BTFSS   PRESSED         ;       Was the CAL button pressed briefly ?
               _GOTO    SEL_OFFSET      ;       N. Continue updating the display.

;-----------------------------------------------------------------------------------------
;               Confirm that changes are to be writen to the power meter.
;-----------------------------------------------------------------------------------------

               _CALL    CONFERM_SCRN    ;       See if changes are to be saved.
                                        ;
                BTFSS   ARG2_0,0        ;       Was the answer NO ?
               _GOTO    RESTORE_SETUP   ;       Y. Go back to the main VFO display.
                                        ;
                                        ;       Power meter command
              P_CALL    TX_COMMAND_7    ; (4)   7 = Saves changes made to the memory.
                MOVLW   1               ;       1 = Save changes to the setup
                                        ;           made by commands 0, 4 or 5.
EXIT_PWR_SETUP:                         ;
              P_CALL    TX_BYTE_END     ; (4)   Send the byte & finish the command.
                                        ;
                                        ;       Wait until the power meter has updated
               _CALL    WAIT_100MS      ; (2)   its EEPROM.
                                        ;
RESTORE_SETUP:P_CALL    EE_TO_RAM       ; (3)   Get setup & cal info from EEPROM.
                BCF     CAL_FLAG        ;       Flag that the cal screens are not in use.
              P_GOTO    RS_PWR_METER    ;       Reset the power meter software.
                                        ;       Return to the main display.

;*****************************************************************************************




;*****************************************************************************************
;
;       Setup screen :-         Calibrate the LTC5508 POWER METER.
;                               Select a memory to calibrate.
;
;*****************************************************************************************

LTC5508_CAL:  P_CALL    SETUP_TO_EE     ; (3)   Save any DDS changes to EEPROM.
              P_CALL    CLEAR_ARG4      ; (1)   Save 0 as the lower limit. (ARG4)
                                        ;
              P_CALL    CLEAR_ARG5      ; (1)
                MOVLW   5               ;
                MOVWF   ARG5_0          ;       Save 5 as the upper limit. (ARG5)
                                        ;
              P_CALL    CLEAR_ARG2      ; (1)   ***** Default = 0 = EXIT *****
                                        ;
               _CALL    LCD_CLEAR       ; (3)
               _MOVLW   MEM_TEXT        ;       *********************************
               _CALL____LCD_TEXT        ; (3)   *  Display 'CAL PWR MEMORY ?'.  *
                                        ;       *********************************
                                        ;
SEL_PWR_MEM:  P_CALL    UP_DN_1         ; (5)   Chk encoder. (inc/dec ARG2, & chk limits)
                                        ;
                MOVLW   H'C5'           ;       Position the cursor, line 2 pos 6.
               _CALL    LCD_CMD         ; (2)
                                        ;
                MOVF    ARG2_0,W        ;
                BTFSS   ZERO            ;       Selected num = 0 ?
               _GOTO    DISP_PWR_MEM    ;
                                        ;
               _MOVLW   EXIT_TEXT       ;
               _CALL____LCD_TEXT        ; (3)   Y. Display 'EXIT'.
               _GOTO    CHK_SEL_PWR     ;
                                        ;
DISP_PWR_MEM:  _CALL    LCD_SPACE       ; (2)   N. Display a space.
                MOVF    ARG2_0,W        ;
                ADDLW   '0'             ;
               _CALL    LCD_CHR         ; (2)      Display the mem number.
               _CALL    LCD_SPACE       ; (2)      Display a space.
               _CALL    LCD_SPACE       ; (2)      Display a space.
                                        ;
CHK_SEL_PWR:  P_CALL    CHK_CAL_BRIEF   ; (5)
                BTFSS   PRESSED         ;       Was the CAL button pressed briefly ?
               _GOTO    SEL_PWR_MEM     ;       N. Continue updating the display.
                                        ;
                MOVF    ARG2_0,W        ;
                BTFSC   ZERO            ;       EXIT selected ?
               _GOTO    RESTORE_SETUP   ; (3)   Y. Ignore changes, Go to main display.
                                        ;
                BSF     RP1             ;       Select bank 2 ram.
                MOVWF   lo CAL_NUM      ;       Save the power mem number to calibrate.
                CLRF    lo CAL_PTR      ;       Clear power table pointer.
                BCF     RP1             ;       Return to bank 0 ram.

;*****************************************************************************************
;
;       Setup screen :-         Calibrate the LTC5508 POWER METER.
;                               Select a memory frequency text. 0 to 99 = 0.0G to 9.9G
;
;*****************************************************************************************

               _CALL    LCD_CLEAR       ; (3)
               _MOVLW   SEL_TEXT        ;       *********************************
               _CALL____LCD_TEXT        ; (3)   *  Display 'MEM   TEXT'.        *
                                        ;       *********************************
                                        ;
                MOVLW   H'84'           ;       Position the cursor, line 1 pos 5.
               _CALL    LCD_CMD         ; (2)
                MOVF    ARG2_0,W        ;
                ADDLW   '0'             ;
               _CALL    LCD_CHR         ; (2)   Display the mem number.
                                        ;
              P_CALL    CLEAR_ARG4      ; (1)   Save 0 as the lower limit. (ARG4)
                                        ;
              P_CALL    CLEAR_ARG5      ; (1)
                MOVLW   99              ;
                MOVWF   ARG5_0          ;       Save 99 as the upper limit. (ARG5)
                                        ;
              P_CALL    CLEAR_ARG2      ; (1)
               _CALL    GET_TBL_CAL     ; (5)   Get the power meter cal values.
                BTFSS   RX_I2C_OK       ;       Were the bytes received ok.
               _GOTO    RESTORE_SETUP   ;       N. Ignore changes, Go to main display.
                MOVF    ARG3_0,W        ;
                MOVWF   ARG2_0          ;       ***** Get current frequency text. *****
                                        ;
SEL_MEM_TEXT: P_CALL    UP_DN_1         ; (5)   Chk encoder. (inc/dec ARG2, & chk limits)
                                        ;
                MOVLW   H'8C'           ;       Position the cursor, line 1 pos 13.
               _CALL    LCD_CMD         ; (2)
                                        ;
                MOVLW   2               ;       Set the number of digits to display.
              P_CALL    DISP_NUM_DP2    ; (3)   Display the freq/mem number.
                                        ;
                MOVLW   'G'             ;
               _CALL    LCD_CHR         ; (2)
                                        ;
              P_CALL    CHK_CAL_BRIEF   ; (5)
                BTFSS   PRESSED         ;       Was the CAL button pressed briefly ?
               _GOTO    SEL_MEM_TEXT    ;       N. Continue updating the display.
                                        ;
                MOVF    ARG2_0,W        ;       Get the freq character.
               _CALL    SAVE_BYTE       ; (1)   Save the result. Will be sent to the power
                                        ;       meter when all bytes are done.

;*****************************************************************************************
;
;       Setup screen :-         Calibrate the LTC5508 POWER METER.
;                               Calibrate a lookup table entry.
;
;*****************************************************************************************

               _CALL    LCD_CLEAR       ; (3)
                MOVLW   2               ;
              P_CALL    DISP_NUM_DP2    ; (3)   Display the freq text.
                MOVLW   'G'             ;
               _CALL    LCD_CHR         ; (2)
                                        ;
                MOVLW   CAL_30_DBM      ;
              P_CALL    CON_TO_ARG3     ; (2)
              P_CALL    NEG_ARG3        ; (1)   ARG3 = -30.0dBm.
              P_CALL    PUSH_ARG3       ; (1)   Save it.
                                        ;
              P_CALL    CLEAR_ARG4      ; (1)   Save 0 as the initial lower limit. (ARG4)
                                        ;
                MOVLW   17              ;
                MOVWF   COUNT_4         ;       17 table values to be done.
                                        ;
NEXT_TABLE:   P_CALL    ARG4_TO_ARG3    ; (1)   Get the lower limit.
                                        ;
              P_CALL    CLEAR_ARG2      ; (1)
                COMF    ARG2_0,F        ;       ARG2 = 255.
              P_CALL    ADD             ; (1)   ARG2 = min limit + 255.
                MOVLW   CON_1023        ;
              P_CALL    CON_TO_ARG3     ; (2)
              P_CALL    MINIMUM         ; (3)   Make sure it is not over 1023.
              P_CALL    ARG2_TO_ARG5    ; (1)   Save it as the upper limit. (ARG5)
                                        ;
              P_CALL    CLEAR_ARG3      ; (1)
                MOVLW   25              ;
                MOVWF   ARG3_0          ;       ARG3 = 2.5dBm.
                                        ;
              P_CALL    PULL_ARG2       ; (1)   Get the saved power.
              P_CALL    ADD             ; (1)   Add 2.5dBm to the power.
              P_CALL    PUSH_ARG2       ; (1)   Save it.
                                        ;
                MOVLW   H'87'           ;       Position the cursor, line 1 pos 8.
               _CALL    LCD_CMD         ; (2)
                                        ;
                MOVLW   5               ;       Set the number of digits to display.
              P_CALL    DISP_NUM_DP2    ; (3)   Display the power level.
                                        ;
               _MOVLW   DBM_TEXT        ;
               _CALL____LCD_TEXT        ; (3)   Display 'dBm'.
                                        ;
              P_CALL    CLEAR_ARG2      ; (1)
               _CALL    GET_TBL_CAL     ; (5)   Get the power meter cal values.
                BTFSS   RX_I2C_OK       ;       Were the bytes received ok.
               _GOTO    RESTORE_SETUP   ;       N. Ignore changes, Go to main display.
                MOVF    ARG3_0,W        ;
                MOVWF   ARG2_0          ;       Get current lookup table value.
                                        ;
              P_CALL    ARG4_TO_ARG3    ; (1)   Get the lower limit.
              P_CALL    ADD             ; (1)   Add the table value to the lower limit.
                                        ;
CAL_TBL_PWR:  P_CALL    UP_DN_1         ; (5)   Chk encoder. (inc/dec ARG2, & chk limits)
                                        ;
                MOVLW   H'CC'           ;       Position the cursor, line 2 pos 13.
               _CALL    LCD_CMD         ; (2)
                                        ;
                BTFSS   MEM_1_SEC       ;       Has the MEM buton been pressed for 1 S ?
               _GOTO    TRY_2_SEC       ;
                MOVLW   4               ;
               _CALL    LCD_SPACES      ; (2)   Y. Display 4 spaces.
               _GOTO    CAL_TBL_END     ;

TRY_2_SEC:      BTFSS   MEM_2_SEC       ;       Has the MEM buton been pressed for 2 S ?
                GOTO    TRY_2_END       ;
              P_CALL    CLEAR_ARG2      ; (1)
               _CALL    GET_TBL_CAL     ; (5)   Y. Get the power meter cal values.
                BTFSS   RX_I2C_OK       ;          Were the bytes received ok.
               _GOTO    RESTORE_SETUP   ;          N. Ignore changes, Go to main display.
                MOVF    ARG3_2,W        ;
                MOVWF   ARG2_0          ;          Get current A/D converter value.
                MOVF    ARG3_3,W        ;
                MOVWF   ARG2_1          ;
                BCF     MEM_2_SEC       ;          Clear the 2 S MEM flag.
TRY_2_END:                              ;
                MOVLW   4               ;
                MOVWF   DIGITS          ;       Set the number of digits to display.
                MOVLW   6               ;       Set the position of the DP.
              P_CALL    DISPLAY_NUMBER  ; (3)   Display the lookup table value.
                                        ;
CAL_TBL_END:  P_CALL    CHK_CAL_BRIEF   ; (5)
                BTFSS   PRESSED         ;       Was the CAL button pressed briefly ?
               _GOTO    CAL_TBL_PWR     ;       N. Continue updating the display.
                                        ;
              P_CALL    PUSH_ARG2       ; (1)   Y. Save the tabel result.
                                        ;
              P_CALL    ARG4_TO_ARG3    ; (1)   Get the lower limit.
                                        ;
              P_CALL    SUB             ; (1)   Take the lower limit from the table value.
                MOVF    ARG2_0,W        ;       Get the the byte to save.
               _CALL    SAVE_BYTE       ; (1)   Save the result. Will be sent to the power
                                        ;       meter when all bytes are done.
                                        ;
              P_CALL    PULL_ARG3       ; (1)   Get the saved tabel result.
              P_CALL    ARG3_TO_ARG4    ; (1)   Save it as the lower limit. (ARG4)
                                        ;
                DECFSZ  COUNT_4,F       ;       All table values done ?
               _GOTO    NEXT_TABLE      ;       N. Do the next table value.
                                        ;
              P_CALL    PULL_ARG3       ; (1)   Scrap the saved power on the stack.

;-----------------------------------------------------------------------------------------
;               Confirm that changes are to be writen to the power meter.
;-----------------------------------------------------------------------------------------

               _CALL    CONFERM_SCRN    ;       See if changes are to be saved.
                                        ;
                BTFSS   ARG2_0,0        ;       Was the answer NO ?
               _GOTO    LTC5508_CAL     ;       Y. Go back and select another cal memory

;-----------------------------------------------------------------------------------------
;               The save changes has been confirmed, now write to the power meter.
;-----------------------------------------------------------------------------------------

                MOVLW   18              ;
                MOVWF   COUNT_4         ;       18 table values to be done.
                                        ;
                BSF     RP1             ;       Select bank 2 ram.
                CLRF    lo CAL_PTR      ;       Clear power table pointer.
                BCF     RP1             ;       Return to bank 0 ram.
                                        ;
SAVE_TBL_CAL:   MOVLW   5               ;       Power lookup table address command.
              P_CALL    SET_TX_COMMAND  ; (4)
                BSF     RP1             ;       Select bank 2 ram.
                MOVF    lo CAL_NUM,W    ;       Get the memory number of the table.
                BCF     RP1             ;       Return to bank 0 ram.
              P_CALL    I2C_TX_BYTE     ; (3)   Send the hi byte to the power meter.
                BSF     RP1             ;       Select bank 2 ram.
                MOVF    lo CAL_PTR,W    ;       Get the table pointer.
                BCF     RP1             ;       Return to bank 0 ram.
              P_CALL    TX_BYTE_END     ; (4)   Send the lo byte & finish the command.
                                        ;
                MOVLW   4               ;       Power lookup table byte command.
              P_CALL    SET_TX_COMMAND  ; (4)
                MOVLW   0               ;       Dummy byte.
              P_CALL    I2C_TX_BYTE     ; (3)   Send the hi byte to the power meter.
                BSF     RP1             ;       Select bank 2 ram.
                BSF     IRP             ;       Select indirect bank 2 & 3.
                MOVLW   low CAL_TBL     ;       Get the base address of the table.
                ADDWF   lo CAL_PTR,W    ;       Add the table pointer.
                MOVWF   FSR             ;
                MOVF    INDF,W          ;       Get a byte from the cal table.
                INCF    lo CAL_PTR,F    ;       Inc the power table pointer.
                BCF     IRP             ;       Return to indirect bank 0 & 1.
                BCF     RP1             ;       Return to bank 0 ram.
              P_CALL    TX_BYTE_END     ; (4)   Send the byte & finish the command.
                                        ;
              P_CALL    TX_COMMAND_7    ; (4)   7 = Saves changes made to the memory.
                MOVLW   2               ;       2 = Save changes to the setup
                                        ;           made by commands 4/5.
              P_CALL    TX_BYTE_END     ; (4)   Send the byte & finish the command.
                                        ;
                MOVLW   50              ;       Wait until the power meter has updated
               _CALL    MS_WAIT         ; (2)   its EEPROM.
                                        ;
                DECFSZ  COUNT_4,F       ;       All table values done ?
               _GOTO    SAVE_TBL_CAL    ;       N. Do the next table value.
               _GOTO    LTC5508_CAL     ;       Y. Go back and select another cal memory

;*****************************************************************************************
;               End of the LTC5508 POWER METER calibration screens.
;*****************************************************************************************




;*****************************************************************************************
;
;       Setup screen :-         Select the POWER METER memory attenuator.
;
;               Uses power meter command 3.
;
;*****************************************************************************************

SETUP_SCRNS:    MOVLW   ATTEN_LIM       ;       Get the 2 byte limit constant.
              P_CALL    CON_TO_ARG3     ; (2)
                                        ;       ***** Calculate the upper limit   *****
              P_CALL    ARG3_TO_ARG5    ; (1)   Save it as the upper limit. (ARG5)
                                        ;
              P_CALL    NEG_ARG3        ; (1)   ***** Calculate the lower limit   *****
              P_CALL    ARG3_TO_ARG4    ; (1)   Save it as the lower limit. (ARG4)
                                        ;
              P_CALL    CLEAR_ARG2      ; (1)   ***** Get the current attenuation. *****
                MOVLW   9               ;       Power meter command 9 = Return the mem
              P_CALL    RX_I2C_BYTES    ; (4)   calibration offset and the attenuation.
                BTFSS   RX_I2C_OK       ;       Were the bytes received ok.
               _GOTO    RESTORE_SETUP   ;       N. Ignore changes, Go to main display.
                                        ;
                MOVF    ARG3_1,W        ;       Y.    Get the two attenuation bytes and,
                MOVWF   ARG2_1          ;       ***** put them in ARG2 for adjustment *****
                MOVF    ARG3_0,W        ;
                MOVWF   ARG2_0          ;
               _CALL    ARG2_EXTEND_2   ; (1)   Sign extend to the 2 MSD bytes.
                                        ;
               _CALL    LCD_CLEAR       ; (3)   Clear the display.
                                        ;
               _CALL    DISP_MEM_NUM    ;       Display the power meter memory.
                                        ;          (ie ' 2.4G' or ' P4 ')
                                        ;
               _MOVLW   ATT_TEXT        ;       ********************************
               _CALL____LCD_TEXT        ; (3)   *  Display 'xxxx ATTENUATOR'.  *
                                        ;       ********************************
                                        ;
                BCF     TIMEOUT         ;       Clear the display timeout flag.
SEL_ATTEN:      BTFSC   TIMEOUT         ;       Has the display timeout been reached ?
               _GOTO    RESTORE_SETUP   ;       Y. Ignore changes, Go to main display.
                                        ;
              P_CALL    UP_DN_1         ; (5)   Chk encoder. (inc/dec ARG2, & chk limits)
                                        ;
                MOVLW   H'C7'           ;       Position the cursor, line 2 pos 8.
               _CALL    LCD_CMD         ; (2)
                                        ;
                MOVLW   5               ;       Set the number of digits to display.
              P_CALL    DISP_NUM_DP2    ; (3)   Display the attenuator.
                                        ;
               _MOVLW   DB_TEXT         ;
               _CALL____LCD_TEXT        ; (3)   Display 'dB'.
                                        ;
              P_CALL    CHK_CAL_BRIEF   ; (5)
                BTFSS   PRESSED         ;       Was the CAL button pressed briefly ?
               _GOTO    SEL_ATTEN       ;       N. Continue updating the display.
                                        ;
                MOVLW   3               ;       Y. Power meter command
              P_CALL    SET_TX_COMMAND  ; (4)      3 = Set the attenuator in use.
                MOVF    ARG2_1,W        ;
              P_CALL    I2C_TX_BYTE     ; (3)      Send the hi byte to the power meter.
                MOVF    ARG2_0,W        ;
              P_CALL    TX_BYTE_END     ; (4)      Send the lo byte & finish the command.

;*****************************************************************************************
;
;       Setup screen :-         Calibrate the POWER METER memory offset.
;
;               Uses power meter command 2.
;
;*****************************************************************************************

                MOVLW   MEM_OFFSET_LIM  ;       Get the 2 byte limit constant.
              P_CALL    CON_TO_ARG3     ; (2)
                                        ;       ***** Calculate the upper limit   *****
              P_CALL    ARG3_TO_ARG5    ; (1)   Save it as the upper limit. (ARG5)
                                        ;
              P_CALL    NEG_ARG3        ; (1)   ***** Calculate the lower limit   *****
              P_CALL    ARG3_TO_ARG4    ; (1)   Save it as the lower limit. (ARG4)
                                        ;
              P_CALL    CLEAR_ARG2      ; (1)   ***** Get the current offset. *****
                MOVLW   9               ;       Power meter command 9 = Return the mem
              P_CALL    RX_I2C_BYTES    ; (4)   calibration offset and the attenuation.
                BTFSS   RX_I2C_OK       ;       Were the bytes received ok.
               _GOTO    RESTORE_SETUP   ;       N. Ignore changes, Go to main display.
                                        ;
                MOVF    ARG3_3,W        ;       Y.    Get the cal offset bytes and,
                MOVWF   ARG2_1          ;       ***** place into ARG2 for adjustment *****
                MOVF    ARG3_2,W        ;
                MOVWF   ARG2_0          ;
               _CALL    ARG2_EXTEND_2   ; (1)   Sign extend to the 2 MSD bytes.
                                        ;
               _CALL    LCD_CLEAR       ; (3)   Clear the display.
                                        ;
               _CALL    DISP_MEM_NUM    ;       Display the power meter memory.
                                        ;          (ie ' 2.4G' or ' P4 ')
                MOVLW   H'C0'           ;
               _CALL    LCD_CMD         ; (2)   Goto LCD line 2 position 1.
                                        ;       *********************************
               _MOVLW   OFF_TEXT        ;       *  Display 'xxxx '.             *
               _CALL____LCD_TEXT        ; (3)   *          'OFFSET '.           *
                                        ;       *********************************
                                        ;
SEL_MEM_CAL:    BTFSC   TIMEOUT         ;       Has the display timeout been reached ?
               _GOTO    RESTORE_SETUP   ;       Y. Ignore changes, Go to main display.
                                        ;
              P_CALL    UP_DN_1         ; (5)   Chk encoder. (inc/dec ARG2, & chk limits)
                                        ;
                MOVLW   H'C8'           ;       Position the cursor, line 1 pos 9.
               _CALL    LCD_CMD         ; (2)
                                        ;
                MOVLW   4               ;       Set the number of digits to display.
              P_CALL    DISP_NUM_DP2    ; (3)   Display the mem offset.
                                        ;
               _MOVLW   DB_TEXT         ;
               _CALL____LCD_TEXT        ; (3)   Display 'dB'.
                                        ;
                BTFSS   METER_FLAG      ;       Is the the power meter update flag set ?
               _GOTO    SKIP_METER_3    ;       N. Skip the power meter commands.
                                        ;
                MOVLW   2               ;       Power meter command
              P_CALL    SET_TX_COMMAND  ; (4)   2 = Set the mem offset.
                MOVF    ARG2_1,W        ;
              P_CALL    I2C_TX_BYTE     ; (3)   Send the hi byte to the power meter.
                MOVF    ARG2_0,W        ;
              P_CALL    TX_BYTE_END     ; (4)   Send the lo byte & finish the command.
                                        ;
              P_CALL    RX_COMMAND_14   ; (4)   Power meter command 14 = Return the power.
                                        ;
                MOVLW   H'87'           ;       Position the cursor, line 1 pos 8.
               _CALL    DISP_POWER      ; (4)   Display the power from the meter.
                                        ;
SKIP_METER_3: P_CALL    CHK_CAL_BRIEF   ; (5)
                BTFSS   PRESSED         ;       Was the CAL button pressed briefly ?
               _GOTO    SEL_MEM_CAL     ;       N. Continue updating the display.


;-----------------------------------------------------------------------------------------
;               Confirm that changes are to be writen to the power meter.
;-----------------------------------------------------------------------------------------

               _CALL    CONFERM_SCRN    ;       See if changes are to be saved.
                                        ;
                BTFSS   ARG2_0,0        ;       Was the answer NO ?
               _GOTO    RESTORE_SETUP   ;       Y. Go back to the main VFO display.
                                        ;
                                        ;       Power meter command
              P_CALL    TX_COMMAND_7    ; (4)   7 = Saves changes made to the memory.
                MOVLW   0               ;       0 = Save changes to the setup
                                        ;           made by commands 1, 2 or 3.
                                        ;
               _GOTO    EXIT_PWR_SETUP  ; (3)   Get setup & cal info from EEPROM.
                                        ;       Reset the power meter then return to the
                                        ;       main VFO display.

;*****************************************************************************************





;*****************************************************************************************
;               Confirm that changes are to be writen to the power meter.
;*****************************************************************************************

CONFERM_SCRN: P_CALL    CLEAR_ARG4      ; (1)   Save 0 as the lower limit. (ARG4)
                                        ;
              P_CALL    CLEAR_ARG5      ; (1)
                INCF    ARG5_0,F        ;       Save 1 as the upper limit. (ARG5)
                                        ;
              P_CALL    CLEAR_ARG2      ; (1)   ***** Initial response is 'NO'      *****
                                        ;       ***** Put it in ARG2 for adjustment *****
                                        ;
               _CALL    LCD_CLEAR       ; (3)   *********************************
               _MOVLW   SAVE_TEXT       ;       *  Clear the display.           *
               _CALL____LCD_TEXT        ; (3)   *  Display ' SAVE CHANGES ?'.   *
                                        ;       *********************************
                                        ;
ANSWER_LOOP:  P_CALL    UP_DN_1         ; (5)   Chk encoder. (inc/dec ARG2, & chk limits)
                                        ;
                MOVLW   H'C6'           ;
               _CALL    LCD_CMD         ; (2)   Position the cursor, line 2 pos 7.
                                        ;
               _MOVLW   NO_TEXT         ;       *********************************
                BTFSC   ARG2_0,0        ;       *  Display ' NO' if ARG2 = 0.   *
               _MOVLW   YES_TEXT        ;       *  Display 'YES' if ARG2 = 1.   *
               _CALL____LCD_TEXT        ; (3)   *********************************
                                        ;
              P_CALL    CHK_CAL_BRIEF   ; (1)
                BTFSS   PRESSED         ;       Was the MEM button pressed briefly ?
               _GOTO    ANSWER_LOOP     ;       N. Wait for an answer.
                RETURN                  ;

;*****************************************************************************************




;*****************************************************************************************
;
;     NAMES:    GET_TBL_CAL             Get the 4 calibration bytes from the 7Ghz
;                                       power meter, place in ARG3. Uses command 5 to
;                                       set the calibration address to read from.
;
;     INPUT:    None.
;
;    OUTPUT:    ARG3.
;
; VARIABLES:    COUNT_0, COUNT_1, TEMP_1, I2C_DATA_WR.
;
; STACK USE:    4
;
;*****************************************************************************************

GET_TBL_CAL:    MOVLW   5               ;       Power lookup table address command
              P_CALL    SET_TX_COMMAND  ; (4)
                BSF     RP1             ;       Select bank 2 ram.
                MOVF    lo CAL_NUM,W    ;       Get the memory number of the table.
                BCF     RP1             ;       Return to bank 0 ram.
              P_CALL    I2C_TX_BYTE     ; (3)   Send the hi byte to the power meter.
                BSF     RP1             ;       Select bank 2 ram.
                MOVF    lo CAL_PTR,W    ;       Get the table pointer.
                BCF     RP1             ;       Return to bank 0 ram.
              P_CALL    TX_BYTE_END     ; (4)   Send the lo byte & finish the command.
                                        ;
                MOVLW   50              ;
               _CALL    MS_WAIT         ;       Wait 50mS. Until meter is in cal mode.
                                        ;
                MOVLW   15              ;       Power meter command 15 = Return the power
              P_CALL    RX_I2C_BYTES    ; (4)   meter calibration values.
                MOVF    ARG3_1,W        ;
                BTFSS   ZERO            ;       Was the 3rd byte = 0 ? (0 = valid cal)
                BCF     RX_I2C_OK       ;       N. Clear the received ok bit.
                RETURN                  ;

;*****************************************************************************************
;
;     NAMES:    SAVE_BYTE               Save a byte to the 18 byte cal buffer, so that the
;                                       18 bytes can later be sent to the power meter.
;                                       CAL_PTR must have been set to point to the
;                                       location in the buffer to save the byte.
;
;     INPUT:    W = byte to save.
;
;    OUTPUT:    None.
;
; VARIABLES:    FSR.
;
; STACK USE:    0
;
;*****************************************************************************************

SAVE_BYTE:      BSF     RP1             ;       Select bank 2 ram.
                BSF     IRP             ;       Select indirect bank 2 & 3.
                MOVWF   lo CAL_TEMP     ;       Save the byte.
                MOVLW   low CAL_TBL     ;       Get the base address of the table.
                ADDWF   lo  CAL_PTR,W   ;       Add the table pointer.
                MOVWF   FSR             ;
                MOVF    lo CAL_TEMP,W   ;       Get the byte.
                MOVWF   INDF            ;       Place it in the cal table.
                INCF    lo CAL_PTR,F    ;       Inc the power table pointer.
                BCF     IRP             ;       Return to indirect bank 0 & 1.
                BCF     RP1             ;       Return to bank 0 ram.
                RETURN                  ;

;*****************************************************************************************
;
;     NAMES:    ARG2_EXTEND_2           Sign extend the two lower bytes in ARG2 to the
;                                       upper 2 bytes . Assumes the upper 2 bytes are 0.
;
;     INPUT:    ARG2.
;
;    OUTPUT:    ARG2.
;
; VARIABLES:    None.
;
; STACK USE:    0
;
;*****************************************************************************************

ARG2_EXTEND_2:  BTFSS   ARG2_1,7        ;       Sign extend to the 2 MSD bytes.
                RETURN                  ;
                COMF    ARG2_2,F        ;
                COMF    ARG2_3,F        ;
                RETURN                  ;

;*****************************************************************************************
;
;     NAMES:    DISP_LINE_1
;
;     INPUT:    None.
;
;    OUTPUT:    None.
;
; VARIABLES:    TEMP1, TEMP_3, COUNT_0.
;
; STACK USE:    4
;
;*****************************************************************************************

DISP_LINE_1:   _CALL    LCD_HOME        ; (2)   Goto the start of line 1.
                BTFSS	SWEEP_ACTIVE    ;       Is the sweep function active ?
               _GOTO    DISP_MEM        ;

              P_CALL    DISP_RATE       ; (2)   Y. Display sweep rate number.
               _CALL    LCD_SPACE       ; (2)      Display a space.
               _CALL    LCD_SPACE       ; (2)      Display a space.
               _GOTO    DISP_PWR        ;

DISP_MEM:       BTFSS   MEM_ACTIVE      ;       Is the DDS MEM function active ?
               _GOTO    DISP_VFO        ;
                                        ;
                MOVLW   'M'             ;
               _CALL    LCD_CHR         ; (2)   Y. Display a 'M'.
                MOVF    MEM_H,W         ;          Display the MEM number.
               _CALL    LCD_CHR         ; (2)
                MOVF    MEM_L,W         ;
               _CALL    LCD_CHR         ; (2)
               _GOTO    DISP_PWR        ;          ie. 'M12'
                                        ;
DISP_VFO:      _MOVLW   VFO             ;       N. Display 'VFO'
               _CALL____LCD_TEXT        ; (3)
                                        ;
		MOVLW	13
		CALL	LCD_SPACES

DISP_PWR:      _CALL    LCD_SPACE       ; (2)   Display a space.
                                        ;
                BTFSC   NO_PWR_METER    ;       Is the power meter fitted ?
                RETURN                  ;       N. All done. Just return.
                                        ;
               _CALL    DISP_MEM_NUM    ; (4)   Display the power meter memory.
               _GOTO    UPDATE_POWER    ;          (ie '1.2G' or ' P4 ')
                                        ;
DISP_POWER:    _CALL    LCD_CMD         ; (2)   Set the LCD position.
               _CALL    LCD_SPACE       ; (2)   Display a space.
UPDATE_POWER:   BTFSS   METER_FLAG      ;       Time to update the power meter display ?
                RETURN                  ;       N. All done.
              P_CALL    PWR_TO_LCD      ; (4)   Y. Get the current power and display it.
                RETURN                  ;

;-----------------------------------------------------------------------------------------

DISP_MEM_NUM:   BTFSC   NOT_LTC5508_PWR ;
               _GOTO    P_MEM_DISP      ;       Using the LTC5508 power meter.
                                        ;
              P_CALL    PUSH_ARG2       ; (1)   Y. Save ARG2.
                                        ;
              P_CALL    CLEAR_ARG2      ; (1)
                MOVF    MEM_FREQ,W      ;          Get the mem freq byte.
                MOVWF   ARG2_0          ;
                                        ;
                MOVLW   2               ;          Set the number of digits to display.
              P_CALL    DISP_NUM_DP2    ; (3)      Display the freq/mem number.
                                        ;
              P_CALL    PULL_ARG2       ; (1)      Recover ARG2.
                                        ;
                MOVLW   'G'             ;
               _GOTO    FLASH_CHR       ; (2)      Display G. (ie '2.4G')
                                        ;          Return via "GOTO".
                                        ;
P_MEM_DISP:    _CALL    LCD_SPACE       ; (2)   N. Display a space.
                MOVLW   'P'             ;
               _CALL    FLASH_CHR       ; (2)      Display P.
                MOVF    PWR_MEM,W       ;
                ADDLW   '0'             ;          Display the power meter memory number.
               _CALL    LCD_CHR         ; (2)      (1 to 8). (ie ' P4 ')
               _GOTO    LCD_SPACE       ; (2)      Display a space.
                                        ;          Return via "GOTO".

;-----------------------------------------------------------------------------------------


FLASH_CHR:      BTFSS   EN_MEM_SEL      ;       Is the select power meter mem active ?
               _GOTO    NON_FLASH       ;
                BTFSS   FLASHER_COUNT,4 ;       Y. Change the chr to a ' ' every 1/8 sec.
                MOVLW   ' '             ;
NON_FLASH:     _GOTO    LCD_CHR         ; (2)   Display the character.
                                        ;       Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    LCD_TEXT_0, LCD_TEXT_1
;
;   PURPOSE:    Display on the LCD, the string of characters pointed to by W.
;               The last char of the string must be a 0.
;               LCD_TEXT_0 is used for text strings stored in page 0.
;               LCD_TEXT_1 is used for text strings stored in page 1.
;
;               The  _CALL____LCD_TEXT macro is used to assemble the correct subroutine.
;               The  _MOVLW macro is used identify which page the text string is from.
;
;     INPUT:    W = String offset address. (offset from the base of the text table)
;
;    OUTPUT:    None.
;
; VARIABLES:    TEMP1, TEMP_3, COUNT_0.
;
; STACK USE:    2
;
;*****************************************************************************************

LCD_TEXT_1:     BSF     PCLATH,0        ;       Select page 1.
LCD_TEXT_0:     MOVWF   TEMP_1          ;       Save the text start address.
               _CALL    TABLE_0         ; (1)
                ANDLW   H'FF'           ;
                BTFSC   ZERO            ;       At end of message ? (0 returned at end)
               _GOTO    LCD_TEXT_EXIT   ;
               _CALL    LCD_CHR         ; (2)   N. Display character.
                INCF    TEMP_1,W        ;          Point to next character.
               _GOTO    LCD_TEXT_0      ;
LCD_TEXT_EXIT:  BCF     PCLATH,0        ;       Return to page 0.
                RETURN                  ;       All done.

;*****************************************************************************************
;
;     NAMES:    LCD_SPACES      Display W spaces.
;
; VARIABLES:    TEMP_3, COUNT_0.
;
; STACK USE:    2
;
;*****************************************************************************************

LCD_SPACES:     MOVWF   COUNT_0         ;       Save the number of spaces.
SPACE_LOOP:    _CALL    LCD_SPACE       ; (2)   Display a space.
                DECFSZ  COUNT_0,F       ;
               _GOTO    SPACE_LOOP      ;       Loop until count = 0.
                RETURN                  ;

;*****************************************************************************************
;
;     NAMES:    LCD_CLEAR       Clear the display and home the cursor.
;
; VARIABLES:    TEMP_3, COUNT_0.
;
; STACK USE:    2
;
;*****************************************************************************************
;
;     NAMES:    WAIT_8MS        Do nothing for 8 mS.
;               WAIT_100MS      Do nothing for 100 mS.
;               MS_WAIT         W = The number of mS to wait.
;
; VARIABLES:    COUNT_0.
;
; STACK USE:    1
;
;*****************************************************************************************

WAIT_100MS:     MOVLW   100             ;       Set the outer loop to 100mS.
               _GOTO    MS_WAIT         ;

LCD_CLEAR:      MOVLW   H'01'           ;       Clear the display. Then wait 8mS.
               _CALL    LCD_CMD         ; (2)

WAIT_8MS:       MOVLW   8               ;       Set the outer loop to 8mS.
MS_WAIT:        MOVWF   COUNT_0         ;       Save the number of mS delay.
OUTER_LOOP:     CLRW                    ;       Set the wait loop to 256.
               _CALL    WAIT_LOOP       ; (1)   256 loops x 4 x 1uS = 1.024mS.
                DECFSZ  COUNT_0,F       ;       Dec the mS counter.
               _GOTO    OUTER_LOOP      ;
                RETURN                  ;

;*****************************************************************************************
;
;     NAMES:    LCD_HOME        Home the cursor.
;               LCD_SPACE       Display a space.
;               LCD_CMD         Send the command byte in W to the LCD.
;               LCD_CHR         Send the data byte in W to the LCD.
;
; VARIABLES:    TEMP_3
;
; STACK USE:    1
;
;*****************************************************************************************

LCD_HOME:       MOVLW   H'80'           ;       Position the cursor, line 1 pos 1.
LCD_CMD:        BCF     LCD_RS          ;       Select the instruction register.
               _GOTO    LCD_BYTE        ; (1)   Send the byte to the LCD.

LCD_SPACE:      MOVLW   " "             ;       Display a space.
LCD_CHR:        BSF     LCD_RS          ;       Select display data register.
LCD_BYTE:       MOVWF   TEMP_3          ;       Save the byte to send to the LCD.
                BCF     GIE             ;       Disable interrupts.
               _CALL    SEND_NIBBLE     ; (1)   Send the high nibble to the LCD.
                SWAPF   TEMP_3,F        ;
               _CALL    SEND_NIBBLE     ; (1)   Send the low nibble to the LCD.
                BSF     GIE             ;       Enable interrupts.
                RETURN                  ;

;-----------------------------------------------------------------------------------------

 IF SIMULATE
SEND_NIBBLE:                            ;       If using the MPLAB simulator
PULSE_LCD_E:                            ;       dont bother doing the subroutines.
WAIT_LOOP:      RETURN                  ;
 ELSE

SEND_NIBBLE:    MOVLW   B'00001111'     ;
                ANDWF   PORTB,F         ;       Clear the high nibble of port B.
                MOVF    TEMP_3,W        ;       Get the byte of data.
                ANDLW   B'11110000'     ;       Keep only the high nibble.
                IORWF   PORTB,F         ;       Send the nibble to port B.
PULSE_LCD_E:    MOVLW   256-50          ;       Set the wait loop to 200uS.
                BCF     LCD_E           ;       Pulse the LCD_E lead low.
                BSF     LCD_E           ;
WAIT_LOOP:      ADDLW   1               ;       1 CLK cycles \
                BTFSS   ZERO            ;       1     "       > 4 CLKs x 1uS
               _GOTO    WAIT_LOOP       ;       2     "      /
                RETURN                  ;
 ENDIF

;*****************************************************************************************
;
;      NAME:    INIT_LCD
;
;   PURPOSE:    Initialise the 16 x 2 liquid crystal display.
;               The LCD controller chip must be equivalent to the HITACHI 44780.
;
;     INPUT:    None.
;
;    OUTPUT:    None.
;
; VARIABLES:    COUNT_0.
;
; STACK USE:    3
;
;*****************************************************************************************

INIT_LCD:      _CALL    LCD_RESET       ; (2)   Reset the LCD.
               _CALL    LCD_RESET       ; (2)   Reset the LCD.
               _CALL    LCD_RESET       ; (2)   Reset the LCD.
                MOVLW   B'00101000'     ;       LCD 4 bit mode command.
                MOVWF   PORTB           ;
               _CALL    PULSE_LCD_E     ; (1)   Pulse the LCD E lead.
                MOVLW   B'00101000'     ;       Set LCD to 2 line, 5x7 dot.
               _CALL    LCD_CMD         ; (2)
                MOVLW   B'00001000'     ;       Turn the display off.
               _CALL    LCD_CMD         ; (2)
               _CALL    LCD_CLEAR       ; (3)   Clear the display.
                MOVLW   B'00000110'     ;       Cursor increments, no display shift.
               _CALL    LCD_CMD         ; (2)
LCD_NORM:       MOVLW   B'00001100'     ;       Turn the display on.
               _GOTO    LCD_CMD         ; (1)   Return via "GOTO".

;-----------------------------------------------------------------------------------------

LCD_RESET:      MOVLW   B'00111000'     ;       LCD reset command.
                MOVWF   PORTB           ;
               _CALL    PULSE_LCD_E     ; (1)   Pulse the LCD E lead low.
               _GOTO    WAIT_8MS        ; (1)   Return via "GOTO".

;*****************************************************************************************

INIT_REGS:      MOVLW   PORTA_INIT      ;       Set Initial port A levels.
                MOVWF   PORTA           ;
                CLRF    PORTB           ;       Set Initial port B levels.
                BSF     RP0             ;       Select bank 1.
                                        ;
                MOVLW   PORTA_DIR       ;       Set the initial port A directions.
                MOVWF   TRISA           ;       (RA0-4 I/P)
                                        ;
                MOVLW   PORTB_DIR       ;       Set the initial port B directions.
                MOVWF   TRISB           ;
                                        ;
                MOVLW   B'00000001'     ;       Set prescaler to TMR0, Enable port B
                MOVWF   OPTION_REG      ;       pull-ups and interrupt rate to 1.024 mS.
                                        ;
                MOVLW   H'80'+VOLTS     ;
                MOVWF   VRCON           ;       Set voltage ref for comparators.
                                        ;
                BCF     RP0             ;       Return to bank 0.
                                        ;
                BSF     DDS_CLOCK       ;       Initialise the DDS chip.
                BCF     DDS_CLOCK       ;       Pulse the DDS W_CLK once.
                BSF     DDS_LOAD        ;       Pulse the DDS FQ_UD once.
                BCF     DDS_LOAD        ;
                                        ;
                MOVLW   PORTB_INIT      ;       Reset port B levels.
                MOVWF   PORTB           ;       ( LCD data, RS & E pins high )
                                        ;       ( DDS _CLOCK high, DDS_LOAD low )
                                        ;
                MOVLW   B'00001010'     ;       Set RA0-3 to mode 2, 4 I/P mux to 2
                MOVWF   CMCON           ;       comparators, with noninverted O/P.
                                        ;
                MOVLW   79              ;
                MOVWF   COUNT_1         ;
                MOVLW   H'21'           ;       Clear 79 bytes of ram starting at 21H.
               _CALL    CLEAR_RAM       ; (1)
                                        ;
                MOVLW   80              ;
                MOVWF   COUNT_1         ;
                MOVLW   H'A0'           ;       Clear 80 bytes of ram starting at A0H.
               _CALL    CLEAR_RAM       ; (1)
                                        ;
                BSF     IRP             ;       Select indirect bank 2 & 3.
                                        ;
                MOVLW   80              ;
                MOVWF   COUNT_1         ;
                MOVLW   low H'120'      ;       Clear 80 bytes of ram starting at 120H.
               _CALL    CLEAR_RAM       ; (1)
                                        ;
                BCF     IRP             ;       Return to indirect bank 0 & 1.
                                        ;
                INCF    METER_COUNT,F   ;       Initialise the POWER METER update timer.
                                        ;
                MOVLW   STACK           ;       Initialise the ARG stack.
                MOVWF   STACK_PTR       ;
                                        ;
                                        ;
                MOVLW   B'10100000'     ;       Enable TMR0 overflow interrupt.
                MOVWF   INTCON          ;
                                        ;
                RETURN                  ;

;-----------------------------------------------------------------------------------------

CLEAR_RAM:      MOVWF   FSR             ;
CLEAR_RAM_LOOP: CLRF    INDF            ;       Clear the RAM byte.
                INCF    FSR,F           ;       Inc the RAM address pointer.
                DECFSZ  COUNT_1,F       ;       Dec the count.
               _GOTO    CLEAR_RAM_LOOP  ;       Loop until all bytes cleared.
                RETURN

;-----------------------------------------------------------------------------------------

 IF SIMULATE                            ;       If simulating use a dummy ISR.

;-----------------------------------------------------------------------------------------
;               Dummy ISR. Check the sweep clock.
;-----------------------------------------------------------------------------------------

INT_SERVICE:    MOVF    INT_COUNT,W     ;
                ANDLW   B'00000011'     ;
                BTFSS   ZERO            ;       Is the interrupt counter equal to 8 or 4 ?
               _GOTO    NO_SWEEP_CLOCK  ;       N. Skip the counter.
                                        ;
                DECFSZ  SWEEP_COUNT,F   ;       Y. Dec the sweep counter. Equal to 0 ?
               _GOTO    NO_SWEEP_CLOCK  ;          N. Skip the update.
                                        ;
                BSF     SWEEP_CLOCK     ;          Y. Set flag for clocking the sweep function.
                MOVLW   1               ;
                BTFSS   FAST_WOBBLE      ;             Is the fast sweep is active ?
                MOVF    SWEEP_RATE,W    ;             Y. Get the sweep rate count.
                MOVWF   SWEEP_COUNT     ;             Reset the sweep counter.
                                        ;
NO_SWEEP_CLOCK: DECFSZ  INT_COUNT,F     ;       Dec the interrupt counter. Equal to 0 ?
               _GOTO    EXIT_INTERRUPT  ;       N. Exit the interrupt service routine.
                BSF     INT_COUNT,3     ;       Y. Set the count back to 8. (8.192 mS)
                                        ;          Debounce buttons.
                                        ;
EXIT_INTERRUPT: CLRF    STATUS          ;       Return to bank 0.
                BCF     T0IF            ;       Clear TMR0 interrupt flag.
                MOVF    SAVE_FSR,W      ;
                MOVWF   FSR             ;       Restore FSR reg.
                MOVF    SAVE_PCLATH,W   ;
                MOVWF   PCLATH          ;       Restore PCLATH reg.
                MOVF    SAVE_PORTB,W    ;
                MOVWF   PORTB           ;       Restore PORT B reg.
                SWAPF   SAVE_STATUS,W   ;
                MOVWF   STATUS          ;       Restore STATUS reg.
                SWAPF   SAVE_W_REG,F    ;
                SWAPF   SAVE_W_REG,W    ;       Restore W reg without changing STATUS.
                RETFIE                  ;
 ELSE                                   ;

;*****************************************************************************************
;
;               Interrupt service routine
;
;               Service the TMR0 interrupt (EVERY 1.024 mS)
;
;               All reads of RA0-3, (encoder, keypad, press buttons) use the comparator
;               module in mode 2, with a Vref of 3.5V.
;               (The 2 diodes in series with the keypad means the digital I/P's dont work)
;
;               EVERY 1.024 mS.
;               Debounce the rotary encoder by sampling the encoder state every mS.
;               If 4 consecutive checks of the pins are at the same state, that state
;               is considered to be valid. (debounced)
;
;               Determine the direction of rotation by looking at the debounced
;               Encoder B pin on the rising edge of the debounced encoder A pin.
;               Encoder B pin low = Clockwise, high = Counter-Clockwise.
;               On each determination of the direction, set the ENC_COUNTER variable
;               to the current state of the STEP_RATE variable.
;               (+ STEP_RATE for CW, - STEP_RATE for CCW)
;               Then set the STEP_RATE variable back to 38.
;               (The variable would have decremented to at least 30 before)
;               (the next rising edge of the debounced encoder A pin.     )
;
;               Decremented the STEP_RATE variable toward 1 every mS. Therefore the step
;               rate increases from 1, for slow rotation, up to 30 for faster rotation,
;               If any switches are pressed, or if the rotary encoder has been moved,
;               reset the display timeout down counter.
;               If the rotary encoder has been moved, reset the EEPROM write counter.
;
;               EVERY 8.192 mS.
;               Sweep the switch array, if a switch is pressed increment the corresponding
;               switch pressed counter, else set the switch counter back to 0.
;               Sweep the keypad, if any button is pressed, assign the correct value to it,
;               increment the keypad pressed counter, else set the keypad counter to 0.
;               Increment the 8 bit EEPROM WRITE counter. When it wraps back to 0, set
;               the EEPROM write flag. (Indicates when to update the EEPROM, ie 2 seconds
;               after stopping the rotary encoder write any changes to EEPROM)
;               Decrement the 16 bit display counter. When it reaches 0, set the
;               TIMEOUT flag.
;               (If the rotary encoder or buttons have not been changed for about 10)
;               (seconds, the flag indicates that its time to return from the setup )
;               (screens, back to the main display.                                 )
;
;*****************************************************************************************


;-----------------------------------------------------------------------------------------
;
;               The following is processed every 1.024 mS.
;
;-----------------------------------------------------------------------------------------

;-----------------------------------------------------------------------------------------
;
;               Keep track of the last 4 samples of the rotary encoder. (debouncing)
;
;-----------------------------------------------------------------------------------------

INT_SERVICE:    MOVLW   B'11101010'     ;
                BTFSC   SAVE_PORTB,2    ;
                MOVLW   B'11101110'     ;       Leave I2C_SCL as it was.
                                        ;
                MOVWF   PORTB           ;       Enable switch column 2. (rotary encoder)
                                        ;
                BCF     CIS             ;       Enable I/P from row 3,4.
                                        ;
                BCF     CARRY           ;
                RRF     ENC_HISTORY,F   ;       Make room in the shift register for
                BCF     CARRY           ;       the two new encoder sample bits.
                RRF     ENC_HISTORY,F   ;
                                        ;
                MOVF    CMCON,W         ;       Get the current encoder state.
                ANDLW   B'11000000'     ;
                IORWF   ENC_HISTORY,F   ;       Put the new bits into the shift reg.
                                        ;
                BSF     CIS             ;       Enable I/P from row 1,2. (SW5,SW6)
                                        ;
                MOVLW   B'01010101'     ;
                ANDWF   ENC_HISTORY,W   ;       Mask the last 4 encoder A samples.
                BTFSC   ZERO            ;       Were all 4 samples low ?
                BCF     ENC_A_DEBOUNCED ;       Y. Enc A is truly low !
                                        ;
                XORLW   B'01010101'     ;       Mask the last 4 encoder A samples.
                BTFSC   ZERO            ;       Were all 4 samples high ?
                BSF     ENC_A_DEBOUNCED ;       Y. Enc A is truly high !
                                        ;
                MOVLW   B'10101010'     ;
                ANDWF   ENC_HISTORY,W   ;       Mask the last 4 encoder B samples.
                BTFSC   ZERO            ;       Were all 4 samples low ?
                BCF     ENC_B_DEBOUNCED ;       Y. Enc B is truly low !
                                        ;
                XORLW   B'10101010'     ;       Mask the last 4 encoder B samples.
                BTFSC   ZERO            ;       Were all 4 samples high ?
                BSF     ENC_B_DEBOUNCED ;       Y. Enc B is truly high !

;-----------------------------------------------------------------------------------------
;
;               Determine if the rotary encoder has been moved.
;
;-----------------------------------------------------------------------------------------

                BTFSC   OLD_A_DEBOUNCED ;       Find direction using ENC A +edge.
               _GOTO    DECODE_DONE     ;       Was the old ENC A low, and the
                BTFSS   ENC_A_DEBOUNCED ;       current ENC A high ?
               _GOTO    DECODE_DONE     ;       N. Done.
                                        ;
                MOVF    STEP_RATE,W     ;       Copy STEP_RATE into ENC_COUNTER.
                MOVWF   ENC_COUNTER     ;
                                        ;
                BTFSS   ENC_B_DEBOUNCED ;       Is direction CCW ?
                BSF     ENC_COUNTER,7   ;       Y. Set bit 7.
                                        ;
                MOVLW   38              ;       Set the step rate to 38. (dec every mS)
                MOVWF   STEP_RATE       ;       (Max rate = 38 - (2 x 4mS debounce)=30)
                                        ;
                CLRF    EEPROM_WRITE    ;       Reset the EEPROM write counter.
                MOVLW   TIMEOUT_NUM     ;       Reset the display timeout counter.
                MOVWF   TIMEOUT_H       ;
                CLRF    TIMEOUT_L       ;       Load timeout with 10 sec.

;-----------------------------------------------------------------------------------------

DECODE_DONE:    BCF     OLD_A_DEBOUNCED ;
                BTFSC   ENC_A_DEBOUNCED ;       Save the current debounced encoder A
                BSF     OLD_A_DEBOUNCED ;       bit for next time.
                                        ;
                DECFSZ  STEP_RATE,W     ;       Dec the step rate counter,
                MOVWF   STEP_RATE       ;       but do not dec below 1.

;-----------------------------------------------------------------------------------------
;               Check the sweep clock.
;-----------------------------------------------------------------------------------------

                MOVF    INT_COUNT,W     ;
                ANDLW   B'00000011'     ;
                BTFSS   ZERO            ;       Is the interrupt counter equal to 8 or 4 ?
               _GOTO    NO_SWEEP_CLOCK  ;       N. Skip the counter.

                DECFSZ  SWEEP_COUNT,F   ;       Y. Dec the sweep counter. Equal to 0 ?
               _GOTO    NO_SWEEP_CLOCK  ;          N. Skip the update.
                                        ;
                BSF     SWEEP_CLOCK     ;          Y. Set flag for clocking the sweep function.
                MOVLW   1               ;
                BTFSS   FAST_WOBBLE      ;             Is the fast sweep is active ?
                MOVF    SWEEP_RATE,W    ;             Y. Get the sweep rate count.
                MOVWF   SWEEP_COUNT     ;             Reset the sweep counter.

NO_SWEEP_CLOCK:

;                BTFSC   SWEEP_CLOCK     ;       For checking sweep timing.
;                BSF     PORTA,7         ;

;-----------------------------------------------------------------------------------------

                DECFSZ  INT_COUNT,F     ;       Dec the interrupt counter. Equal to 0 ?
               _GOTO    EXIT_INTERRUPT  ;       N. Exit the interrupt service routine.
                BSF     INT_COUNT,3     ;       Y. Set the count back to 8. (8.192 mS)
                                        ;          Debounce buttons.

;-----------------------------------------------------------------------------------------
;
;               The following is processed every 8.192 mS.
;
;-----------------------------------------------------------------------------------------

;-----------------------------------------------------------------------------------------
;               Sweep the keypad and 6 buttons.
;-----------------------------------------------------------------------------------------

                SWAPF   CMCON,W         ;       read column 2, row 1,2.       (SW5,SW6)
                BSF     SW_COL_2        ;       Disable switch column 2.
                BCF     SW_COL_3        ;       Enable switch column 3.
                ANDLW   B'00001100'     ;
                MOVWF   COL_2           ;       Save the col 2, row 1,2 data.
                                        ;
                SWAPF   CMCON,W         ;       read column 3, row 1,2.    (keypad 1,4)
                BCF     CIS             ;       Enable I/P from row 3,4.
                ANDLW   B'00001100'     ;
                MOVWF   COL_3           ;       Save the column 3, row 1,2 data.
                                        ;
                MOVF    CMCON,W         ;       read column 3, row 3,4.    (keypad 7,*)
                BSF     SW_COL_3        ;       Disable switch column 3.
                BCF     SW_COL_4        ;       Enable switch column 4.
                ANDLW   B'11000000'     ;
                IORWF   COL_3,F         ;       Save the column 3, row 3,4 data.
                                        ;
                MOVF    CMCON,W         ;       read column 4, row 3,4.    (keypad 8,0)
                BSF     CIS             ;       Enable I/P from row 1,2.
                ANDLW   B'11000000'     ;
                MOVWF   COL_4           ;       Save the column 4, row 3,4 data.
                                        ;
                SWAPF   CMCON,W         ;       read column 4, row 1,2.    (keypad 2,5)
                BSF     SW_COL_4        ;       Disable switch column 4.
                BCF     SW_COL_5        ;       Enable switch column 5.
                ANDLW   B'00001100'     ;
                IORWF   COL_4,F         ;       Save the column 4, row 1,2 data.
                                        ;
                SWAPF   CMCON,W         ;       read column 5, row 1,2.    (keypad 3,6)
                BCF     CIS             ;       Enable I/P from row 3,4.
                ANDLW   B'00001100'     ;
                MOVWF   COL_5           ;       Save the column 5, row 1,2 data.
                                        ;
                MOVF    CMCON,W         ;       read column 5, row 3,4.    (keypad 9,#)
                BSF     SW_COL_5        ;       Disable switch column 5.
                BCF     SW_COL_1        ;       Enable switch column 1.
                ANDLW   B'11000000'     ;
                IORWF   COL_5,F         ;       Save the column 5, row 3,4 data.
                                        ;
                MOVF    CMCON,W         ;       read column 1, row 3,4.    (SW3,SW4 or)
                BSF     CIS             ;       Enable I/P from row 1,2.   (keypad C,D)
                ANDLW   B'11000000'     ;
                MOVWF   COL_1           ;       Save the column 1, row 3,4 data.
                                        ;
                SWAPF   CMCON,W         ;       read column 1, row 1,2.    (SW1,SW2 or)
                ANDLW   B'00001100'     ;                                  (keypad A,B)
                IORWF   COL_1,F         ;       Save the column 1, row 1,2 data.

;-----------------------------------------------------------------------------------------
;               Debounce any keypad button, and assign the correct value to it.
;-----------------------------------------------------------------------------------------

                BCF     ANY_BUTTON      ;
                INCFSZ  KEYPAD_CNT,W    ;       Inc the keypad count.
                MOVWF   KEYPAD_CNT      ;       But dont inc past 255.
                                        ;
                CLRW                    ;       W will hold the key value.
                BTFSC   KP_0            ;       Is the 0 button pressed?
                MOVLW   H'80' + 0       ;       Y. Set the MSB & assign the value 0 to W.
                BTFSC   KP_1            ;       Is the 1 button pressed?
                MOVLW   H'80' + 1       ;       Y. Set the MSB & assign the value 1 to W.
                BTFSC   KP_2            ;
                MOVLW   H'80' + 2       ;           "
                BTFSC   KP_3            ;
                MOVLW   H'80' + 3       ;           "
                BTFSC   KP_4            ;
                MOVLW   H'80' + 4       ;           "
                BTFSC   KP_5            ;
                MOVLW   H'80' + 5       ;           "
                BTFSC   KP_6            ;
                MOVLW   H'80' + 6       ;           "
                BTFSC   KP_7            ;
                MOVLW   H'80' + 7       ;           "
                BTFSC   KP_8            ;
                MOVLW   H'80' + 8       ;           "
                BTFSC   KP_9            ;
                MOVLW   H'80' + 9       ;           "
                BTFSC   KP_.            ;
                MOVLW   B'10100000' +10 ;           "
                BTFSC   KP_#            ;
                MOVLW   B'11000000' +11 ;           "
                                        ;
                ADDLW   0               ;
                BTFSS   ZERO            ;       Are any keypad button pressed ?
                MOVWF   KEYPAD_NUM      ;       Y. Update the keypad number.
                BTFSC   ZERO            ;
                CLRF    KEYPAD_CNT      ;       N.  Clear the keypad count.
                IORWF   COL_1,W         ;       Or in the press buttons.
                IORWF   COL_2,W         ;
                BTFSC   ZERO            ;       Any keypad or press buttons pressed ?
               _GOTO    NO_BUTTON       ;
                MOVLW   TIMEOUT_NUM     ;       Y. Reset the display timeout counter.
                MOVWF   TIMEOUT_H       ;
                CLRF    TIMEOUT_L       ;          Load timeout with 10 sec.
                BSF     ANY_BUTTON      ;          Flag that a button is pressed.
NO_BUTTON:      BCF     KEYPAD_NUM,7    ;       Clear the MSB of the keypad number.

;-----------------------------------------------------------------------------------------
;               Debounce the S/F button.
;-----------------------------------------------------------------------------------------

                INCFSZ  SW_SF_CNT,W     ;       Inc the VFO A/B button count.
                MOVWF   SW_SF_CNT       ;       But dont inc past 255.
                BTFSS   SW_SF           ;       Is S_F button pressed ?   (hi if pressed)
                CLRF    SW_SF_CNT       ;       N. Clear the count.

;-----------------------------------------------------------------------------------------
;               Debounce the MEM button.
;-----------------------------------------------------------------------------------------

                INCFSZ  SW_MEM_CNT,W    ;       Inc the MEM button count.
                MOVWF   SW_MEM_CNT      ;       But dont inc past 255.
                BTFSC   SW_MEM          ;       Is MEM button pressed ?   (hi if pressed)
               _GOTO    SW_MEM_END_1    ;
                CLRF    SW_MEM_CNT      ;       N. Clear the count.
                BCF     MEM_1_SEC       ;          Clear the long press flags.
                BCF     MEM_2_SEC       ;                       "
SW_MEM_END_1:                           ;
                MOVLW   100             ;       MEM button pressed for 1 sec ?
                SUBWF   SW_MEM_CNT,W    ;
                BTFSC   ZERO            ;
                BSF     MEM_1_SEC       ;       Y. Set the 1S long press flag.
                                        ;
                MOVLW   200             ;       MEM button pressed for 2 sec ?
                SUBWF   SW_MEM_CNT,W    ;
                BTFSS   ZERO            ;
               _GOTO    SW_MEM_END_2    ;
                BSF     MEM_2_SEC       ;       Y. Set the 2S long press flag.
                BCF     MEM_1_SEC       ;          Clear the 1S long press flag.
SW_MEM_END_2:                           ;

;-----------------------------------------------------------------------------------------
;               Debounce the CAL button.
;-----------------------------------------------------------------------------------------

                INCFSZ  SW_CAL_CNT,W    ;       Inc the CAL button count.
                MOVWF   SW_CAL_CNT      ;       But dont inc past 255.
                BTFSS   SW_CAL          ;       Is CAL button pressed ?   (hi if pressed)
                CLRF    SW_CAL_CNT      ;       N. Clear the count.

;-----------------------------------------------------------------------------------------
;               Increment the LCD flasher counter.
;-----------------------------------------------------------------------------------------

                INCF    FLASHER_COUNT,F ;       Inc the counter.

;-----------------------------------------------------------------------------------------
;               Increment the EEPROM update counter.
;-----------------------------------------------------------------------------------------

                INCFSZ  EEPROM_WRITE,W  ;       Inc the counter.
                MOVWF   EEPROM_WRITE    ;       But dont inc past 255.
                SUBLW   254             ;
                BTFSC   ZERO            ;       Is the count = 254?
                BSF     EEPROM_UPDATE   ;       Y. Set the flag. (update the EEPROM)

;-----------------------------------------------------------------------------------------
;               Decrement the Display timeout counter.
;-----------------------------------------------------------------------------------------

                DECF    TIMEOUT_L,F     ;       Dec the timeout counter.
                BTFSC   ZERO            ;
                DECF    TIMEOUT_H,F     ;       Is it = 0 ?
                BTFSC   ZERO            ;
                BSF     TIMEOUT         ;       Y. Set the timeout flag.

;-----------------------------------------------------------------------------------------
;               Check the Power meter display update counter.
;-----------------------------------------------------------------------------------------

                DECFSZ  METER_COUNT,F   ;       Dec the meter counter. Equal to 0 ?
               _GOTO    CLOCK_OK        ;
                MOVLW   METER_TIME      ;       Y. Reset the counter.
                MOVWF   METER_COUNT     ;
                BSF     METER_FLAG      ;          Set the power meter flag.
CLOCK_OK:                               ;

;-----------------------------------------------------------------------------------------
;               Interrupt exit.
;
;               Restore PORT B, STATUS, and W registers.
;
;-----------------------------------------------------------------------------------------

EXIT_INTERRUPT: CLRF    STATUS          ;       Return to bank 0.
                BCF     T0IF            ;       Clear TMR0 interrupt flag.
                MOVF    SAVE_FSR,W      ;
                MOVWF   FSR             ;       Restore FSR reg.
                MOVF    SAVE_PCLATH,W   ;
                MOVWF   PCLATH          ;       Restore PCLATH reg.
                MOVF    SAVE_PORTB,W    ;
                MOVWF   PORTB           ;       Restore PORT B reg.
                SWAPF   SAVE_STATUS,W   ;
                MOVWF   STATUS          ;       Restore STATUS reg.
                SWAPF   SAVE_W_REG,F    ;
                SWAPF   SAVE_W_REG,W    ;       Restore W reg without changing STATUS.
                RETFIE                  ;

;*****************************************************************************************
;               End of interrupt service routine
;*****************************************************************************************

 ENDIF                                  ;

;-----------------------------------------------------------------------------------------

 IF $ >= PAGE1 + H'800'
 ERROR "Page 1 to long"
 ENDIF

;*****************************************************************************************
;               End of PAGE 1 routines.
;*****************************************************************************************

;*****************************************************************************************
;                       EEPROM VARIABLE MACROs
;*****************************************************************************************

VAR_1           MACRO   NAME, VAL       ;       1 Byte variable definition.
NAME            EQU $ - EE_VAR_START    ;
                DE (VAL) & H'FF'        ;
                ENDM                    ;

VAR_4           MACRO   NAME, VAL       ;       4 Byte variable definition.
NAME            EQU $ - EE_VAR_START    ;
                DE (VAL>>24) & H'FF'    ;
                DE (VAL>>16) & H'FF'    ;
                DE (VAL>>8) & H'FF'     ;
                DE (VAL) & H'FF'        ;
                ENDM                    ;

;*****************************************************************************************
;                       EEPROM variable definitions.
;*****************************************************************************************

 ORG     H'2100'

EE_VAR_START

; size          name                          value       comment

 IF AD9851_CHIP                                         ; If assembling for the AD9851.

 VAR_4          EE_MAX_DDS_FREQ:        ,  70000000     ;
 VAR_4          EE_DDS_CAL:             , 400319966     ; 2^56 / 180.00 MHz
 VAR_1          EE_DDS_OPT:             ,         1     ; AD9851 x6.
 VAR_1          EE_STEP_NUM:            ,         0     ; 1 Hz
 VAR_4          EE_S_DDS_FREQ:          ,  10500000     ;
 VAR_4          EE_F_DDS_FREQ:          ,  10501500     ;
 VAR_1          EE_SWEEP_NUM:           ,         4     ; Medium sweep rate and step size.

 ELSE                                                   ; If assembling for the AD9850.

 VAR_4          EE_MAX_DDS_FREQ:        ,  30000000     ;
 VAR_4          EE_DDS_CAL:             , 720575940     ; 2^56 / 100.00 MHz
 VAR_1          EE_DDS_OPT:             ,         0     ; AD9850.
 VAR_1          EE_STEP_NUM:            ,         0     ; 1 Hz
 VAR_4          EE_S_DDS_FREQ:          ,  10500000     ;
 VAR_4          EE_F_DDS_FREQ:          ,  10501500     ;
 VAR_1          EE_SWEEP_NUM:           ,         4     ; Medium sweep rate and step size.

 ENDIF

 VAR_1          EE_SPARE:               ,       255     ;


EE_MEM_START    EQU $ - EE_VAR_START

 VAR_4          EE_S_DDS_FREQ1:         ,  10550000     ; Memory 1	10.7MHz @ 300KHz BW
 VAR_4          EE_F_DDS_FREQ1:         ,  10850000     ;
 VAR_1          EE_SWEEP_NUM_1:         ,         9     ;

 VAR_4          EE_S_DDS_FREQ2:         ,  10685000     ; Memory 2	10.7MHz @ 30KHz BW
 VAR_4          EE_F_DDS_FREQ2:         ,  10715000     ;
 VAR_1          EE_SWEEP_NUM_2:         ,         7     ;

 VAR_4          EE_S_DDS_FREQ3:         ,  21392500     ; Memory 3  21.4MHz @ 15KHz BW
 VAR_4          EE_F_DDS_FREQ3:         ,  21407500     ;
 VAR_1          EE_SWEEP_NUM_3:         ,         6     ;

 VAR_4          EE_S_DDS_FREQ4:         ,  32650000     ; Memory 4	36.15MHz @ 7MHz BW
 VAR_4          EE_F_DDS_FREQ4:         ,  39650000     ;
 VAR_1          EE_SWEEP_NUM_4:         ,         9     ;

 VAR_4          EE_S_DDS_FREQ5:         ,  45992500     ; Memory 5	45.0MHz @ 15KHz BW
 VAR_4          EE_F_DDS_FREQ5:         ,  45007500     ;
 VAR_1          EE_SWEEP_NUM_5:         ,         6     ;

 VAR_4          EE_S_DDS_FREQ6:         ,  10250000     ; Memory 6
 VAR_4          EE_F_DDS_FREQ6:         ,  11250000     ;
 VAR_1          EE_SWEEP_NUM_6:         ,         6     ;

 VAR_4          EE_S_DDS_FREQ7:         ,  10300000     ; Memory 7
 VAR_4          EE_F_DDS_FREQ7:         ,  11300000     ;
 VAR_1          EE_SWEEP_NUM_7:         ,         6     ;

 VAR_4          EE_S_DDS_FREQ8:         ,  10350000     ; Memory 8
 VAR_4          EE_F_DDS_FREQ8:         ,  11350000     ;
 VAR_1          EE_SWEEP_NUM_8:         ,         6     ;

 VAR_4          EE_S_DDS_FREQ9:         ,  10400000     ; Memory 9
 VAR_4          EE_F_DDS_FREQ9:         ,  11400000     ;
 VAR_1          EE_SWEEP_NUM_9:         ,         6     ;

 VAR_4          EE_S_DDS_FREQ10:        ,  10450000     ; Memory 10
 VAR_4          EE_F_DDS_FREQ10:        ,  11450000     ;
 VAR_1          EE_SWEEP_NUM_10:        ,         6     ;

 VAR_4          EE_S_DDS_FREQ11:        ,  10500000     ; Memory 11
 VAR_4          EE_F_DDS_FREQ11:        ,  11500000     ;
 VAR_1          EE_SWEEP_NUM_11:        ,         6     ;

;*****************************************************************************************

                END
